@@ -53,6 +53,7 @@ gda_dir_blob_op_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaDirBlobOpClass),
 			(GBaseInitFunc) NULL,
@@ -64,7 +65,10 @@ gda_dir_blob_op_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_dir_blob_op_init
 		};
-		type = g_type_register_static (GDA_TYPE_BLOB_OP, "GdaDirBlobOp", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (GDA_TYPE_BLOB_OP, "GdaDirBlobOp", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
