@@ -28,11 +28,11 @@
 
 #include "gda-decl.h"
 #include <libgda/gda-data-model.h>
-#include <libgda/gda-data-model-index.h>
 #include <libgda/gda-connection-event.h>
 #include <libgda/gda-transaction-status.h>
 #include <libgda/gda-statement.h>
 #include <libgda/gda-meta-store.h>
+#include <libgda/gda-server-operation.h>
 
 G_BEGIN_DECLS
 
@@ -98,7 +98,9 @@ typedef enum {
 	GDA_CONNECTION_FEATURE_UPDATABLE_CURSOR,
 	GDA_CONNECTION_FEATURE_USERS,
 	GDA_CONNECTION_FEATURE_VIEWS,
-	GDA_CONNECTION_FEATURE_XML_QUERIES
+	GDA_CONNECTION_FEATURE_XA_TRANSACTIONS,
+
+	GDA_CONNECTION_FEATURE_LAST
 } GdaConnectionFeature;
 
 typedef enum {
@@ -142,22 +144,21 @@ gboolean             gda_connection_is_opened            (GdaConnection *cnc);
 
 GdaConnectionOptions gda_connection_get_options          (GdaConnection *cnc);
 
-GdaServerProvider   *gda_connection_get_provider_obj     (GdaConnection *cnc);
+GdaServerProvider   *gda_connection_get_provider         (GdaConnection *cnc);
 const gchar         *gda_connection_get_provider_name    (GdaConnection *cnc);
 
+GdaServerOperation  *gda_connection_create_operation     (GdaConnection *cnc, GdaServerOperationType type,
+                                                          GdaSet *options, GError **error);
+gboolean             gda_connection_perform_operation    (GdaConnection *cnc, GdaServerOperation *op, GError **error);
+                                                          
 const gchar         *gda_connection_get_dsn              (GdaConnection *cnc);
-gboolean             gda_connection_set_dsn              (GdaConnection *cnc, const gchar *datasource);
 const gchar         *gda_connection_get_cnc_string       (GdaConnection *cnc);
-const gchar         *gda_connection_get_authentification (GdaConnection *cnc);
+const gchar         *gda_connection_get_authentication   (GdaConnection *cnc);
 
-void                 gda_connection_add_event            (GdaConnection *cnc, GdaConnectionEvent *event);
-GdaConnectionEvent  *gda_connection_add_event_string     (GdaConnection *cnc, const gchar *str, ...);
-void                 gda_connection_add_events_list      (GdaConnection *cnc, GList *events_list);
-void                 gda_connection_clear_events_list    (GdaConnection *cnc);
 const GList         *gda_connection_get_events           (GdaConnection *cnc);
 
 GdaSqlParser        *gda_connection_create_parser        (GdaConnection *cnc);
-GObject             *gda_connection_batch_execute        (GdaConnection *cnc,
+GSList              *gda_connection_batch_execute        (GdaConnection *cnc,
 							  GdaBatch *batch, GdaSet *params,
 							  GdaStatementModelUsage model_usage, GError **error);
 
@@ -198,7 +199,6 @@ GdaMetaStore        *gda_connection_get_meta_store       (GdaConnection *cnc);
 gboolean             gda_connection_update_meta_store    (GdaConnection *cnc, GdaMetaContext *context, GError **error);
 GdaDataModel        *gda_connection_get_meta_store_data  (GdaConnection *cnc, GdaConnectionMetaType meta_type,
 							  GError **error, gint nb_filters, ...);
-
 G_END_DECLS
 
 #endif
