@@ -128,7 +128,7 @@ static gint                 gda_data_model_dir_get_n_rows      (GdaDataModel *mo
 static gint                 gda_data_model_dir_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_dir_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_model_dir_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_model_dir_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_model_dir_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_model_dir_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
 static gboolean             gda_data_model_dir_set_value_at (GdaDataModel *model, gint col, gint row, const GValue *value, GError **error);
@@ -161,6 +161,7 @@ gda_data_model_dir_data_model_init (GdaDataModelClass *iface)
         iface->i_iter_prev = NULL;
 
         iface->i_set_value_at = gda_data_model_dir_set_value_at;
+	iface->i_iter_set_value = NULL;
         iface->i_set_values = gda_data_model_dir_set_values;
         iface->i_append_values = gda_data_model_dir_append_values;
         iface->i_append_row = NULL;
@@ -553,7 +554,7 @@ update_file_md5sum (FileRow *row, const gchar *complete_filename)
  md5end:
 	if (value) {
 		if (row->md5sum_value && (G_VALUE_TYPE (row->md5sum_value) == G_TYPE_STRING)
-		    && !gda_value_compare_ext (row->md5sum_value, value))
+		    && !gda_value_compare (row->md5sum_value, value))
 			changed = FALSE;
 		else {
 			if (row->md5sum_value)
@@ -613,7 +614,7 @@ update_file_mime (FileRow *row, const gchar *complete_filename)
 
 	if (value) {
 		if (row->mime_value && (G_VALUE_TYPE (row->mime_value) == G_TYPE_STRING)
-		    && !gda_value_compare_ext (row->mime_value, value))
+		    && !gda_value_compare (row->mime_value, value))
 			changed = FALSE;
 		else {
 			if (row->mime_value)
@@ -776,7 +777,7 @@ gda_data_model_dir_get_access_flags (GdaDataModel *model)
 }
 
 static const GValue *
-gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataModelDir *imodel;
 	GValue *value = NULL;
@@ -787,7 +788,12 @@ gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
 	g_return_val_if_fail (imodel->priv, NULL);
 
 	if ((col < 0) || (col > COL_LAST)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
+		add_error (imodel, tmp);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			     tmp);
+		g_free (tmp);
 		return NULL;
 	}
 
@@ -796,6 +802,8 @@ gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
                 str = g_strdup_printf (_("Row %d out of range (0-%d)"), row,
 				       imodel->priv->rows->len - 1);
 		add_error (imodel, str);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
+			     str);
 		g_free (str);
                 return NULL;
         }
@@ -859,6 +867,9 @@ gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row)
 			break;
 		}
 	}
+	else
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+			     _("Row not found"));
 
 	return value;
 }
@@ -873,7 +884,10 @@ gda_data_model_dir_get_attributes_at (GdaDataModel *model, gint col, gint row)
 	g_return_val_if_fail (imodel->priv, 0);
 
 	if ((col < 0) || (col > COL_LAST)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
+		add_error (imodel, tmp);
+		g_free (tmp);
 		return 0;
 	}
 
@@ -912,9 +926,11 @@ gda_data_model_dir_set_value_at (GdaDataModel *model, gint col, gint row, const
 	g_return_val_if_fail (imodel->priv, FALSE);
 
 	if ((col < 0) || (col > COL_LAST)) {
-		add_error (imodel, _("Column number out of range"));
-		g_set_error (error, 0, 0,
-			     _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, COL_LAST-1);
+		add_error (imodel, tmp);
+		g_set_error (error, 0, 0, tmp);
+		g_free (tmp);
 		return FALSE;
 	}
 
@@ -964,7 +980,10 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 
 	for (col = 0, list = values; list; list = list->next, col++) {
 		GValue *value = (GValue *) list->data;
-		if (!value || !gda_value_compare_ext (value, gda_data_model_get_value_at (model, col, row))) 
+		const GValue *cvalue = gda_data_model_get_value_at (model, col, row, error);
+		if (!cvalue)
+			return FALSE;
+		if (!value || !gda_value_compare (value, cvalue)) 
 			continue;
 
 		switch (col) {
