@@ -36,6 +36,8 @@
 #include <locale.h>
 #endif
 
+extern gchar *gda_lang_locale;
+
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-decl.h>
 #include <libgda/gda-enums.h>
@@ -44,6 +46,7 @@
 #include <libgda/gda-data-model-extra.h>
 #include <libgda/gda-data-access-wrapper.h>
 #include <libgda/gda-data-model-iter.h>
+#include <libgda/gda-data-model-iter-extra.h>
 #include <libgda/gda-holder.h>
 #include <libgda/gda-set.h>
 #include <libgda/gda-data-model-private.h> /* For gda_data_model_add_data_from_xml_node() */
@@ -158,7 +161,7 @@ static gint                 gda_data_model_import_get_n_rows      (GdaDataModel
 static gint                 gda_data_model_import_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_import_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_model_import_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row);
 static GdaDataModelIter    *gda_data_model_import_create_iter      (GdaDataModel *model);
 static gboolean             gda_data_model_import_iter_next       (GdaDataModel *model, GdaDataModelIter *iter); 
@@ -181,6 +184,7 @@ gda_data_model_import_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelImportClass),
 			(GBaseInitFunc) NULL,
@@ -199,8 +203,12 @@ gda_data_model_import_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelImport", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaDataModelImport", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -216,7 +224,7 @@ gda_data_model_import_class_init (GdaDataModelImportClass *klass)
 	object_class->set_property = gda_data_model_import_set_property;
         object_class->get_property = gda_data_model_import_get_property;
 	g_object_class_install_property (object_class, PROP_RANDOM_ACCESS,
-                                         g_param_spec_boolean ("random_access", "Features random access", NULL,
+                                         g_param_spec_boolean ("random-access", "Features random access", NULL,
 							       FALSE,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE |
 							       G_PARAM_CONSTRUCT_ONLY));
@@ -225,11 +233,11 @@ gda_data_model_import_class_init (GdaDataModelImportClass *klass)
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_DATA_STRING,
-                                         g_param_spec_string ("data_string", "String to import", NULL, NULL,
+                                         g_param_spec_string ("data-string", "String to import", NULL, NULL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_XML_NODE,
-                                         g_param_spec_pointer ("xml_node", "XML node to import from", NULL,
+                                         g_param_spec_pointer ("xml-node", "XML node to import from", NULL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OPTIONS,
@@ -259,6 +267,7 @@ gda_data_model_import_data_model_init (GdaDataModelClass *iface)
 	iface->i_iter_prev = gda_data_model_import_iter_prev;
 
 	iface->i_set_value_at = NULL;
+	iface->i_iter_set_value = NULL;
 	iface->i_set_values = NULL;
         iface->i_append_values = NULL;
 	iface->i_append_row = NULL;
@@ -689,7 +698,7 @@ gda_data_model_import_new_file   (const gchar *filename, gboolean random_access,
 	g_return_val_if_fail (filename, NULL);
 	
 	model = g_object_new (GDA_TYPE_DATA_MODEL_IMPORT,
-			      "random_access", random_access,
+			      "random-access", random_access,
 			      "options", options,
 			      "filename", filename, NULL);
 
@@ -715,9 +724,9 @@ gda_data_model_import_new_mem (const gchar *data, gboolean random_access, GdaSet
 	GdaDataModelImport *model;
 
 	model = g_object_new (GDA_TYPE_DATA_MODEL_IMPORT,
-			      "random_access", random_access,
+			      "random-access", random_access,
 			      "options", options,
-			      "data_string", data, NULL);
+			      "data-string", data, NULL);
 
 	return GDA_DATA_MODEL (model);
 }
@@ -737,7 +746,7 @@ gda_data_model_import_new_xml_node (xmlNodePtr node)
 	GdaDataModelImport *model;
 
 	model = g_object_new (GDA_TYPE_DATA_MODEL_IMPORT,
-			      "xml_node", node, NULL);
+			      "xml-node", node, NULL);
 
 	return GDA_DATA_MODEL (model);
 }
@@ -812,7 +821,6 @@ init_csv_import (GdaDataModelImport *model)
 			str = g_strdup_printf ("column_%d", col);
 		gda_column_set_name (column, str);
 		gda_column_set_title (column, str);
-		gda_column_set_caption (column, str);
 		g_free (str);			
 		
 		gda_column_set_g_type (column, G_TYPE_STRING);
@@ -1121,6 +1129,7 @@ init_xml_import (GdaDataModelImport *model)
 			g_free (str);
 			xmlFreeTextReader (reader);
 			model->priv->extract.xml.reader = NULL;
+			return;
 		}
 		node = xmlTextReaderCurrentNode (reader);
 		
@@ -1225,16 +1234,9 @@ init_xml_import (GdaDataModelImport *model)
 			g_object_set (G_OBJECT (column), "id", spec->id, NULL);
 			gda_column_set_title (column, (gchar*)spec->title);
 			gda_column_set_name (column, (gchar*)spec->name);
-			gda_column_set_defined_size (column, spec->size);
-			gda_column_set_caption (column, (gchar*)spec->caption);
 			gda_column_set_dbms_type (column, (gchar*)spec->dbms_type);
-			gda_column_set_scale (column, spec->scale);
 			gda_column_set_g_type (column, spec->gdatype);
 			gda_column_set_allow_null (column, spec->nullok);
-			gda_column_set_primary_key (column, spec->pkey);
-			gda_column_set_unique_key (column, spec->unique);
-			gda_column_set_table (column, (gchar*)spec->table);
-			gda_column_set_references (column, (gchar*)spec->ref);
 			
 			list = g_slist_next (list);
 			pos++;
@@ -1289,18 +1291,12 @@ xml_fetch_next_row (GdaDataModelImport *model)
 	const xmlChar *name;
 	gint ret;
 
-	const gchar *lang = NULL;
+	const gchar *lang = gda_lang_locale;
 
 	GSList *columns = model->priv->columns;
 	GdaColumn *last_column = NULL;
 	GSList *values = NULL;
 
-#ifdef HAVE_LC_MESSAGES
-	lang = setlocale (LC_MESSAGES, NULL);
-#else
-	lang = setlocale (LC_CTYPE, NULL);
-#endif
-
 	if (model->priv->cursor_values) {
 		g_slist_foreach (model->priv->cursor_values, (GFunc) gda_value_free, NULL);
 		g_slist_free (model->priv->cursor_values);
@@ -1569,16 +1565,9 @@ init_node_import (GdaDataModelImport *model)
 		g_object_set (G_OBJECT (column), "id", spec->id, NULL);
 		gda_column_set_title (column, (gchar*)spec->title);
 		gda_column_set_name (column, (gchar*)spec->name);
-		gda_column_set_defined_size (column, spec->size);
-		gda_column_set_caption (column, (gchar*)spec->caption);
 		gda_column_set_dbms_type (column, (gchar*)spec->dbms_type);
-		gda_column_set_scale (column, spec->scale);
 		gda_column_set_g_type (column, spec->gdatype);
 		gda_column_set_allow_null (column, spec->nullok);
-		gda_column_set_primary_key (column, spec->pkey);
-		gda_column_set_unique_key (column, spec->unique);
-		gda_column_set_table (column, (gchar*)spec->table);
-		gda_column_set_references (column, (gchar*)spec->ref);
 
 		model->priv->columns = g_slist_prepend (model->priv->columns, gda_column_copy (column));
 
@@ -1710,7 +1699,7 @@ gda_data_model_import_get_access_flags (GdaDataModel *model)
 }
 
 static const GValue *
-gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataModelImport *imodel;
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_IMPORT (model), NULL);
@@ -1719,10 +1708,13 @@ gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row)
 
 	if (imodel->priv->random_access_model)
 		/* if there is a random access model, then use it */
-		return gda_data_model_get_value_at (imodel->priv->random_access_model, col, row);
-	else
+		return gda_data_model_get_value_at (imodel->priv->random_access_model, col, row, error);
+	else {
 		/* otherwise, bail out */
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     _("Data model does not support random access"));
 		return NULL;
+	}
 }
 
 static GdaValueAttribute
@@ -1765,7 +1757,7 @@ gda_data_model_import_create_iter (GdaDataModel *model)
 	}
 	else
 		iter = (GdaDataModelIter *) g_object_new (GDA_TYPE_DATA_MODEL_ITER, 
-							  "data_model", model, NULL);
+							  "data-model", model, NULL);
 	return iter;
 }
 
@@ -1820,7 +1812,7 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 
 	/* if there is a random access model, then use it */
 	if (imodel->priv->format == FORMAT_XML_NODE)
-		return gda_data_model_move_iter_next_default (model, iter);
+		return gda_data_model_iter_move_next_default (model, iter);
 
 	/* fetch the next row if necessary */
 	switch (imodel->priv->format) {
@@ -1857,24 +1849,30 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 		GSList *plist;
 		GSList *vlist;
 		gboolean update_model;
+		gboolean allok = TRUE;
 		
-		g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-		g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-		plist = ((GdaSet *) iter)->holders;
-		vlist = next_values;
-		while (plist && vlist) {
-			gda_holder_set_value (GDA_HOLDER (plist->data), 
-					      (GValue *) vlist->data);
-			plist = g_slist_next (plist);
-			vlist = g_slist_next (vlist);
+		g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+		g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+		for (plist = ((GdaSet *) iter)->holders, vlist = next_values;
+		     plist && vlist;
+		     plist = plist->next, vlist = vlist->next) {
+			GError *lerror = NULL;
+			if (! gda_holder_set_value (GDA_HOLDER (plist->data), 
+						    (GValue *) vlist->data, &lerror)) {
+				gchar *tmp;
+				tmp = g_strdup_printf (_("Could not set iterator's value: %s"),
+						       lerror && lerror->message ? lerror->message : _("No detail"));
+				add_error (imodel, tmp);
+				g_free (tmp);
+				allok = FALSE;
+			}
 		}
 		if (plist || vlist) {
+			allok = FALSE;
 			if (plist) {
 				add_error_too_few_values (imodel);
-				while (plist) {
-					gda_holder_set_value (GDA_HOLDER (plist->data), NULL);
-					plist = g_slist_next (plist);
-				}
+				for (; plist; plist = plist->next)
+					gda_holder_force_invalid (GDA_HOLDER (plist->data));
 			}
 			else
 				add_error_too_many_values (imodel);
@@ -1885,12 +1883,12 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 			imodel->priv->iter_row = 0;
 
 		g_object_set (G_OBJECT (iter), "current-row", imodel->priv->iter_row, 
-			      "update_model", update_model, NULL);
+			      "update-model", update_model, NULL);
 
-		return TRUE;
+		return allok;
 	}
 	else {
-		g_signal_emit_by_name (iter, "end_of_data");
+		g_signal_emit_by_name (iter, "end-of-data");
 		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
 		return FALSE;
 	}
@@ -1910,7 +1908,7 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 
 	/* if there is a random access model, then use it */
 	if (imodel->priv->format == FORMAT_XML_NODE)
-		return gda_data_model_move_iter_prev_default (model, iter);
+		return gda_data_model_iter_move_prev_default (model, iter);
 
 	/* fetch the previous row if necessary */
 	switch (imodel->priv->format) {
@@ -1924,24 +1922,30 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 		GSList *plist;
 		GSList *vlist;
 		gboolean update_model;
-		
-		g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-		g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-		plist = ((GdaSet *) iter)->holders;
-		vlist = imodel->priv->cursor_values;
-		while (plist && vlist) {
-			gda_holder_set_value (GDA_HOLDER (plist->data), 
-					      (GValue *) vlist->data);
-			plist = g_slist_next (plist);
-			vlist = g_slist_next (vlist);
+		gboolean allok = TRUE;
+
+		g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+		g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+		for (plist = ((GdaSet *) iter)->holders, vlist = imodel->priv->cursor_values;
+		     plist && vlist;
+		     plist = plist->next, vlist = vlist->next) {
+			GError *lerror = NULL;
+			if (! gda_holder_set_value (GDA_HOLDER (plist->data), 
+						    (GValue *) vlist->data, &lerror)) {
+				gchar *tmp;
+				tmp = g_strdup_printf (_("Could not set iterator's value: %s"),
+						       lerror && lerror->message ? lerror->message : _("No detail"));
+				add_error (imodel, tmp);
+				g_free (tmp);
+				allok = FALSE;
+			}
 		}
 		if (plist || vlist) {
+			allok = FALSE;
 			if (plist) {
 				add_error_too_few_values (imodel);
-				while (plist) {
-					gda_holder_set_value (GDA_HOLDER (plist->data), NULL);
-					plist = g_slist_next (plist);
-				}				
+				for (; plist; plist = plist->next)
+					gda_holder_force_invalid (GDA_HOLDER (plist->data));
 			}
 			else 
 				add_error_too_many_values (imodel);
@@ -1952,8 +1956,8 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 
 		g_assert (imodel->priv->iter_row >= 0);
 		g_object_set (G_OBJECT (iter), "current-row", imodel->priv->iter_row, 
-			      "update_model", update_model, NULL);
-		return TRUE;
+			      "update-model", update_model, NULL);
+		return allok;
 	}
 	else {
 		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
