@@ -21,12 +21,14 @@
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include "gda-data-model-iter.h"
+#include "gda-data-model-iter-extra.h"
 #include "gda-data-model.h"
 #include "gda-data-model-private.h"
 #include "gda-holder.h"
 #include "gda-marshal.h"
 #include "gda-data-proxy.h"
 #include "gda-enums.h"
+#include "gda-data-select.h"
 
 /* 
  * Main static functions 
@@ -49,7 +51,7 @@ static void gda_data_model_iter_get_property (GObject *object,
 static void model_row_updated_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter);
 static void model_row_removed_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter);
 
-static void holder_changed_cb (GdaSet *paramlist, GdaHolder *param);
+static GError *validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *new_value);
 static void holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param);
 
 /* get a pointer to the parents to be able to cvalue their destructor */
@@ -58,13 +60,12 @@ static GObjectClass  *parent_class = NULL;
 /* signals */
 enum
 {
-	ROW_TO_CHANGE,
         ROW_CHANGED,
 	END_OF_DATA,
         LAST_SIGNAL
 };
 
-static gint gda_data_model_iter_signals[LAST_SIGNAL] = { 0, 0, 0 };
+static gint gda_data_model_iter_signals[LAST_SIGNAL] = { 0, 0 };
 
 /* properties */
 enum
@@ -123,26 +124,6 @@ gda_data_model_iter_get_type (void)
 	return type;
 }
 
-static gboolean
-row_to_change_accumulator (GSignalInvocationHint *ihint,
-			   GValue *return_accu,
-			   const GValue *handler_return,
-			   gpointer data)
-{
-	gboolean thisvalue;
-
-	thisvalue = g_value_get_boolean (handler_return);
-	g_value_set_boolean (return_accu, thisvalue);
-
-	return thisvalue; /* stop signal if 'thisvalue' is FALSE */
-}
-
-static gboolean
-m_row_to_change (GdaDataModelIter *iter, gint row)
-{
-	return TRUE; /* defaults allows row to change */
-}
-
 static void
 gda_data_model_iter_class_init (GdaDataModelIterClass *class)
 {
@@ -151,55 +132,47 @@ gda_data_model_iter_class_init (GdaDataModelIterClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
-	gda_data_model_iter_signals [ROW_TO_CHANGE] =
-                g_signal_new ("row_to_change",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelIterClass, row_to_change),
-                              row_to_change_accumulator, NULL,
-                              gda_marshal_BOOLEAN__INT, G_TYPE_BOOLEAN, 1, G_TYPE_INT);
 	gda_data_model_iter_signals [ROW_CHANGED] =
-                g_signal_new ("row_changed",
+                g_signal_new ("row-changed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaDataModelIterClass, row_changed),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
 	gda_data_model_iter_signals [END_OF_DATA] =
-                g_signal_new ("end_of_data",
+                g_signal_new ("end-of-data",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaDataModelIterClass, end_of_data),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
-	class->row_to_change = m_row_to_change;
 	class->row_changed = NULL;
 	class->end_of_data = NULL;
 
 	object_class->dispose = gda_data_model_iter_dispose;
 	object_class->finalize = gda_data_model_iter_finalize;
-	paramlist_class->holder_changed = holder_changed_cb;
+	paramlist_class->validate_holder_change = validate_holder_change_cb;
 	paramlist_class->holder_attr_changed = holder_attr_changed_cb;
 
 	/* Properties */
 	object_class->set_property = gda_data_model_iter_set_property;
 	object_class->get_property = gda_data_model_iter_get_property;
 	g_object_class_install_property (object_class, PROP_DATA_MODEL,
-					 g_param_spec_object ("data_model", "Data model for which the iter is for", NULL,
+					 g_param_spec_object ("data-model", "Data model for which the iter is for", NULL,
                                                                GDA_TYPE_DATA_MODEL, 
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE |
 								G_PARAM_CONSTRUCT_ONLY)));
 	g_object_class_install_property (object_class, PROP_FORCED_MODEL,
-					 g_param_spec_object ("forced_model", NULL, NULL, 
+					 g_param_spec_object ("forced-model", NULL, NULL, 
                                                                GDA_TYPE_DATA_MODEL,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_CURRENT_ROW,
-					 g_param_spec_int ("current_row", "Current represented row in the data model", 
+					 g_param_spec_int ("current-row", "Current represented row in the data model", 
 							   NULL, -1, G_MAXINT, -1,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_UPDATE_MODEL,
-					 g_param_spec_boolean ("update_model", "Tells if parameters changes are forwarded "
+					 g_param_spec_boolean ("update-model", "Tells if parameters changes are forwarded "
 							       "to the GdaDataModel", NULL, TRUE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 }
@@ -223,7 +196,7 @@ model_row_updated_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter)
 	/* sync parameters with the new values in row */
 	if (iter->priv->row == row) {
 		iter->priv->keep_param_changes = TRUE;
-		gda_data_model_move_iter_at_row (iter->priv->data_model, iter, row);
+		gda_data_model_iter_move_at_row (iter, row);
 		iter->priv->keep_param_changes = FALSE;
 	}
 }
@@ -239,7 +212,7 @@ model_row_removed_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter)
 	 * then make all the parameters invalid */
 	if (iter->priv->row == row) {
 		gda_data_model_iter_invalidate_contents (iter);
-		gda_data_model_iter_set_at_row (iter, -1);
+		gda_data_model_iter_move_at_row (iter, -1);
 	}
 	else {
 		/* shift iter's row by one to keep good numbers */
@@ -251,13 +224,14 @@ model_row_removed_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter)
 /*
  * This function is called when a parameter in @paramlist is changed
  * to make sure the change is propagated to the GdaDataModel
- * paramlist is an iter for
+ * paramlist is an iter for, return an error if the data model could not be modified
  */
-static void
-holder_changed_cb (GdaSet *paramlist, GdaHolder *param)
+static GError *
+validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *new_value)
 {
 	GdaDataModelIter *iter;
 	gint col;
+	GError *error = NULL;
 
 	iter = (GdaDataModelIter *) paramlist;
 	if (!iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
@@ -266,24 +240,33 @@ holder_changed_cb (GdaSet *paramlist, GdaHolder *param)
 		
 		/* propagate the value update to the data model */
 		col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
-		g_return_if_fail (col >= 0);
-		
-		if (! gda_data_model_set_value_at ((GdaDataModel *) iter->priv->data_model, 
-						   col, iter->priv->row, gda_holder_get_value (param), NULL)) {
-			/* writing to the model failed, revert back the change to parameter */
-			iter->priv->keep_param_changes = TRUE;
-			gda_holder_set_value (param, gda_data_model_get_value_at (GDA_DATA_MODEL (iter->priv->data_model), 
-										     col, iter->priv->row)); 
-			iter->priv->keep_param_changes = FALSE;
+		if (col < 0) 
+			g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+				     _("Column %d out of range (0-%d)"), col, g_slist_length (paramlist->holders) - 1);
+		else if (GDA_DATA_MODEL_GET_CLASS ((GdaDataModel *) iter->priv->data_model)->i_iter_set_value) {
+			if (! (GDA_DATA_MODEL_GET_CLASS ((GdaDataModel *) iter->priv->data_model)->i_iter_set_value) 
+			    ((GdaDataModel *) iter->priv->data_model, iter, col, new_value, &error)) {
+				if (!error)
+					g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+						     _("GdaDataModel refused value change"));
+			}
+		}
+		else if (! gda_data_model_set_value_at ((GdaDataModel *) iter->priv->data_model, 
+							col, iter->priv->row, gda_holder_get_value (param), &error)) {
+			if (!error)
+				g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+					     _("GdaDataModel refused value change"));
 		}
 		
 		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
 		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
 	}
 
-	/* for the parent class */
-	if (((GdaSetClass *) parent_class)->holder_changed)
-		((GdaSetClass *) parent_class)->holder_changed (paramlist, param);
+	if (!error && ((GdaSetClass *) parent_class)->validate_holder_change)
+		/* for the parent class */
+		return ((GdaSetClass *) parent_class)->validate_holder_change (paramlist, param, new_value);
+
+	return error;
 }
 
 /*
@@ -366,6 +349,79 @@ gda_data_model_iter_finalize (GObject   * object)
 	parent_class->finalize (object);
 }
 
+static void
+set_holders_properties_from_select_stmt (GdaDataModelIter *iter, GdaConnection *cnc, GdaStatement *sel_stmt)
+{
+	GdaSqlStatement *sqlst = NULL;
+	GdaSqlStatementSelect *select;
+	GdaSqlSelectTarget *target;
+
+	g_object_get (G_OBJECT (sel_stmt), "structure", &sqlst, NULL);
+	g_assert (sqlst->stmt_type == GDA_SQL_STATEMENT_SELECT);
+	select = (GdaSqlStatementSelect*) sqlst->contents;
+
+	/* we only want a single target */
+	if (!select->from || !select->from->targets || select->from->targets->next)
+		goto out;
+	
+	target = (GdaSqlSelectTarget *) select->from->targets->data;
+	if (!target || !target->table_name)
+		goto out;
+
+	if (! gda_sql_statement_check_validity (sqlst, cnc, NULL))
+		goto out;
+
+	if (!target->validity_meta_object) {
+		g_warning ("Internal gda_sql_statement_check_validity() error: target->validity_meta_object is not set");
+		goto out;
+	}
+
+	GSList *fields, *holders;
+	for (fields = select->expr_list, holders = GDA_SET (iter)->holders; 
+	     fields && holders; 
+	     fields = fields->next, holders = holders->next) {
+		GdaSqlSelectField *selfield = (GdaSqlSelectField*) fields->data;
+		if (selfield->validity_meta_table_column) {
+			GdaMetaTableColumn *tcol = selfield->validity_meta_table_column;
+
+			/*g_print ("==> %s\n", tcol->column_name);*/
+			gda_holder_set_not_null (GDA_HOLDER (holders->data), ! tcol->nullok);
+			if (tcol->default_value) {
+				GValue *dvalue;
+				g_value_set_string ((dvalue = gda_value_new (G_TYPE_STRING)), tcol->default_value);
+				gda_holder_set_default_value (GDA_HOLDER (holders->data), dvalue);
+				gda_value_free (dvalue);
+			}
+		}
+		else if (selfield->validity_meta_object && (selfield->validity_meta_object->obj_type == GDA_META_DB_TABLE) &&
+			 selfield->expr && selfield->expr->value && !selfield->expr->param_spec && 
+			 (G_VALUE_TYPE (selfield->expr->value) == G_TYPE_STRING) &&
+			 !strcmp (g_value_get_string (selfield->expr->value), "*")) {
+			/* expand all the fields */
+			GdaMetaTable *mtable = GDA_META_TABLE (selfield->validity_meta_object);
+			GSList *tmplist;
+			for (tmplist = mtable->columns; tmplist; tmplist = tmplist->next) {
+				GdaMetaTableColumn *tcol = (GdaMetaTableColumn*) tmplist->data;
+				/*g_print ("*==> %s\n", tcol->column_name);*/
+				gda_holder_set_not_null (GDA_HOLDER (holders->data), ! tcol->nullok);
+				if (tcol->default_value) {
+					GValue *dvalue;
+					g_value_set_string ((dvalue = gda_value_new (G_TYPE_STRING)), tcol->default_value);
+					gda_holder_set_default_value (GDA_HOLDER (holders->data), dvalue);
+					gda_value_free (dvalue);
+				}
+				if (tmplist != mtable->columns)
+					holders = holders->next;
+			}
+		}
+	}
+	if (fields || holders)
+		g_warning ("Internal error: GdaDataModelIter has %d GdaHolders, and SELECT statement has %d expressions",
+			   g_slist_length (GDA_SET (iter)->holders), g_slist_length (select->expr_list));
+
+ out:
+	gda_sql_statement_free (sqlst);
+}
 
 static void 
 gda_data_model_iter_set_property (GObject *object,
@@ -388,23 +444,37 @@ gda_data_model_iter_set_property (GObject *object,
 			g_return_if_fail (ptr && GDA_IS_DATA_MODEL (ptr));
 			model = GDA_DATA_MODEL (ptr);
 			
-			/* REM: model is actually set using the next property */
+			/* REM: model is actually set in the next property's code (there is no break statement) */
 
 			/* compute parameters */
 			ncols = gda_data_model_get_n_columns (model);
 			for (col = 0; col < ncols; col++) {
-				const gchar *str;
+				gchar *str;
 				column = gda_data_model_describe_column (model, col);
 				param = (GdaHolder *) g_object_new (GDA_TYPE_HOLDER, 
-								    "g_type", 
+								    "g-type", 
 								    gda_column_get_g_type (column), NULL);
 
+				/*g_print ("COL %d allow null=%d\n", col, gda_column_get_allow_null (column));*/
 				gda_holder_set_not_null (param, !gda_column_get_allow_null (column));
-				str = gda_column_get_title (column);
-				if (!str)
-					str = gda_column_get_name (column);
-				if (str)
+				g_object_get (G_OBJECT (column), "id", &str, NULL);
+				if (str) {
 					g_object_set (G_OBJECT (param), "id", str, NULL);
+					g_free (str);
+				}
+				else {
+					const gchar *cstr;
+					cstr = gda_column_get_title (column);
+					if (!cstr)
+						cstr = gda_column_get_name (column);
+					if (cstr)
+						g_object_set (G_OBJECT (param), "id", cstr, NULL);
+					else {
+						gchar *tmp = g_strdup_printf ("col%d", col);
+						g_object_set (G_OBJECT (param), "id", tmp, NULL);
+						g_free (tmp);
+					}
+				}
 				if (gda_column_get_default_value (column))
 					gda_holder_set_default_value (param, gda_column_get_default_value (column));
 				else if (gda_column_get_auto_increment (column)) {
@@ -433,10 +503,23 @@ gda_data_model_iter_set_property (GObject *object,
 			iter->priv->data_model = GDA_DATA_MODEL (ptr);
 			g_object_add_weak_pointer (G_OBJECT (iter->priv->data_model),
 						   (gpointer*) &(iter->priv->data_model));
-			iter->priv->model_changes_signals [0] = g_signal_connect (G_OBJECT (ptr), "row_updated",
+			iter->priv->model_changes_signals [0] = g_signal_connect (G_OBJECT (ptr), "row-updated",
 										  G_CALLBACK (model_row_updated_cb), iter);
-			iter->priv->model_changes_signals [1] = g_signal_connect (G_OBJECT (ptr), "row_removed",
+			iter->priv->model_changes_signals [1] = g_signal_connect (G_OBJECT (ptr), "row-removed",
 										  G_CALLBACK (model_row_removed_cb), iter);
+
+			if (GDA_IS_DATA_SELECT (iter->priv->data_model)) {
+				GdaStatement *sel_stmt;
+				GdaConnection *cnc;
+				g_object_get (G_OBJECT (iter->priv->data_model), "connection", &cnc, 
+					      "select-stmt", &sel_stmt, NULL);
+				if (sel_stmt && cnc) 
+					set_holders_properties_from_select_stmt (iter, cnc, sel_stmt);
+				if (sel_stmt)
+					g_object_unref (sel_stmt);
+				if (cnc)
+					g_object_unref (cnc);
+			}
 			break;
                 }
 		case PROP_CURRENT_ROW:
@@ -479,47 +562,22 @@ gda_data_model_iter_get_property (GObject *object,
 }
 
 /**
- * gda_data_model_iter_can_be_moved
- * @iter: a #GdaDataModelIter object
- *
- * Tells if @iter can point to another row. Note the @iter by itself will not refuse
- * a row change, but that the row change may be refused by another object using
- * @iter.
- *
- * Returns: TRUE if the row represented by @iter can be changed
- */
-gboolean
-gda_data_model_iter_can_be_moved (GdaDataModelIter *iter)
-{
-	gboolean move_ok = TRUE;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
-	g_return_val_if_fail (iter->priv, FALSE);
-
-	if (!gda_data_model_iter_is_valid (iter))
-		return TRUE;
-
-	/* optionally validate the row change */
-	g_signal_emit (G_OBJECT (iter),
-		       gda_data_model_iter_signals[ROW_TO_CHANGE],
-		       0, iter->priv->row, &move_ok);
-
-	return move_ok;
-}
-
-/**
- * gda_data_model_iter_set_at_row
+ * gda_data_model_iter_move_at_row
  * @iter: a #GdaDataModelIter object
  * @row: the row to set @iter to
  *
- * Synchronizes the values of the parameters in @iter with the values at the @row row
+ * Synchronizes the values of the parameters in @iter with the values at the @row row.
  *
- * If @row < 0 then @iter is not bound to any row of the data model it iters through.
+ * If @row is not a valid row, then the returned value is FALSE, and the "current-row"
+ * property is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE)
+ *
+ * If any other error occurred then the returned value is FALSE, but the "current-row"
+ * property is set to the @row row.
  *
  * Returns: TRUE if no error occurred
  */
 gboolean
-gda_data_model_iter_set_at_row (GdaDataModelIter *iter, gint row)
+gda_data_model_iter_move_at_row (GdaDataModelIter *iter, gint row)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
 	g_return_val_if_fail (iter->priv, FALSE);
@@ -533,26 +591,170 @@ gda_data_model_iter_set_at_row (GdaDataModelIter *iter, gint row)
 		}
 		return TRUE;
 	}
-	else
-		return gda_data_model_move_iter_at_row (iter->priv->data_model, iter, row);
+	else {
+		GdaDataModel *model;
+		if ((gda_data_model_iter_get_row (iter) >= 0) &&
+		    (gda_data_model_iter_get_row (iter) != row) && 
+		    ! gda_set_is_valid ((GdaSet*) iter, NULL))
+			return FALSE;
+		
+		model = iter->priv->data_model;
+		if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row)
+			return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter, row);
+		else {
+			/* default method */
+			return gda_data_model_iter_move_at_row_default (model, iter, row);
+		}
+	}
 }
 
+static void
+set_param_attributes (GdaHolder *holder, GdaValueAttribute flags)
+{
+	if (flags & GDA_VALUE_ATTR_IS_DEFAULT)
+		gda_holder_set_value_to_default (holder);
+
+	if (flags & GDA_VALUE_ATTR_IS_NULL)
+		gda_holder_set_value (holder, NULL, NULL);
+	if (flags & GDA_VALUE_ATTR_DATA_NON_VALID)
+		gda_holder_force_invalid (holder);
+}
+
+/**
+ * gda_data_model_iter_move_at_row_default
+ */
+gboolean
+gda_data_model_iter_move_at_row_default (GdaDataModel *model, GdaDataModelIter *iter, gint row)
+{
+	/* default method */
+	GSList *list;
+	gint col;
+	GdaDataModel *test;
+	gboolean update_model;
+	
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+
+	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
+		return FALSE;
+
+	/* validity tests */
+	if ((row < 0) || (row >= gda_data_model_get_n_rows (model))) {
+		gda_data_model_iter_invalidate_contents (iter);
+		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
+		return FALSE;
+	}
+		
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
+	
+	g_object_get (G_OBJECT (iter), "data-model", &test, NULL);
+	g_return_val_if_fail (test == model, FALSE);
+	g_object_unref (test);
+	
+	/* actual sync. */
+	g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
+		const GValue *cvalue;
+		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
+		if (!cvalue || 
+		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, NULL)) {
+			g_object_set (G_OBJECT (iter), "current-row", row, 
+				      "update-model", update_model, NULL);
+			gda_data_model_iter_invalidate_contents (iter);
+			return FALSE;
+		}
+		set_param_attributes ((GdaHolder*) list->data, 
+				      gda_data_model_get_attributes_at (model, col, row));
+	}
+	g_object_set (G_OBJECT (iter), "current-row", row, "update-model", update_model, NULL);
+	return TRUE;
+}
+
+
 /**
  * gda_data_model_iter_move_next
  * @iter: a #GdaDataModelIter object
  *
- * Moves @iter one row further than where it already is (synchronizes the values of the parameters in @iter 
- * with the values at the new row).
+ * Moves @iter one row further than where it already is 
+ * (synchronizes the values of the parameters in @iter with the values at the new row).
  *
- * Returns: TRUE if no error occurred
+ * If the iterator was on the data model's last row, then it can't be moved forward
+ * anymore, and the returned value is FALSE; nore also that the "current-row" property
+ * is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE)
+ *
+ * If any other error occurred then the returned value is FALSE, but the "current-row"
+ * property is set to the new current row (one row more than it was before the call).
+ *
+ * Returns: TRUE if the iterator is now at the next row
  */
 gboolean
 gda_data_model_iter_move_next (GdaDataModelIter *iter)
 {
+	GdaDataModel *model;
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
 	g_return_val_if_fail (iter->priv, FALSE);
 
-	return gda_data_model_move_iter_next (iter->priv->data_model, iter);
+	if ((gda_data_model_iter_get_row (iter) >= 0) &&
+	    ! gda_set_is_valid ((GdaSet*) iter, NULL))
+		return FALSE;
+
+	model = iter->priv->data_model;
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next)
+		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next) (model, iter);
+	else 
+		/* default method */
+		return gda_data_model_iter_move_next_default (model, iter);
+}
+
+/**
+ * gda_data_model_iter_move_next_default
+ */
+gboolean
+gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *iter)
+{
+	GSList *list;
+	gint col;
+	gint row;
+	GdaDataModel *test;
+	gboolean update_model;
+	
+	/* validity tests */
+	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
+		return FALSE;
+	
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
+	
+	g_object_get (G_OBJECT (iter), "data-model", &test, NULL);
+	g_return_val_if_fail (test == model, FALSE);
+	g_object_unref (test);
+
+	g_object_get (G_OBJECT (iter), "current-row", &row, NULL);
+	row++;
+	if (row >= gda_data_model_get_n_rows (model)) {
+		gda_data_model_iter_invalidate_contents (iter);
+		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
+		return FALSE;
+	}
+	
+	/* actual sync. */
+	g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
+		const GValue *cvalue;
+		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
+		if (!cvalue || 
+		    !gda_holder_set_value ((GdaHolder *) list->data, cvalue, NULL)) {
+			g_object_set (G_OBJECT (iter), "current-row", row, 
+				      "update-model", update_model, NULL);
+			gda_data_model_iter_invalidate_contents (iter);
+			return FALSE;
+		}
+		set_param_attributes ((GdaHolder *) list->data, 
+				      gda_data_model_get_attributes_at (model, col, row));
+	}
+	g_object_set (G_OBJECT (iter), "current-row", row, 
+		      "update-model", update_model, NULL);
+	return TRUE;
 }
 
 /**
@@ -562,24 +764,94 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
  * Moves @iter one row before where it already is (synchronizes the values of the parameters in @iter 
  * with the values at the new row).
  *
- * Returns: TRUE if no error occurred
+ * If the iterator was on the data model's first row, then it can't be moved backwards
+ * anymore, and the returned value is FALSE; note also that the "current-row" property
+ * is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE).
+ *
+ * If any other error occurred then the returned value is FALSE, but the "current-row"
+ * property is set to the new current row (one row less than it was before the call).
+ *
+ * Returns: TRUE if the iterator is now at the previous row
  */
 gboolean
 gda_data_model_iter_move_prev (GdaDataModelIter *iter)
 {
+	GdaDataModel *model;
+
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
 	g_return_val_if_fail (iter->priv, FALSE);
 
-	return gda_data_model_move_iter_prev (iter->priv->data_model, iter);
+	if ((gda_data_model_iter_get_row (iter) >= 0) &&
+	    ! gda_set_is_valid ((GdaSet*) iter, NULL))
+		return FALSE;
+
+	model = iter->priv->data_model;
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev)
+		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev) (model, iter);
+	else 
+		/* default method */
+		return gda_data_model_iter_move_prev_default (model, iter);
 }
 
+/**
+ * gda_data_model_iter_move_prev_default
+ */
+gboolean
+gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *iter)
+{
+	GSList *list;
+	gint col;
+	gint row;
+	GdaDataModel *test;
+	gboolean update_model;
+	
+	/* validity tests */
+	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
+		return FALSE;
+	
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
+	
+	g_object_get (G_OBJECT (iter), "data-model", &test, NULL);
+	g_return_val_if_fail (test == model, FALSE);
+	g_object_unref (test);
+
+	g_object_get (G_OBJECT (iter), "current-row", &row, NULL);
+	row--;
+	if (row < 0) {
+		gda_data_model_iter_invalidate_contents (iter);
+		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
+		return FALSE;
+	}
+	
+	/* actual sync. */
+	g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
+		const GValue *cvalue;
+		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
+		if (!cvalue || 
+		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, NULL)) {
+			g_object_set (G_OBJECT (iter), "current-row", row, 
+				      "update-model", update_model, NULL);
+			gda_data_model_iter_invalidate_contents (iter);
+			return FALSE;
+		}
+		set_param_attributes ((GdaHolder*) list->data,
+				      gda_data_model_get_attributes_at (model, col, row));
+	}
+	g_object_set (G_OBJECT (iter), "current-row", row, 
+		      "update-model", update_model, NULL);
+	return TRUE;
+}
+
+
 /**
  * gda_data_model_iter_get_row
  * @iter: a #GdaDataModelIter object
  *
  * Get the row which @iter represents in the data model
  *
- * Returns: the row number, or -1 if not available
+ * Returns: the row number, or -1 if @iter is invalid
  */
 gint
 gda_data_model_iter_get_row (GdaDataModelIter *iter)
@@ -676,9 +948,9 @@ gda_data_model_iter_get_holder_for_field (GdaDataModelIter *iter, gint col)
  * @iter: a #GdaDataModelIter object
  * @col: the requested column
  *
- * Get the value stored at the column @col in @iter
+ * Get the value stored at the column @col in @iter. The returned value must not be modified.
  *
- * Returns: the #GValue, or %NULL
+ * Returns: the #GValue, or %NULL if the value could not be fetched
  */
 const GValue *
 gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
@@ -695,6 +967,36 @@ gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
 		return NULL;
 }
 
+/**
+ * gda_data_model_iter_set_value_at
+ * @iter: a #GdaDataModelIter object
+ * @col: the column number
+ * @value: a #GValue (not %NULL)
+ * @error: a place to store errors, or %NULL
+ * 
+ * Sets a value in @iter, at the column specified by @col
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_data_model_iter_set_value_at (GdaDataModelIter *iter, gint col, const GValue *value, GError **error)
+{
+	GdaHolder *holder;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
+	g_return_val_if_fail (iter->priv, FALSE);
+	g_return_val_if_fail (value, FALSE);
+
+	holder = gda_data_model_iter_get_holder_for_field (iter, col);
+	if (!holder) {
+		g_set_error (error, GDA_DATA_MODEL_ITER_ERROR, GDA_DATA_MODEL_ITER_COLUMN_OUT_OF_RANGE_ERROR,
+			     _("Column %d out of range (0-%d)"), col, 
+			     g_slist_length (((GdaSet *) iter)->holders) - 1);
+		return FALSE;
+	}
+	return gda_holder_set_value (holder, value, error);
+}
+
 /**
  * gda_data_model_iter_get_value_for_field
  * @iter: a #GdaDataModelIter object
