@@ -40,6 +40,8 @@
 #endif
 #include "csv.h"
 
+extern gchar *gda_lang_locale;
+
 static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
 static void gda_data_model_class_init (gpointer g_class);
 
@@ -55,6 +57,15 @@ enum {
 
 static guint gda_data_model_signals[LAST_SIGNAL];
 
+/* module error */
+GQuark gda_data_model_error_quark (void)
+{
+        static GQuark quark;
+        if (!quark)
+                quark = g_quark_from_static_string ("gda_data_model_error");
+        return quark;
+}
+
 GType
 gda_data_model_get_type (void)
 {
@@ -99,7 +110,7 @@ gda_data_model_class_init (gpointer g_class)
 				      g_cclosure_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 		gda_data_model_signals[ROW_INSERTED] =
-			g_signal_new ("row_inserted",
+			g_signal_new ("row-inserted",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
 				      G_STRUCT_OFFSET (GdaDataModelClass, row_inserted),
@@ -107,7 +118,7 @@ gda_data_model_class_init (gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		gda_data_model_signals[ROW_UPDATED] =
-			g_signal_new ("row_updated",
+			g_signal_new ("row-updated",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
 				      G_STRUCT_OFFSET (GdaDataModelClass, row_updated),
@@ -115,7 +126,7 @@ gda_data_model_class_init (gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		gda_data_model_signals[ROW_REMOVED] =
-			g_signal_new ("row_removed",
+			g_signal_new ("row-removed",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
 				      G_STRUCT_OFFSET (GdaDataModelClass, row_removed),
@@ -188,10 +199,9 @@ gda_data_model_row_inserted (GdaDataModel *model, gint row)
 		nbcols = gda_data_model_get_n_columns (model);
 		for (i = 0; i < nbcols; i++) {
 			column = gda_data_model_describe_column (model, i);
-			value = gda_data_model_get_value_at (model, i, 0);
-			if ((gda_column_get_g_type (column) == G_TYPE_INVALID))
-				gda_column_set_g_type (column, 
-							 value ? G_VALUE_TYPE ((GValue *)value) : GDA_TYPE_NULL);
+			value = gda_data_model_get_value_at (model, i, 0, NULL);
+			if (value && (gda_column_get_g_type (column) == G_TYPE_INVALID))
+				gda_column_set_g_type (column, G_VALUE_TYPE ((GValue *)value));
 		}
 	}
 
@@ -306,24 +316,6 @@ gda_data_model_thaw (GdaDataModel *model)
 		g_warning ("%s() method not supported\n", __FUNCTION__);
 }
 
-/**
- * gda_data_model_is_updatable
- * @model: a #GdaDataModel object.
- *
- * Tells if @model can be modified
- *
- * Returns: TRUE if @model can be modified
- */
-gboolean
-gda_data_model_is_updatable (GdaDataModel *model)
-{
-	GdaDataModelAccessFlags flags;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	flags = gda_data_model_get_access_flags (model);
-	return (flags & GDA_DATA_MODEL_ACCESS_WRITE);
-}
-
 /**
  * gda_data_model_get_access_flags
  * @model: a #GdaDataModel object.
@@ -411,16 +403,16 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
 }
 
 /**
- * gda_data_model_get_column_index_by_name
+ * gda_data_model_get_column_index
  * @model: a #GdaDataModel object.
  * @name: a column name
  *
- * Get the index of the column named @name in @model
+ * Get the index of the first column named @name in @model.
  *
  * Returns: the column index, or -1 if no column named @name was found
  */
 gint
-gda_data_model_get_column_index_by_name (GdaDataModel *model, const gchar *name)
+gda_data_model_get_column_index (GdaDataModel *model, const gchar *name)
 {
 	gint nbcols, ncol, ret;
 
@@ -429,8 +421,8 @@ gda_data_model_get_column_index_by_name (GdaDataModel *model, const gchar *name)
 	
 	ret = -1;
 	nbcols = gda_data_model_get_n_columns (model);
-	for(ncol = 0; ncol < nbcols; ncol++) {
-		if(g_str_equal (name, gda_data_model_get_column_title (model, ncol))) {
+	for (ncol = 0; ncol < nbcols; ncol++) {
+		if (g_str_equal (name, gda_data_model_get_column_title (model, ncol))) {
 			
 			ret = ncol;
 			break;
@@ -540,6 +532,7 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * @model: a #GdaDataModel object.
  * @col: a valid column number.
  * @row: a valid row number.
+ * @error: a place to store errors, or %NULL
  *
  * Retrieves the data stored in the given position (identified by
  * the @col and @row parameters) on a data model.
@@ -548,55 +541,80 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * To access data in a data model using a cursor, use a #GdaDataModelIter object, obtained using
  * gda_data_model_create_iter().
  *
- * Note that the returned #GValue must not be modified directly (unexpected behaviours may
- * occur if you do so). Also that value may become invalid as soon as any Libgda part is executed again,
- * which means if you want to keep the value, a copy must be made. 
+ * Note1: the returned #GValue must not be modified directly (unexpected behaviours may
+ * occur if you do so).
  *
- * If you want to modify a value stored in a #GdaDataModel, use the gda_data_model_set_value() method.
+ * Note2: the returned value may become invalid as soon as any Libgda part is executed again,
+ * which means if you want to keep the value, a copy must be made, however it will remain valid
+ * as long as the only Libgda usage is calling gda_data_model_get_value_at() for different values
+ * of the same row.
+ *
+ * If you want to modify a value stored in a #GdaDataModel, use the gda_data_model_set_value_at() or 
+ * gda_data_model_set_values() methods.
  *
  * Returns: a #GValue containing the value stored in the given
  * position, or %NULL on error (out-of-bound position, etc).
  */
 const GValue *
-gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at)
-		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at) (model, col, row);
+		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at) (model, col, row, error);
 	else {
 		/* method not supported */
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
+			     _("Data model does not support getting individual value"));
 		return NULL;
 	}
 }
 
 /**
- * gda_data_model_get_value_at_col_name
+ * gda_data_model_get_typed_value_at
  * @model: a #GdaDataModel object.
- * @column_name: a valid column name.
+ * @col: a valid column number.
  * @row: a valid row number.
+ * @expected_type: the expected data type of the returned value
+ * @nullok: if TRUE, then NULL values (value of type %GDA_TYPE_NULL) will not generate any error
+ * @error: a place to store errors, or %NULL
  *
- * Retrieves the data stored in the given position (identified by
- * the @col_name column and @row parameters) on a data model.
+ * This method is similar to gda_data_model_get_value_at(), except that it also allows one to specify the expected
+ * #GType of the value to get: if the data model returned a #GValue of a type different than the expected one, then
+ * this method returns %NULL and an error code.
  *
- * See also gda_data_model_get_value_at().
+ * Note: the same limitations and usage instructions apply as for gda_data_model_get_value_at().
  *
  * Returns: a #GValue containing the value stored in the given
- * position, or %NULL on error (out-of-bound position, etc).
+ * position, or %NULL on error (out-of-bound position, wrong data type, etc).
  */
 const GValue *
-gda_data_model_get_value_at_col_name (GdaDataModel *model, 
-				      const gchar *column_name, gint row)
+gda_data_model_get_typed_value_at (GdaDataModel *model, gint col, gint row, GType expected_type, gboolean nullok, GError **error)
 {
-	gint ncol;
+	const GValue *cvalue = NULL;
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-	g_return_val_if_fail (column_name, NULL);
 
-	ncol = gda_data_model_get_column_index_by_name (model, column_name);
-	if (ncol == -1)
-		return NULL;
-	else
-		return gda_data_model_get_value_at (model, ncol, row);
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at)
+		cvalue = (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at) (model, col, row, error);
+
+	if (cvalue) {
+		if (nullok && (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL) && (G_VALUE_TYPE (cvalue) != expected_type)) {
+			cvalue = NULL;
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUE_TYPE_ERROR,
+				     _("Data model returned value of invalid '%s' type"), gda_g_type_to_string (G_VALUE_TYPE (cvalue)));
+		}
+		else if (!nullok && (G_VALUE_TYPE (cvalue) != expected_type)) {
+			cvalue = NULL;
+			if (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL)
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUE_TYPE_ERROR,
+					     _("Data model returned invalid NULL value"));
+			else
+				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUE_TYPE_ERROR,
+					     _("Data model returned value of invalid '%s' type"), 
+					     gda_g_type_to_string (G_VALUE_TYPE (cvalue)));
+		}
+	}
+	return cvalue;
 }
 
 /**
@@ -628,9 +646,11 @@ gda_data_model_get_attributes_at (GdaDataModel *model, gint col, gint row)
  * @model: a #GdaDataModel object.
  * @col: column number.
  * @row: row number.
- * @value: a #GValue, or %NULL
+ * @value: a #GValue (not %NULL)
  * @error: a place to store errors, or %NULL
  *
+ * Modifies a value in @model, at (@col, @row).
+ *
  * Returns: TRUE if the value in the data model has been updated and no error occurred
  */
 gboolean
@@ -642,6 +662,8 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_set_value_at) (model, col, row, value, error);
 	else {
 		/* method not supported */
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
+			     _("Data model does not support setting individual value"));
 		return FALSE;
 	}
 }
@@ -650,11 +672,14 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * gda_data_model_set_values
  * @model: a #GdaDataModel object.
  * @row: row number.
- * @values: a list of #GValue, one for each n (&lt;nb_cols) columns of @model
+ * @values: a list of #GValue, one for at most the number of columns of @model
  * @error: a place to store errors, or %NULL
  *
- * If any value in @values is actually %NULL, then 
- * it is considered as a default value.
+ * In a similar way to gda_data_model_set_value_at(), this method modifies a data model's contents
+ * by setting several values at once.
+ *
+ * If any value in @values is actually %NULL, then the value in the corresponding column is left
+ * unchanged.
  *
  * Returns: TRUE if the value in the data model has been updated and no error occurred
  */
@@ -679,9 +704,9 @@ gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError
  * rows in @model.
  *
  * The row the returned #GdaDataModelIter represents is undefined. For models which can be accessed 
- * randomly the correspoding row can be set using gda_data_model_move_iter_at_row(), 
+ * randomly the correspoding row can be set using gda_data_model_iter_move_at_row(), 
  * and for models which are accessible sequentially only then the first row will be
- * fetched using gda_data_model_move_iter_next().
+ * fetched using gda_data_model_iter_move_next().
  *
  * Returns: a new #GdaDataModelIter object, or %NULL if an error occurred
  */
@@ -694,220 +719,9 @@ gda_data_model_create_iter (GdaDataModel *model)
 	else 
 		/* default method */
 		return  g_object_new (GDA_TYPE_DATA_MODEL_ITER, 
-				      "data_model", model, NULL);
+				      "data-model", model, NULL);
 }
 
-/**
- * gda_data_model_move_iter_at_row
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object
- * @row: row number.
- *
- * Sets @iter to represent the @row row. @iter must be a valid iterator object obtained
- * using the gda_data_model_create_iter() method.
- *
- * Returns: TRUE if no error occurred
- */
-gboolean
-gda_data_model_move_iter_at_row (GdaDataModel *model, GdaDataModelIter *iter, gint row)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-
-	if ((gda_data_model_iter_get_row (iter) != row) && ! gda_data_model_iter_can_be_moved (iter))
-		return FALSE;
-
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row)
-		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter, row);
-	else 
-		/* default method */
-		return gda_data_model_move_iter_at_row_default (model, iter, row);
-}
-
-static void set_param_attributes (GdaHolder *holder, GdaValueAttribute flags);
-
-gboolean
-gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *iter, gint row)
-{
-	/* default method */
-	GSList *list;
-	gint col;
-	GdaDataModel *test;
-	gboolean update_model;
-	
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-
-	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
-		return FALSE;
-
-	/* validity tests */
-	if (row >= gda_data_model_get_n_rows (model)) {
-		gda_data_model_iter_invalidate_contents (iter);
-		g_object_set (G_OBJECT (iter), "current_row", -1, NULL);
-		return FALSE;
-	}
-		
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
-	
-	g_object_get (G_OBJECT (iter), "data_model", &test, NULL);
-	g_return_val_if_fail (test == model, FALSE);
-	g_object_unref (test);
-	
-	/* actual sync. */
-	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-	g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
-		gda_holder_set_value ((GdaHolder*) list->data, 
-				      gda_data_model_get_value_at (model, col, row));
-
-		set_param_attributes ((GdaHolder*) list->data, 
-				      gda_data_model_get_attributes_at (model, col, row));
-	}
-	g_object_set (G_OBJECT (iter), "current_row", row, "update_model", update_model, NULL);
-	return TRUE;
-}
-
-static void
-set_param_attributes (GdaHolder *holder, GdaValueAttribute flags)
-{
-	if (flags & GDA_VALUE_ATTR_IS_DEFAULT)
-		gda_holder_set_value_to_default (holder);
-
-	gda_holder_set_not_null (holder, ! (flags & GDA_VALUE_ATTR_CAN_BE_NULL));
-	if (flags & GDA_VALUE_ATTR_IS_NULL)
-		gda_holder_set_value (holder, NULL);
-	if (flags & GDA_VALUE_ATTR_DATA_NON_VALID)
-		gda_holder_force_invalid (holder);
-}
-
-/**
- * gda_data_model_move_iter_next
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object
- *
- * Sets @iter to the next available row in @model. @iter must be a valid iterator object obtained
- * using the gda_data_model_create_iter() method.
- *
- * Returns: TRUE if no error occurred
- */
-gboolean
-gda_data_model_move_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-
-	if (! gda_data_model_iter_can_be_moved (iter))
-		return FALSE;
-
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next)
-		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next) (model, iter);
-	else 
-		/* default method */
-		return gda_data_model_move_iter_next_default (model, iter);
-}
-
-gboolean
-gda_data_model_move_iter_next_default (GdaDataModel *model, GdaDataModelIter *iter)
-{
-	GSList *list;
-	gint col;
-	gint row;
-	GdaDataModel *test;
-	gboolean update_model;
-	
-	/* validity tests */
-	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
-		return FALSE;
-	
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
-	
-	g_object_get (G_OBJECT (iter), "data_model", &test, NULL);
-	g_return_val_if_fail (test == model, FALSE);
-	g_object_unref (test);
-
-	g_object_get (G_OBJECT (iter), "current_row", &row, NULL);
-	row++;
-	if (row >= gda_data_model_get_n_rows (model)) {
-		g_object_set (G_OBJECT (iter), "current_row", -1, NULL);
-		return FALSE;
-	}
-	
-	/* actual sync. */
-	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-	g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
-		gda_holder_set_value ((GdaHolder *) list->data, 
-				      gda_data_model_get_value_at (model, col, row));
-		set_param_attributes ((GdaHolder *) list->data, 
-				      gda_data_model_get_attributes_at (model, col, row));
-	}
-	g_object_set (G_OBJECT (iter), "current_row", row, 
-		      "update_model", update_model, NULL);
-	return TRUE;
-}
-
-/**
- * gda_data_model_move_iter_prev
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object
- *
- * Sets @iter to the previous available row in @model. @iter must be a valid iterator object obtained
- * using the gda_data_model_create_iter() method.
- *
- * Returns: TRUE if no error occurred
- */
-gboolean
-gda_data_model_move_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-
-	if (! gda_data_model_iter_can_be_moved (iter))
-		return FALSE;
-
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev)
-		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev) (model, iter);
-	else 
-		/* default method */
-		return gda_data_model_move_iter_prev_default (model, iter);
-}
-
-gboolean
-gda_data_model_move_iter_prev_default (GdaDataModel *model, GdaDataModelIter *iter)
-{
-	GSList *list;
-	gint col;
-	gint row;
-	GdaDataModel *test;
-	gboolean update_model;
-	
-	/* validity tests */
-	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
-		return FALSE;
-	
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
-	
-	g_object_get (G_OBJECT (iter), "data_model", &test, NULL);
-	g_return_val_if_fail (test == model, FALSE);
-	g_object_unref (test);
-
-	g_object_get (G_OBJECT (iter), "current_row", &row, NULL);
-	row--;
-	if (row < 0)
-		return FALSE;
-	
-	/* actual sync. */
-	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-	g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
-		gda_holder_set_value ((GdaHolder*) list->data, 
-				      gda_data_model_get_value_at (model, col, row));
-		set_param_attributes ((GdaHolder*) list->data,
-				      gda_data_model_get_attributes_at (model, col, row));
-	}
-	g_object_set (G_OBJECT (iter), "current_row", row, 
-		      "update_model", update_model, NULL);
-	return TRUE;	
-}
-
-
 /**
  * gda_data_model_append_values
  * @model: a #GdaDataModel object.
@@ -939,7 +753,8 @@ gda_data_model_append_values (GdaDataModel *model, const GList *values, GError *
  * @model: a #GdaDataModel object.
  * @error: a place to store errors, or %NULL
  * 
- * Appends a row to the data model. 
+ * Appends a row to the data model (the new row will possibliy have NULL values for all columns,
+ * or some other values depending on the data model implementation)
  *
  * Returns: the number of the added row, or -1 if an error occurred
  */
@@ -995,7 +810,7 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 /**
  * gda_data_model_get_row_from_values
  * @model: a #GdaDataModel object.
- * @values: a list of #GValue values
+ * @values: a list of #GValue values (no %NULL is allowed)
  * @cols_index: an array of #gint containing the column number to match each value of @values
  *
  * Returns the first row where all the values in @values at the columns identified at
@@ -1045,9 +860,9 @@ gda_data_model_get_row_from_values (GdaDataModel *model, GSList *values, gint *c
                 while (list && allequal) {
                         if (cols_index)
                                 g_return_val_if_fail (cols_index [index] < n_cols, FALSE);
-                        value = gda_data_model_get_value_at (model, cols_index [index], current_row);
+                        value = gda_data_model_get_value_at (model, cols_index [index], current_row, NULL);
 
-                        if (gda_value_compare_ext ((GValue *) (list->data), (GValue *) value))
+                        if (!value || !(list->data) || gda_value_compare ((GValue *) (list->data), (GValue *) value))
                                 allequal = FALSE;
 
                         list = g_slist_next (list);
@@ -1318,7 +1133,7 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, c
 			GValue *value;
 			gchar *txt;
 
-			value = (GValue *) gda_data_model_get_value_at (model, cols[c], rows ? rows[r] : r);
+			value = (GValue *) gda_data_model_get_value_at (model, cols[c], rows ? rows[r] : r, NULL);
 			if (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)
 				txt = g_strdup (g_value_get_boolean (value) ? "TRUE" : "FALSE");
 			else {
@@ -1357,15 +1172,6 @@ xml_set_boolean (xmlNodePtr node, const gchar *name, gboolean value)
 	xmlSetProp (node, (xmlChar*)name, value ? (xmlChar*)"TRUE" : (xmlChar*)"FALSE");
 }
 
-static void
-xml_set_int (xmlNodePtr node, const gchar *name, gint value)
-{
-	char s[80];
-
-	sprintf (s, "%d", value);
-	xmlSetProp (node, (xmlChar*)name, (xmlChar*)s);
-}
-
 /**
  * gda_data_model_to_xml_node
  * @model: a #GdaDataModel object.
@@ -1377,7 +1183,7 @@ xml_set_int (xmlNodePtr node, const gchar *name, gint value)
  *
  * Converts a #GdaDataModel into a xmlNodePtr (as used in libxml).
  *
- * Returns: a xmlNodePtr representing the whole data model.
+ * Returns: a xmlNodePtr representing the whole data model, or %NULL if an error occurred
  */
 xmlNodePtr
 gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, 
@@ -1386,16 +1192,26 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 	xmlNodePtr node;
 	gint i;
 	gint *rcols, rnb_cols;
+	const gchar *cstr;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	node = xmlNewNode (NULL, BAD_CAST "gda_array");
-	xmlSetProp (node, BAD_CAST "id", BAD_CAST "EXPORT");
+	cstr = g_object_get_data (G_OBJECT (model), "id");
+	if (cstr)
+		xmlSetProp (node, BAD_CAST "id", BAD_CAST cstr);
+	else
+		xmlSetProp (node, BAD_CAST "id", BAD_CAST "EXPORT");
 
 	if (name)
 		xmlSetProp (node, BAD_CAST "name", BAD_CAST name);
-	else
-		xmlSetProp (node, BAD_CAST "name", BAD_CAST _("Exported Data"));
+	else {
+		cstr = g_object_get_data (G_OBJECT (model), "name");
+		if (cstr)
+			xmlSetProp (node, BAD_CAST "name", BAD_CAST cstr);
+		else
+			xmlSetProp (node, BAD_CAST "name", BAD_CAST _("Exported Data"));
+	}
 
 	/* compute columns if not provided */
 	if (!cols) {
@@ -1423,42 +1239,30 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 		}
 
 		field = xmlNewChild (node, NULL, BAD_CAST "gda_array_field", NULL);
-		str = g_strdup_printf ("FI%d", i);
+		g_object_get (G_OBJECT (column), "id", &str, NULL);
+		if (!str)
+			str = g_strdup_printf ("FI%d", i);
 		xmlSetProp (field, BAD_CAST "id", BAD_CAST str);
 		g_free (str);
 		xmlSetProp (field, BAD_CAST "name", BAD_CAST gda_column_get_name (column));
 		cstr = gda_column_get_title (column);
 		if (cstr && *cstr)
 			xmlSetProp (field, BAD_CAST "title", BAD_CAST cstr);
-		cstr = gda_column_get_caption (column);
-		if (cstr && *cstr)
-			xmlSetProp (field, BAD_CAST "caption", BAD_CAST cstr);
 		cstr = gda_column_get_dbms_type (column);
 		if (cstr && *cstr)
 			xmlSetProp (field, BAD_CAST "dbms_type", BAD_CAST cstr);
 		xmlSetProp (field, BAD_CAST "gdatype", BAD_CAST gda_g_type_to_string (gda_column_get_g_type (column)));
-		if (gda_column_get_defined_size (column) != 0)
-			xml_set_int (field, "size", gda_column_get_defined_size (column));
-		if (gda_column_get_scale (column) != 0)
-			xml_set_int (field, "scale", gda_column_get_scale (column));
-		if (gda_column_get_primary_key (column))
-			xml_set_boolean (field, "pkey", gda_column_get_primary_key (column));
-		if (gda_column_get_unique_key (column))
-			xml_set_boolean (field, "unique", gda_column_get_unique_key (column));
 		if (gda_column_get_allow_null (column))
 			xml_set_boolean (field, "nullok", gda_column_get_allow_null (column));
 		if (gda_column_get_auto_increment (column))
 			xml_set_boolean (field, "auto_increment", gda_column_get_auto_increment (column));
-		cstr = gda_column_get_references (column);
-		if (cstr && *cstr)
-			xmlSetProp (field, BAD_CAST "ref", BAD_CAST cstr);
-		cstr = gda_column_get_table (column);
-		if (cstr && *cstr)
-			xmlSetProp (field, BAD_CAST "table", BAD_CAST cstr);
 	}
 	
 	/* add the model data to the XML output */
-	gda_utility_data_model_dump_data_to_xml (model, node, cols, nb_cols, rows, nb_rows, FALSE);
+	if (!gda_utility_data_model_dump_data_to_xml (model, node, cols, nb_cols, rows, nb_rows, FALSE)) {
+		xmlFreeNode (node);
+		node = NULL;
+	}
 
 	if (!cols)
 		g_free (rcols);
@@ -1505,12 +1309,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 	gboolean retval = TRUE;
 	gint pos = 0;
 
-	const gchar *lang = NULL;
-#ifdef HAVE_LC_MESSAGES
-	lang = setlocale (LC_MESSAGES, NULL);
-#else
-	lang = setlocale (LC_CTYPE, NULL);
-#endif
+	const gchar *lang = gda_lang_locale;
 
 	values = g_ptr_array_new ();
 	g_ptr_array_set_size (values, gda_data_model_get_n_columns (model));
@@ -1703,7 +1502,8 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 		return FALSE;
 	}
 
-	/* make a list of the parameters which will be used during copy (stored in reverse order!) ,
+	/* make a list of the parameters which will be used during copy (stored in reverse order: last column of data model
+	 * represented by the 1st GdaHolder) ,
 	 * an some tests */
 	for (i = 0; i < to_nb_cols; i++) {
 		GdaHolder *param = NULL;
@@ -1747,17 +1547,29 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 				return FALSE;
 			}
 		}
+		
 		copy_params = g_slist_prepend (copy_params, param);
 	}
 
+#ifdef GDA_DEBUG_NO
+	{
+		GSList *list;
+		for (list = copy_params; list; list = list->next) {
+			GdaHolder *param = GDA_HOLDER (list->data);
+			g_print ("Copy Param: %s (%s)\n", gda_holder_get_id (param), 
+				 gda_g_type_to_string (gda_holder_get_g_type (param)));
+		}
+	}
+#endif
+
 	/* build the append_values list (stored in reverse order!)
 	 * node->data is:
 	 * - NULL if the value must be replaced by the value of the copied model
 	 * - a GValue of type GDA_VALYE_TYPE_NULL if a null value must be inserted in the dest data model
-	 * - a GValue of a different type if the value must be converted from the ser data model
+	 * - a GValue of a different type if the value must be converted from the src data model
 	 */
 	append_types = g_new0 (GType, to_nb_cols);
-	for (plist = copy_params, i = 0; plist; plist = plist->next, i++) {
+	for (plist = copy_params, i = to_nb_cols - 1; plist; plist = plist->next, i--) {
 		GdaColumn *column;
 
 		column = gda_data_model_describe_column (to, i);
@@ -1779,6 +1591,13 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 			append_values = g_list_prepend (append_values, gda_value_new_null ());
 	}
 	append_values = g_list_reverse (append_values);
+#ifdef GDA_DEBUG_NO
+	{
+		GList *list;
+		for (list = append_values; list; list = list->next) 
+			g_print ("Append Value: %s\n", list->data ? gda_g_type_to_string (G_VALUE_TYPE ((GValue *) (list->data))) : "NULL");
+	}
+#endif
 	
 	/* actual data copy (no memory allocation is needed here) */
 	gda_data_model_send_hint (to, GDA_DATA_MODEL_HINT_START_BATCH_UPDATE, NULL);
@@ -1788,7 +1607,14 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 		to_nb_rows = gda_data_model_get_n_rows (to);
 	}
 
-	gda_data_model_iter_move_next (from_iter); /* move to first row */
+	gboolean mstatus;
+	mstatus = gda_data_model_iter_move_next (from_iter); /* move to first row */
+	if (!mstatus) {
+		gint crow;
+		g_object_get (from_iter, "current-row", &crow, NULL);
+		if (crow >= 0)
+			retval = FALSE;
+	}
 	while (retval && gda_data_model_iter_is_valid (from_iter)) {
 		GList *values = NULL;
 		GList *avlist = append_values;
@@ -1847,7 +1673,13 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 		}
 		
 		g_list_free (values);
-		gda_data_model_iter_move_next (from_iter);
+		mstatus = gda_data_model_iter_move_next (from_iter);
+		if (!mstatus) {
+			gint crow;
+			g_object_get (from_iter, "current-row", &crow, NULL);
+			if (crow >= 0)
+				retval = FALSE;
+		}
 	}
 	
 	/* free memory */
@@ -1940,7 +1772,8 @@ gda_data_model_import_from_file (GdaDataModel *model,
 }
 
 static gchar *real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes, 
-						  gboolean dump_rows, gboolean dump_title, gboolean null_as_empty);
+						  gboolean dump_rows, gboolean dump_title, gboolean null_as_empty,
+						  GError **error);
 
 /**
  * gda_data_model_dump
@@ -1965,6 +1798,7 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 	gboolean dump_rows = FALSE;
 	gboolean dump_title = FALSE;
 	gboolean null_as_empty = FALSE;
+	GError *error = NULL;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	g_return_if_fail (to_stream);
@@ -1978,13 +1812,29 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) 
 		null_as_empty = TRUE;
 
-	str = real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty);
-	g_fprintf (to_stream, "%s", str);
-	g_free (str);
-	if (dump_attrs) {
-		str = real_gda_data_model_dump_as_string (model, TRUE, dump_rows, dump_title, null_as_empty);
+	str = real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty, &error);
+	if (str) {
 		g_fprintf (to_stream, "%s", str);
 		g_free (str);
+		if (dump_attrs) {
+			str = real_gda_data_model_dump_as_string (model, TRUE, dump_rows, dump_title, null_as_empty, &error);
+			if (str) {
+				g_fprintf (to_stream, "%s", str);
+				g_free (str);
+			}
+			else {
+				g_warning (_("Could not dump data model's attributes: %s"),
+					   error && error->message ? error->message : _("No detail"));
+				if (error)
+					g_error_free (error);
+			}
+		}
+	}
+	else {
+		g_warning (_("Could not dump data model's contents: %s"),
+			   error && error->message ? error->message : _("No detail"));
+		if (error)
+			g_error_free (error);
 	}
 }
 
@@ -2018,7 +1868,7 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 	if (getenv ("GDA_DATA_MODEL_NULL_AS_EMPTY")) 
 		null_as_empty = TRUE;
 
-	return real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty);
+	return real_gda_data_model_dump_as_string (model, FALSE, dump_rows, dump_title, null_as_empty, NULL);
 }
 
 static void
@@ -2045,10 +1895,11 @@ string_get_dimensions (const gchar *string, gint *width, gint *rows)
 
 static gchar *
 real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes, 
-				    gboolean dump_rows, gboolean dump_title, gboolean null_as_empty)
+				    gboolean dump_rows, gboolean dump_title, gboolean null_as_empty, GError **error)
 {
 #define MULTI_LINE_NO_SEPARATOR
 
+	gboolean allok = TRUE;
 	GString *string;
 	gchar *offstr, *str;
 	gint n_cols, n_rows;
@@ -2128,7 +1979,11 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	for (j = 0; j < n_rows; j++) {
 		for (i = 0; i < n_cols; i++) {
 			if (! dump_attributes) {
-				value = gda_data_model_get_value_at (model, i, j);
+				value = gda_data_model_get_value_at (model, i, j, error);
+				if (!value) {
+					allok = FALSE;
+					break;
+				}
 				str = NULL;
 				if (null_as_empty) {
 					if (!value || gda_value_is_null (value))
@@ -2153,10 +2008,12 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 				g_free (str);
 			}
 		}
+		if (!allok)
+			break;
 	}
 	
 	/* actual dumping of the contents: title */
-	if (dump_title) {
+	if (allok && dump_title) {
 		const gchar *title;
 		title = g_object_get_data (G_OBJECT (model), "name");
 		if (title) {
@@ -2176,10 +2033,10 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	}
 
 	/* ...column titles...*/
-	if (dump_rows) 
+	if (allok && dump_rows) 
 		g_string_append_printf (string, "%*s", cols_size [0], "#row");
 
-	for (i = 0; i < n_cols; i++) {
+	for (i = 0; (i < n_cols) && allok; i++) {
 		gint j, max;
 		str = (gchar *) gda_data_model_get_column_title (model, i);
 		if (dump_rows || (i != 0))
@@ -2199,7 +2056,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	g_string_append_c (string, '\n');
 		
 	/* ... separation line ... */
-	for (i = 0; i < n_cols + col_offset; i++) {
+	for (i = 0; (i < n_cols + col_offset) && allok; i++) {
 		if (i != 0)
 			g_string_append_printf (string, "%s", sep_row);
 		for (j = 0; j < cols_size [i]; j++)
@@ -2208,7 +2065,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	g_string_append_c (string, '\n');
 
 	/* ... and data */
-	if (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM) {
+	if (allok && gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM) {
 		for (j = 0; j < n_rows; j++) {
 			/* determine height for each column in that row */
 			gint *cols_height = g_new (gint, n_cols + col_offset);
@@ -2224,7 +2081,11 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 
 			for (i = 0; i < n_cols; i++) {
 				if (!dump_attributes) {
-					value = gda_data_model_get_value_at (model, i, j);
+					value = gda_data_model_get_value_at (model, i, j, error);
+					if (!value) {
+						allok = FALSE;
+						break;
+					}
 					str = NULL;
 					if (null_as_empty) {
 						if (!value || gda_value_is_null (value))
@@ -2249,6 +2110,8 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 					cols_height [i + col_offset] = 0;
 				}
 			}
+			if (!allok)
+				break;
 
 			for (k = 0; k < kmax; k++) {
 				for (i = 0; i < n_cols + col_offset; i++) {
@@ -2327,7 +2190,13 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 
 	g_free (offstr);
 	
-	str = string->str;
-	g_string_free (string, FALSE);
+	if (allok) {
+		str = string->str;
+		g_string_free (string, FALSE);
+	}
+	else {
+		str = NULL;
+		g_string_free (string, TRUE);
+	}
 	return str;
 }
