@@ -18,6 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <glib/gi18n-lib.h>
 #include <stdarg.h>
 #include <string.h>
 #include "gda-holder.h"
@@ -56,10 +57,11 @@ enum
 {
 	CHANGED,
         SOURCE_CHANGED,
+	VALIDATE_CHANGE,
         LAST_SIGNAL
 };
 
-static gint gda_holder_signals[LAST_SIGNAL] = { 0, 0 };
+static gint gda_holder_signals[LAST_SIGNAL] = { 0, 0, 0 };
 
 
 /* properties */
@@ -140,6 +142,26 @@ gda_holder_get_type (void)
 	return type;
 }
 
+static gboolean
+validate_change_accumulator (GSignalInvocationHint *ihint,
+			   GValue *return_accu,
+			   const GValue *handler_return,
+			   gpointer data)
+{
+	GError *error;
+
+	error = g_value_get_pointer (handler_return);
+	g_value_set_pointer (return_accu, error);
+
+	return error ? FALSE : TRUE; /* stop signal if 'thisvalue' is FALSE */
+}
+
+static GError *
+m_validate_change (GdaHolder *holder, const GValue *new_value)
+{
+	return NULL;
+}
+
 static void
 gda_holder_class_init (GdaHolderClass *class)
 {
@@ -148,7 +170,7 @@ gda_holder_class_init (GdaHolderClass *class)
 	parent_class = g_type_class_peek_parent (class);
 
 	gda_holder_signals[SOURCE_CHANGED] =
-                g_signal_new ("source_changed",
+                g_signal_new ("source-changed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaHolderClass, source_changed),
@@ -161,9 +183,28 @@ gda_holder_class_init (GdaHolderClass *class)
                               G_STRUCT_OFFSET (GdaHolderClass, changed),
                               NULL, NULL,
                               gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+	/**
+	 * GdaHolder::before-change:
+	 * @holder: the object which received the signal
+	 * @new_value: the proposed new value for @holder
+	 * 
+	 * Gets emitted when @holder is going to change its value. One can connect to
+	 * this signal to control which values @holder can have (for example to implement some business rules)
+	 *
+	 * Return value: NULL if @holder is allowed to change its value to @new_value, or a #GError
+	 * otherwise.
+	 */
+	gda_holder_signals[VALIDATE_CHANGE] =
+		g_signal_new ("validate-change",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaHolderClass, validate_change),
+                              validate_change_accumulator, NULL,
+                              gda_marshal_POINTER__POINTER, G_TYPE_POINTER, 1, G_TYPE_POINTER);
 
         class->changed = NULL;
         class->source_changed = NULL;
+        class->validate_change = m_validate_change;
 
 	/* virtual functions */
 	object_class->dispose = gda_holder_dispose;
@@ -182,7 +223,7 @@ gda_holder_class_init (GdaHolderClass *class)
 					 g_param_spec_string ("description", NULL, NULL, NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property (object_class, PROP_GDA_TYPE,
-                                         g_param_spec_ulong ("g_type", NULL, NULL,
+                                         g_param_spec_ulong ("g-type", NULL, NULL,
 							   0, G_MAXULONG, GDA_TYPE_NULL,
 							   (G_PARAM_READABLE | 
 							    G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
@@ -190,22 +231,22 @@ gda_holder_class_init (GdaHolderClass *class)
 					 g_param_spec_string ("plugin", NULL, NULL, NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_NOT_NULL,
-					 g_param_spec_boolean ("not_null", NULL, NULL, FALSE,
+					 g_param_spec_boolean ("not-null", NULL, NULL, FALSE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_SIMPLE_BIND,
-					 g_param_spec_object ("simple_bind", NULL, NULL, 
+					 g_param_spec_object ("simple-bind", NULL, NULL, 
                                                                GDA_TYPE_HOLDER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_FULL_BIND,
-					 g_param_spec_object ("full_bind", NULL, NULL, 
+					 g_param_spec_object ("full-bind", NULL, NULL, 
                                                                GDA_TYPE_HOLDER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_SOURCE_MODEL,
-                                         g_param_spec_object ("source_model", NULL, NULL,
+                                         g_param_spec_object ("source-model", NULL, NULL,
                                                                GDA_TYPE_DATA_MODEL,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property (object_class, PROP_SOURCE_COLUMN,
-                                         g_param_spec_int ("source_column", NULL, NULL,
+                                         g_param_spec_int ("source-column", NULL, NULL,
 							   0, G_MAXINT, 0,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 }
@@ -250,7 +291,7 @@ gda_holder_new (GType type)
 
 	g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
 
-        obj = g_object_new (GDA_TYPE_HOLDER, "g_type", type, NULL);
+        obj = g_object_new (GDA_TYPE_HOLDER, "g-type", type, NULL);
 
         return (GdaHolder *) obj;
 }
@@ -268,11 +309,12 @@ gda_holder_copy (GdaHolder *orig)
 {
 	GObject *obj;
 	GdaHolder *holder;
+	gboolean allok = TRUE;
 
 	g_return_val_if_fail (orig && GDA_IS_HOLDER (orig), NULL);
 	g_return_val_if_fail (orig->priv, NULL);
 
-	obj = g_object_new (GDA_TYPE_HOLDER, "g_type", orig->priv->g_type, NULL);
+	obj = g_object_new (GDA_TYPE_HOLDER, "g-type", orig->priv->g_type, NULL);
 	holder = GDA_HOLDER (obj);
 
 	if (orig->priv->id)
@@ -284,28 +326,34 @@ gda_holder_copy (GdaHolder *orig)
 
 	if (orig->priv->full_bind)
 		gda_holder_set_full_bind (holder, orig->priv->full_bind);
-	if (orig->priv->simple_bind)
-		gda_holder_set_bind (holder, orig->priv->simple_bind);
+	if (orig->priv->simple_bind) 
+		allok = gda_holder_set_bind (holder, orig->priv->simple_bind, NULL);
 	
-	if (orig->priv->source_model) {
+	if (allok && orig->priv->source_model) {
 		/*g_print ("Source holder %p\n", holder);*/
-		gda_holder_set_source_model (holder, orig->priv->source_model, orig->priv->source_col,
-					       NULL);
+		allok = gda_holder_set_source_model (holder, orig->priv->source_model, orig->priv->source_col,
+						     NULL);
 	}
 
-	/* direct settings */
-	holder->priv->invalid_forced = orig->priv->invalid_forced;
-	holder->priv->valid = orig->priv->valid;
-	holder->priv->default_forced = orig->priv->default_forced;	
-	if (orig->priv->value)
-		holder->priv->value = gda_value_copy (orig->priv->value);
-	if (orig->priv->default_value)
-		holder->priv->default_value = gda_value_copy (orig->priv->default_value);
-	holder->priv->not_null = orig->priv->not_null;
-	if (orig->priv->plugin)
-		holder->priv->plugin = g_strdup (orig->priv->plugin);
-
-	return holder;
+	if (allok) {
+		/* direct settings */
+		holder->priv->invalid_forced = orig->priv->invalid_forced;
+		holder->priv->valid = orig->priv->valid;
+		holder->priv->default_forced = orig->priv->default_forced;	
+		if (orig->priv->value)
+			holder->priv->value = gda_value_copy (orig->priv->value);
+		if (orig->priv->default_value)
+			holder->priv->default_value = gda_value_copy (orig->priv->default_value);
+		holder->priv->not_null = orig->priv->not_null;
+		if (orig->priv->plugin)
+			holder->priv->plugin = g_strdup (orig->priv->plugin);
+		return holder;
+	}
+	else {
+		g_warning ("Internal error: could not copy GdaHolder (please report a bug).");
+		g_object_unref (holder);
+		return NULL;
+	}
 }
 
 /**
@@ -335,6 +383,7 @@ gda_holder_new_inline (GType type, const gchar *id, ...)
 	if (holder) {
 		GValue *value;
 		va_list ap;
+		GError *lerror = NULL;
 
 		if (id)
 			holder->priv->id = g_strdup (id);
@@ -378,12 +427,22 @@ gda_holder_new_inline (GType type, const gchar *id, ...)
 			gda_value_set_numeric (value, va_arg (ap, GdaNumeric *));
 		else if (type == G_TYPE_DATE)
 			g_value_set_boxed (value, va_arg (ap, GDate *));
-		else
+		else {
 			g_warning ("%s() does not handle values of type %s, value will not be assigned.",
 				   __FUNCTION__, g_type_name (type));
+			g_object_unref (holder);
+			holder = NULL;
+		}
 		va_end (ap);
 
-		gda_holder_set_value (holder, value);
+		if (holder && !gda_holder_set_value (holder, value, &lerror)) {
+			g_warning (_("Unable to set holder's value: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+			if (lerror)
+				g_error_free (lerror);
+			g_object_unref (holder);
+			holder = NULL;
+		}
 		gda_value_free (value);
 	}
 
@@ -397,7 +456,7 @@ gda_holder_dispose (GObject *object)
 
 	holder = GDA_HOLDER (object);
 	if (holder->priv) {
-		gda_holder_set_bind (holder, NULL);
+		gda_holder_set_bind (holder, NULL, NULL);
 		gda_holder_set_full_bind (holder, NULL);
 
 		if (holder->priv->source_model) {
@@ -501,7 +560,8 @@ gda_holder_set_property (GObject *object,
 			break;
 		}
 		case PROP_SIMPLE_BIND:
-			gda_holder_set_bind (holder, GDA_HOLDER (g_value_get_object (value)));
+			if (!gda_holder_set_bind (holder, GDA_HOLDER (g_value_get_object (value)), NULL))
+				g_warning ("Could not set the 'simple-bind' property");
 			break;
 		case PROP_FULL_BIND:
 			gda_holder_set_full_bind (holder, GDA_HOLDER (g_value_get_object (value)));
@@ -509,7 +569,7 @@ gda_holder_set_property (GObject *object,
 		case PROP_SOURCE_MODEL: {
 			GdaDataModel* ptr = g_value_get_object (value);
 			g_return_if_fail (gda_holder_set_source_model (holder, 
-									 (GdaDataModel *)ptr, -1, NULL));
+								       (GdaDataModel *)ptr, -1, NULL));
 			break;
                 }
 		case PROP_SOURCE_COLUMN:
@@ -609,7 +669,7 @@ gda_holder_get_id (GdaHolder *holder)
  * Get the value held into the holder. If @holder is set to use its default value
  * and that default value is not of the same type as @holder, then %NULL is returned.
  *
- * If @holder is set to NULL, then the returned value is a GDA_TYPE_NULL GValue.
+ * If @holder is set to NULL, then the returned value is a #GDA_TYPE_NULL GValue.
  *
  * Returns: the value, or %NULL
  */
@@ -669,31 +729,41 @@ gda_holder_get_value_str (GdaHolder *holder, GdaDataHandler *dh)
         }
 }
 
-static gboolean real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy);
+static gboolean real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, GError **error);
 
 /**
  * gda_holder_set_value
  * @holder: a #GdaHolder object
- * @value: a value to set the holder to
+ * @value: a value to set the holder to, or %NULL
+ * @error: a place to store errors, or %NULL
  *
  * Sets the value within the holder. If @holder is an alias for another
  * holder, then the value is also set for that other holder.
  *
- * The action of any call to gda_holder_force_invalid() is cancelled
- * as soon as this method is called, even if @holder's value does not change.
+ * On success, the action of any call to gda_holder_force_invalid() is cancelled
+ * as soon as this method is called (even if @holder's value does not actually change)
  * 
  * If the value is not different from the one already contained within @holder,
- * then @holder is not chaged and no signal is emitted.
+ * then @holder is not changed and no signal is emitted.
+ *
+ * Note1: the @value argument is treated the same way if it is %NULL or if it is a #GDA_TYPE_NULL value
+ *
+ * Note2: if @holder can't accept the @value value, then this method returns FALSE, and @holder will be left
+ * in an invalid state.
+ *
+ * Note3: before the change is accepted by @holder, the "validate-change" signal will be emitted (the value
+ * of which can prevent the change from happening) which can be connected to to have a greater control
+ * of which values @holder can have, or implement some business rules.
  *
  * Returns: TRUE if value has been set
  */
 gboolean
-gda_holder_set_value (GdaHolder *holder, const GValue *value)
+gda_holder_set_value (GdaHolder *holder, const GValue *value, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
 
-	return real_gda_holder_set_value (holder, (GValue*) value, TRUE);
+	return real_gda_holder_set_value (holder, (GValue*) value, TRUE, error);
 }
 
 /**
@@ -701,26 +771,27 @@ gda_holder_set_value (GdaHolder *holder, const GValue *value)
  * @holder: a #GdaHolder object
  * @dh: a #GdaDataHandler to use, or %NULL
  * @value: a value to set the holder to, as a string
+ * @error: a place to store errors, or %NULL
  *
  * Same functionality as gda_holder_set_value() except that it uses a string representation
  * of the value to set, which will be converted into a GValue first (using default data handler if
  * @dh is %NULL).
  *
- * Note that is @value is %NULL or is the "NULL" string, then @holder's value is set to %NULL.
+ * Note1: if @value is %NULL or is the "NULL" string, then @holder's value is set to %NULL.
+ * Note2: if @holder can't accept the @value value, then this method returns FALSE, and @holder will be left
+ * in an invalid state.
  *
  * Returns: TRUE if value has been set
  */
 gboolean
-gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *value)
+gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *value, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
 	g_return_val_if_fail (!dh || GDA_IS_DATA_HANDLER (dh), FALSE);
 
-	if (!value || !g_ascii_strcasecmp (value, "NULL")) {
-                gda_holder_set_value (holder, NULL);
-                return TRUE;
-        }
+	if (!value || !g_ascii_strcasecmp (value, "NULL")) 
+                return gda_holder_set_value (holder, NULL, error);
         else {
                 GValue *gdaval = NULL;
 
@@ -729,13 +800,14 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
                 if (dh)
                         gdaval = gda_data_handler_get_value_from_str (dh, value, holder->priv->g_type);
 
-                if (gdaval) {
-                        gda_holder_set_value (holder, gdaval);
-                        gda_value_free (gdaval);
-                        return TRUE;
-                }
-                else
+                if (gdaval)
+			return real_gda_holder_set_value (holder, gdaval, FALSE, error);
+                else {
+			g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_STRING_CONVERSION_ERROR,
+				     _("Unable to convert string to '%s' type"), 
+				     gda_g_type_to_string (holder->priv->g_type));
                         return FALSE;
+		}
         }
 }
 
@@ -743,31 +815,38 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
  * gda_holder_take_value
  * @holder: a #GdaHolder object
  * @value: a value to set the holder to
+ * @error: a place to store errors, or %NULL
  *
  * Sets the value within the holder. If @holder is an alias for another
  * holder, then the value is also set for that other holder.
  *
- * The action of any call to gda_holder_force_invalid() is cancelled
- * as soon as this method is called, even if @holder's value does not change.
+ * On success, the action of any call to gda_holder_force_invalid() is cancelled
+ * as soon as this method is called (even if @holder's value does not actually change).
  * 
  * If the value is not different from the one already contained within @holder,
  * then @holder is not chaged and no signal is emitted.
  *
+ * Note1: if @holder can't accept the @value value, then this method returns FALSE, and @holder will be left
+ * in an invalid state.
+ *
+ * Note2: before the change is accepted by @holder, the "validate-change" signal will be emitted (the value
+ * of which can prevent the change from happening) which can be connected to to have a greater control
+ * of which values @holder can have, or implement some business rules.
+ *
  * Returns: TRUE if value has been set
  */
 gboolean
-gda_holder_take_value (GdaHolder *holder, GValue *value)
+gda_holder_take_value (GdaHolder *holder, GValue *value, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
 
-	return real_gda_holder_set_value (holder, (GValue*) value, FALSE);
+	return real_gda_holder_set_value (holder, (GValue*) value, FALSE, error);
 }
 
 static gboolean
-real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
+real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, GError **error)
 {
-	gboolean retval = FALSE;
 	gboolean changed = TRUE;
 	gboolean newvalid;
 	const GValue *current_val;
@@ -779,8 +858,6 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 	gboolean was_valid = gda_holder_is_valid (holder);
 #endif
 
-	holder->priv->invalid_forced = FALSE;
-
 	/* holder will be changed? */
 	newnull = !value || gda_value_is_null (value);
 	current_val = gda_holder_get_value (holder);
@@ -790,16 +867,24 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 		changed = FALSE;
 	else if (value && current_val &&
 		 (G_VALUE_TYPE (value) == G_VALUE_TYPE ((GValue *)current_val)))
-		changed = gda_value_bcompare (value, (GValue *)current_val);
+		changed = gda_value_differ (value, (GValue *)current_val);
 		
 	/* holder's validity */
 	newvalid = TRUE;
-	if (newnull)
-		if (holder->priv->not_null)
-			newvalid = FALSE;
-
-	if (!newnull && (G_VALUE_TYPE (value) != holder->priv->g_type)) 
+	if (newnull && holder->priv->not_null) {
+		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_NULL_ERROR,
+			     _("Holder does not allow NULL values"));
 		newvalid = FALSE;
+		changed = TRUE;
+	}
+	else if (!newnull && (G_VALUE_TYPE (value) != holder->priv->g_type)) {
+		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
+			     _("Wrong value type: expected type '%s' when value's type is '%s'"),
+			     gda_g_type_to_string (holder->priv->g_type),
+			     gda_g_type_to_string (G_VALUE_TYPE (value)));
+		newvalid = FALSE;
+		changed = TRUE;
+	}
 
 #ifdef DEBUG_HOLDER
 	g_print ("Changed holder %p (%s): value %s --> %s \t(type %d -> %d) VALID: %d->%d CHANGED: %d\n", 
@@ -815,9 +900,23 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 	if (!changed) {
 		if (!do_copy && value)
 			gda_value_free (value);
+		holder->priv->invalid_forced = FALSE;
 		return TRUE;
 	}
 
+	/* check if we are allowed to change value */
+	GError *lerror = NULL;
+	g_signal_emit (holder, gda_holder_signals[VALIDATE_CHANGE], 0, value, &lerror);
+	if (lerror) {
+		/* change refused by signal callback */
+		g_propagate_error (error, lerror);
+		if (!do_copy) 
+			gda_value_free (value);
+		return FALSE;
+	}
+
+	/* new valid status */
+	holder->priv->invalid_forced = FALSE;
 	holder->priv->valid = newvalid;
 
 	/* check is the new value is the default one */
@@ -827,7 +926,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 			holder->priv->default_forced = TRUE;
 		else if ((G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) &&
 			 value && (G_VALUE_TYPE (value) == holder->priv->g_type))
-			holder->priv->default_forced = !gda_value_compare_ext (holder->priv->default_value, value);
+			holder->priv->default_forced = !gda_value_compare (holder->priv->default_value, value);
 	}
 
 	/* real setting of the value */
@@ -836,7 +935,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 		g_print ("Holder %p is alias of holder %p => propagating changes to holder %p\n",
 			 holder, holder->priv->full_bind, holder->priv->full_bind);
 #endif
-		gda_holder_set_value (holder->priv->full_bind, value);
+		return real_gda_holder_set_value (holder->priv->full_bind, value, do_copy, error);
 	}
 	else {
 		if (holder->priv->value) {
@@ -845,17 +944,20 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy)
 		}
 
 		if (value) {
-			retval = TRUE;
-			if (do_copy)
-				holder->priv->value = gda_value_copy (value);
-			else
-				holder->priv->value = value;
+			if (newvalid) {
+				if (do_copy)
+					holder->priv->value = gda_value_copy (value);
+				else
+					holder->priv->value = value;
+			}
+			else if (!do_copy) 
+				gda_value_free (value);
 		}
 
 		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
 	}
 
-	return retval;
+	return newvalid;
 }
 
 /**
@@ -998,7 +1100,7 @@ gda_holder_get_default_value (GdaHolder *holder)
  * @value: a value to set the holder's default value, or %NULL
  *
  * Sets the default value within the holder. If @value is %NULL then @holder won't have a
- * default value anymore. To set a default value to %NULL, then pass GValue created using
+ * default value anymore. To set a default value to %NULL, then pass a #GValue created using
  * gda_value_new_null().
  *
  * NOTE: the default value does not need to be of the same type as the one required by @holder.
@@ -1011,7 +1113,7 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 
 	if (holder->priv->default_value) {
 		if (holder->priv->default_forced) {
-			gda_holder_take_value (holder, holder->priv->default_value);
+			gda_holder_take_value (holder, holder->priv->default_value, NULL);
 			holder->priv->default_forced = FALSE;
 			holder->priv->default_value = NULL;
 		}
@@ -1030,7 +1132,7 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 		    (!current || gda_value_is_null (current)))
 			holder->priv->default_forced = TRUE;
 		else if ((G_VALUE_TYPE (value) == holder->priv->g_type) &&
-			 current && !gda_value_compare_ext (value, current))
+			 current && !gda_value_compare (value, current))
 			holder->priv->default_forced = TRUE;
 
 		holder->priv->default_value = gda_value_copy ((GValue *)value);
@@ -1052,7 +1154,7 @@ gda_holder_set_not_null (GdaHolder *holder, gboolean not_null)
 	g_return_if_fail (GDA_IS_HOLDER (holder));
 	g_return_if_fail (holder->priv);
 
-	g_object_set (G_OBJECT (holder), "not_null", not_null, NULL);
+	g_object_set (G_OBJECT (holder), "not-null", not_null, NULL);
 }
 
 /**
@@ -1155,25 +1257,29 @@ gda_holder_get_source_model (GdaHolder *holder, gint *col)
  *
  * If @bind_to is %NULL, then @holder will not be bound anymore.
  */
-void
-gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to)
+gboolean
+gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to, GError **error)
 {
 	const GValue *cvalue;
 	GValue *value1 = NULL;
 	const GValue *value2 = NULL;
 
-	g_return_if_fail (GDA_IS_HOLDER (holder));
-	g_return_if_fail (holder->priv);
-	g_return_if_fail (holder != bind_to);
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
+	g_return_val_if_fail (holder->priv, FALSE);
+	g_return_val_if_fail (holder != bind_to, FALSE);
 
 	if (holder->priv->simple_bind == bind_to)
-		return;
+		return TRUE;
 
 	/* get a copy of the current values of @holder and @bind_to */
 	if (bind_to) {
-		g_return_if_fail (GDA_IS_HOLDER (bind_to));
-		g_return_if_fail (bind_to->priv);
-		g_return_if_fail (holder->priv->g_type == bind_to->priv->g_type);
+		g_return_val_if_fail (GDA_IS_HOLDER (bind_to), FALSE);
+		g_return_val_if_fail (bind_to->priv, FALSE);
+		if (holder->priv->g_type != bind_to->priv->g_type) {
+			g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
+				     _("Cannot bind holders if their type is not the same"));
+			return FALSE;
+		}
 		value2 = gda_holder_get_value (bind_to);
 	}
 
@@ -1197,12 +1303,12 @@ gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to)
 				  G_CALLBACK (full_bind_changed_cb), holder);
 
 		/* if bind_to has a different value than holder, then we set holder to the new value */
-		gda_holder_set_value (holder, value2);
 		if (value1)
 			gda_value_free (value1);
+		return gda_holder_set_value (holder, value2, error);
 	}
 	else
-		gda_holder_take_value (holder, value1);
+		return gda_holder_take_value (holder, value1, error);
 }
 
 /*
