@@ -37,6 +37,7 @@
 #include "gda-util.h"
 
 extern xmlDtdPtr gda_paramlist_dtd;
+extern gchar *gda_lang_locale;
 
 /* 
  * Main static functions 
@@ -50,6 +51,7 @@ static void set_remove_node (GdaSet *set, GdaSetNode *node);
 static void set_remove_source (GdaSet *set, GdaSetSource *source);
 
 static void changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
+static GError *validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *dataset);
 static void source_changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
 static void notify_holder_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset);
 
@@ -76,10 +78,12 @@ enum
 	PUBLIC_DATA_CHANGED,
 	HOLDER_PLUGIN_CHANGED,
 	HOLDER_ATTR_CHANGED,
+	VALIDATE_HOLDER_CHANGE,
+	VALIDATE_SET,
 	LAST_SIGNAL
 };
 
-static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
+static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0};
 
 
 /* private structure */
@@ -193,6 +197,31 @@ gda_set_get_type (void)
 	return type;
 }
 
+static gboolean
+validate_accumulator (GSignalInvocationHint *ihint,
+				  GValue *return_accu,
+				  const GValue *handler_return,
+				  gpointer data)
+{
+	GError *error;
+
+	error = g_value_get_pointer (handler_return);
+	g_value_set_pointer (return_accu, error);
+
+	return error ? FALSE : TRUE; /* stop signal if 'thisvalue' is FALSE */
+}
+
+static GError *
+m_validate_holder_change (GdaSet *set, GdaHolder *holder, const GValue *new_value)
+{
+	return NULL;
+}
+
+static GError *
+m_validate_set (GdaSet *set)
+{
+	return NULL;
+}
 
 static void
 gda_set_class_init (GdaSetClass *class)
@@ -202,15 +231,53 @@ gda_set_class_init (GdaSetClass *class)
 	parent_class = g_type_class_peek_parent (class);
 
 	gda_set_signals[HOLDER_CHANGED] =
-		g_signal_new ("holder_changed",
+		g_signal_new ("holder-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, holder_changed),
 			      NULL, NULL,
 			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 			      GDA_TYPE_HOLDER);
+
+	/**
+	 * GdaSet::validate-holder-change:
+	 * @set: the object which received the signal
+	 * @holder: the #GdaHolder which is going to change
+	 * @new_value: the proposed new value for @holder
+	 * 
+	 * Gets emitted when a #GdaHolder's in @set is going to change its value. One can connect to
+	 * this signal to control which values @holder can have (for example to implement some business rules)
+	 *
+	 * Return value: NULL if @holder is allowed to change its value to @new_value, or a #GError
+	 * otherwise.
+	 */
+	gda_set_signals[VALIDATE_HOLDER_CHANGE] =
+		g_signal_new ("validate-holder-change",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GdaSetClass, validate_holder_change),
+			      validate_accumulator, NULL,
+			      gda_marshal_POINTER__OBJECT_POINTER, G_TYPE_POINTER, 2,
+			      GDA_TYPE_HOLDER, G_TYPE_POINTER);
+	/**
+	 * GdaSet::validate-set:
+	 * @set: the object which received the signal
+	 * 
+	 * Gets emitted when gda_set_is_valid() is called, use
+	 * this signal to control which combination of values @set's holder can have (for example to implement some business rules)
+	 *
+	 * Return value: NULL if @set's contents has been validated, or a #GError
+	 * otherwise.
+	 */
+	gda_set_signals[VALIDATE_SET] =
+		g_signal_new ("validate-set",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GdaSetClass, validate_set),
+			      validate_accumulator, NULL,
+			      gda_marshal_POINTER__VOID, G_TYPE_POINTER, 0);
 	gda_set_signals[HOLDER_PLUGIN_CHANGED] =
-		g_signal_new ("holder_plugin_changed",
+		g_signal_new ("holder-plugin-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, holder_plugin_changed),
@@ -218,7 +285,7 @@ gda_set_class_init (GdaSetClass *class)
 			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 			      GDA_TYPE_HOLDER);
 	gda_set_signals[HOLDER_ATTR_CHANGED] =
-		g_signal_new ("holder_attr_changed",
+		g_signal_new ("holder-attr-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, holder_attr_changed),
@@ -226,7 +293,7 @@ gda_set_class_init (GdaSetClass *class)
 			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 			      GDA_TYPE_HOLDER);
 	gda_set_signals[PUBLIC_DATA_CHANGED] =
-		g_signal_new ("public_data_changed",
+		g_signal_new ("public-data-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaSetClass, public_data_changed),
@@ -234,6 +301,8 @@ gda_set_class_init (GdaSetClass *class)
 			      gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->holder_changed = NULL;
+	class->validate_holder_change = m_validate_holder_change;
+	class->validate_set = m_validate_set;
 	class->holder_plugin_changed = NULL;
 	class->holder_attr_changed = NULL;
 	class->public_data_changed = NULL;
@@ -312,6 +381,7 @@ gda_set_copy (GdaSet *set)
 
 	copy = g_object_new (GDA_TYPE_SET, "holders", holders, NULL);
 	g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
+	g_slist_free (holders);
 
 	return copy;
 }
@@ -334,11 +404,12 @@ gda_set_copy (GdaSet *set)
 GdaSet *
 gda_set_new_inline (gint nb, ...)
 {
-	GdaSet *set;
+	GdaSet *set = NULL;
 	GSList *holders = NULL;
 	va_list ap;
 	gchar *id;
 	gint i;
+	gboolean allok = TRUE;
 
 	/* build the list of holders */
 	va_start (ap, nb);
@@ -346,10 +417,11 @@ gda_set_new_inline (gint nb, ...)
 		GType type;
 		GdaHolder *holder;
 		GValue *value;
+		GError *lerror = NULL;
 
 		id = va_arg (ap, char *);
 		type = va_arg (ap, GType);
-		holder = (GdaHolder *) g_object_new (GDA_TYPE_HOLDER, "g_type", type, "id", id, NULL);
+		holder = (GdaHolder *) g_object_new (GDA_TYPE_HOLDER, "g-type", type, "id", id, NULL);
 
 		value = gda_value_new (type);
 		if (type == G_TYPE_BOOLEAN) 
@@ -388,29 +460,41 @@ gda_set_new_inline (gint nb, ...)
 			g_value_set_long (value, va_arg (ap, glong));
 		else if (type == G_TYPE_ULONG)
 			g_value_set_ulong (value, va_arg (ap, gulong));
-		else
-			g_warning (_("%s() does not handle values of type %s, value not assigned."),
+		else {
+			g_warning (_("%s() does not handle values of type '%s'."),
 				   __FUNCTION__, g_type_name (type));
+			g_object_unref (holder);
+			allok = FALSE;
+			break;
+		}
 
-		if (!gda_holder_take_value (holder, value))
-			g_warning (_("Could not set GdaHolder's value, value not assigned"));
+		if (!gda_holder_take_value (holder, value, &lerror)) {
+			g_warning (_("Unable to set holder's value: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+			if (lerror)
+				g_error_free (lerror);
+			g_object_unref (holder);
+			allok = FALSE;
+			break;
+		}
 		holders = g_slist_append (holders, holder);
         }
 	va_end (ap);
 
 	/* create the set */
-	set = gda_set_new (holders);
+	if (allok) 
+		set = gda_set_new (holders);
 	if (holders) {
 		g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
 		g_slist_free (holders);
 	}
-
 	return set;
 }
 
 /**
  * gda_set_set_holder_value
  * @set: a #GdaSet object
+ * @error: a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
  * @...: value, of the correct type, depending on the requested holder's type
  *
@@ -419,7 +503,7 @@ gda_set_new_inline (gint nb, ...)
  * Returns: TRUE if no error occurred and the value was set correctly
  */
 gboolean
-gda_set_set_holder_value (GdaSet *set, const gchar *holder_id, ...)
+gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, ...)
 {
 	GdaHolder *holder;
 	va_list ap;
@@ -431,7 +515,8 @@ gda_set_set_holder_value (GdaSet *set, const gchar *holder_id, ...)
 
 	holder = gda_set_get_holder (set, holder_id);
 	if (!holder) {
-		g_warning (_("GdaHolder with ID '%s' not found in set"), holder_id);
+		g_set_error (error, GDA_SET_ERROR, GDA_SET_HOLDER_NOT_FOUND_ERROR,
+			     _("GdaHolder with ID '%s' not found in set"), holder_id);
 		return FALSE;
 	}
 	type = gda_holder_get_g_type (holder);
@@ -473,16 +558,16 @@ gda_set_set_holder_value (GdaSet *set, const gchar *holder_id, ...)
 		g_value_set_long (value, va_arg (ap, glong));
 	else if (type == G_TYPE_ULONG)
 		g_value_set_ulong (value, va_arg (ap, gulong));
-	else
-		g_warning (_("%s() does not handle values of type %s, value not assigned."),
-			   __FUNCTION__, g_type_name (type));
-	
-	va_end (ap);
-	if (!gda_holder_take_value (holder, value)) {
-		g_warning (_("Could not set GdaHolder's value, value not assigned"));
+	else {
+		g_set_error (error, 0, 0,
+			     _("%s() does not handle values of type '%s'."),
+			     __FUNCTION__, g_type_name (type));
+		va_end (ap);
 		return FALSE;
 	}
-	return TRUE;
+
+	va_end (ap);
+	return gda_holder_take_value (holder, value, error);
 }
 
 /**
@@ -640,18 +725,12 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 	GdaSet *set = NULL;
 	GSList *holders = NULL, *sources = NULL;
 	GSList *list;
-	const gchar *lang;
+	const gchar *lang = gda_lang_locale;
 
 	xmlNodePtr cur;
 	gboolean allok = TRUE;
 	gchar *str;
 
-#ifdef HAVE_LC_MESSAGES
-	lang = setlocale (LC_MESSAGES, NULL);
-#else
-	lang = setlocale (LC_CTYPE, NULL);
-#endif
-
 	if (strcmp ((gchar*)xml_spec->name, "parameters") != 0){
 		g_set_error (error, GDA_SET_ERROR, GDA_SET_XML_SPEC_ERROR,
 			     _("Missing node <parameters>: '%s'"), xml_spec->name);
@@ -730,15 +809,17 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 
 			if (!holder) {
 				holder = (GdaHolder*) (g_object_new (GDA_TYPE_HOLDER,
-								     "g_type", 
+								     "g-type", 
 								     gdatype ? gda_g_type_from_string ((gchar *) gdatype) : G_TYPE_STRING,
 								     NULL));
 				holders = g_slist_append (holders, holder);
 			}
-			if (gdatype) xmlFree (gdatype);
+			if (gdatype)
+				xmlFree (gdatype);
 			
 			/* set holder's attributes */
-			gda_utility_holder_load_attributes (holder, cur, sources);
+			if (! gda_utility_holder_load_attributes (holder, cur, sources, error))
+				allok = FALSE;
 		}
 	}
 
@@ -819,7 +900,6 @@ gda_set_get_spec (GdaSet *set)
 		xmlNodePtr node;
 		GdaHolder *holder = GDA_HOLDER (list->data);
 		gchar *str;
-		const gchar *cstr;
 
 		node = xmlNewTextChild (root, NULL, (xmlChar*)"parameter", NULL);
 		g_object_get (G_OBJECT (holder), "id", &str, NULL);
@@ -829,7 +909,7 @@ gda_set_get_spec (GdaSet *set)
 
 		g_object_get (G_OBJECT (holder), "name", &str, NULL);
 		if (str)
-			xmlSetProp(node, (xmlChar*)"name", (xmlChar*)cstr);
+			xmlSetProp(node, (xmlChar*)"name", (xmlChar*)str);
 		g_free (str);
 
 		g_object_get (G_OBJECT (holder), "description", &str, NULL);
@@ -869,6 +949,8 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 					      G_CALLBACK (changed_holder_cb), set);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+					      G_CALLBACK (validate_change_holder_cb), set);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 					      G_CALLBACK (source_changed_holder_cb), set);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
@@ -922,6 +1004,21 @@ notify_holder_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *set)
 	}
 }
 
+static GError *
+validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *set)
+{
+	/* signal the holder validate-change */
+	GError *error = NULL;
+#ifdef GDA_DEBUG_signal
+	g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_HOLDER_CHANGE], 0, holder, value, &error);
+#ifdef GDA_DEBUG_signal
+	g_print ("<< 'VALIDATE_HOLDER_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	return error;
+}
+
 static void
 changed_holder_cb (GdaHolder *holder, GdaSet *set)
 {
@@ -957,6 +1054,8 @@ gda_set_dispose (GObject *object)
 		for (list = set->holders; list; list = list->next) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
 				G_CALLBACK (changed_holder_cb), set);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+				G_CALLBACK (validate_change_holder_cb), set);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
 				G_CALLBACK (source_changed_holder_cb), set);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
@@ -1235,6 +1334,7 @@ static gboolean
 gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 {
 	GdaHolder *similar;
+	const gchar *hid;
 
 	if (g_slist_find (set->holders, holder))
 		return FALSE;
@@ -1243,15 +1343,23 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 	 * try to find a similar holder in the set->holders:
 	 * a holder B is similar to a holder A if it has the same ID
 	 */
-	similar = (GdaHolder*) g_hash_table_lookup (set->priv->holders_hash, gda_holder_get_id (holder));
+	hid = gda_holder_get_id (holder);
+	if (!hid) {
+		g_warning (_("GdaHolder needs to have an ID"));
+		return FALSE;
+	}
+	g_return_val_if_fail (hid, FALSE);
+	similar = (GdaHolder*) g_hash_table_lookup (set->priv->holders_hash, hid);
 	if (!similar) {
 		/* really add @holder to the set */
 		set->holders = g_slist_append (set->holders, holder);
-		g_hash_table_insert (set->priv->holders_hash, (gchar*) gda_holder_get_id (holder), holder);
+		g_hash_table_insert (set->priv->holders_hash, (gchar*) hid, holder);
 		g_object_ref (holder);
 		g_signal_connect (G_OBJECT (holder), "changed",
 				  G_CALLBACK (changed_holder_cb), set);
-		g_signal_connect (G_OBJECT (holder), "source_changed",
+		g_signal_connect (G_OBJECT (holder), "validate-change",
+				  G_CALLBACK (validate_change_holder_cb), set);
+		g_signal_connect (G_OBJECT (holder), "source-changed",
 				  G_CALLBACK (source_changed_holder_cb), set);
 		g_signal_connect (G_OBJECT (holder), "notify",
 				  G_CALLBACK (notify_holder_cb), set);
@@ -1316,13 +1424,18 @@ set_remove_source (GdaSet *set, GdaSetSource *source)
 /**
  * gda_set_is_valid
  * @set: a #GdaSet object
+ * @error: a place to store validation errors, or %NULL
  *
- * Tells if all the set's holders have valid data
+ * This method tells if all @set's #GdaHolder objects are valid, and if
+ * they represent a valid combination of values, as defined by rules
+ * external to Libgda: the "validate-set" signal is emitted and if none of the signal handlers return an
+ * error, then the returned value is TRUE, otherwise the return value is FALSE as soon as a signal handler
+ * returns an error.
  *
  * Returns: TRUE if the set is valid
  */
 gboolean
-gda_set_is_valid (GdaSet *set)
+gda_set_is_valid (GdaSet *set, GError **error)
 {
 	GSList *holders;
 	gboolean retval = TRUE;
@@ -1331,9 +1444,28 @@ gda_set_is_valid (GdaSet *set)
 	g_return_val_if_fail (set->priv, FALSE);
 
 	for (holders = set->holders; holders && retval; holders = g_slist_next (holders)) {
-		if (!gda_holder_is_valid ((GdaHolder*) holders->data)) 
+		if (!gda_holder_is_valid ((GdaHolder*) holders->data)) {
+			g_set_error (error, GDA_SET_ERROR, GDA_SET_INVALID_ERROR,
+			     _("One or more values are invalid"));
 			retval = FALSE;
+		}
 		
+	if (retval) {
+		/* signal the holder validate-set */
+		GError *lerror = NULL;
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'VALIDATE_SET' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_SET], 0, &lerror);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'VALIDATE_SET' from %s\n", __FUNCTION__);
+#endif
+		if (lerror) {
+			g_propagate_error (error, lerror);
+			retval = FALSE;
+		}
+	}
+
 #ifdef GDA_DEBUG_NO
 		g_print ("== HOLDER %p: valid= %d, value=%s\n", holders->data, gda_holder_is_valid (GDA_HOLDER (holders->data)),
 			 gda_holder_get_value (GDA_HOLDER (holders->data)) ?
