@@ -39,15 +39,9 @@ extern GQuark gda_set_error_quark (void);
 
 typedef enum
 {
-	/*GDA_SET_NO_NODE_ERROR,
-	GDA_SET_NODE_OUTDATED_ERROR,
-	GDA_SET_VALUE_PROV_OBJ_TYPE_ERROR,
-	GDA_SET_VALUE_PROV_DATA_TYPE_ERROR,
-	GDA_SET_VALUE_PROV_INVISIBLE_ERROR,
-	GDA_SET_VALUE_PROV_QUERY_TYPE_ERROR,
-	GDA_SET_DEPENDENCY_NOT_IN_SET_ERROR,
-	GDA_SET_DEPENDENCY_POSITION_ERROR,*/
-	GDA_SET_XML_SPEC_ERROR
+	GDA_SET_XML_SPEC_ERROR,
+	GDA_SET_HOLDER_NOT_FOUND_ERROR,
+	GDA_SET_INVALID_ERROR
 } GdaSetError;
 
 typedef enum {
@@ -124,6 +118,8 @@ struct _GdaSetClass
 {
 	GObjectClass            parent_class;
 
+	GError               *(*validate_holder_change)(GdaSet *set, GdaHolder *holder, const GValue *new_value);
+	GError               *(*validate_set)          (GdaSet *set);
 	void                  (*holder_changed)        (GdaSet *set, GdaHolder *holder);
 	void                  (*holder_plugin_changed) (GdaSet *set, GdaHolder *holder);
 	void                  (*holder_attr_changed)   (GdaSet *set, GdaHolder *holder);
@@ -139,13 +135,13 @@ GdaSet       *gda_set_new_from_spec_string     (const gchar *xml_spec, GError **
 GdaSet       *gda_set_new_from_spec_node       (xmlNodePtr xml_spec, GError **error);
 gchar        *gda_set_get_spec                 (GdaSet *set);
 
-gboolean      gda_set_set_holder_value         (GdaSet *set, const gchar *holder_id, ...);
+gboolean      gda_set_set_holder_value         (GdaSet *set, GError **error, const gchar *holder_id, ...);
 const GValue *gda_set_get_holder_value         (GdaSet *set, const gchar *holder_id);
 GdaHolder    *gda_set_get_holder               (GdaSet *set, const gchar *holder_id);
 gboolean      gda_set_add_holder               (GdaSet *set, GdaHolder *holder);
 void          gda_set_remove_holder            (GdaSet *set, GdaHolder *holder);
 void          gda_set_merge_with_set           (GdaSet *set, GdaSet *set_to_merge);
-gboolean      gda_set_is_valid                 (GdaSet *set);
+gboolean      gda_set_is_valid                 (GdaSet *set, GError **error);
 
 /* public data lookup functions */
 GdaSetNode   *gda_set_get_node                 (GdaSet *set, GdaHolder *param);
