@@ -477,8 +477,6 @@ get_params_foreach_func (GdaSqlAnyPart *node, GdaSet **params, GError **error)
 	    (pspec = ((GdaSqlExpr*) node)->param_spec)) {
 		GdaHolder *h;
 
-		if ((pspec->g_type == 0) && pspec->type)
-			pspec->g_type = gda_g_type_from_string (pspec->type);
 		if (pspec->g_type == 0) {
 			g_set_error (error, GDA_STATEMENT_ERROR, GDA_STATEMENT_PARAM_TYPE_ERROR,
 				     _("Could not determine GType for parameter '%s'"),
@@ -725,7 +723,7 @@ gda_statement_to_sql_extended (GdaStatement *stmt, GdaConnection *cnc, GdaSet *p
 	context.flags = flags;
 	if (cnc) {
 		context.cnc = cnc;
-		context.provider = gda_connection_get_provider_obj (cnc);
+		context.provider = gda_connection_get_provider (cnc);
 	}
 
 	str = gda_statement_to_sql_real (stmt, &context, error);
@@ -1246,13 +1244,13 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 			
 			g_string_append (string, " /* ");
 			g_string_append_printf (string, "name:%s", quoted_pname);
-			if (pspec->type) {
-				str = _add_quotes (pspec->type);
+			if (pspec->g_type) {
+				str = _add_quotes (gda_g_type_to_string (pspec->g_type));
 				g_string_append_printf (string, " type:%s", str);
 				g_free (str);
 			}
 			if (pspec->descr) {
-				str = _add_quotes (pspec->type);
+				str = _add_quotes (pspec->descr);
 				g_string_append_printf (string, " descr:%s", str);
 				g_free (str);
 			}
@@ -1264,9 +1262,9 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 		else {
 			g_string_append (string, "##");
 			g_string_append (string, pspec->name);
-			if (pspec->type) {
+			if (pspec->g_type != G_TYPE_INVALID) {
 				g_string_append (string, "::");
-				g_string_append (string, pspec->type);
+				g_string_append (string, gda_g_type_to_string (pspec->g_type));
 				if (pspec->nullok) 
 					g_string_append (string, "::NULL");
 			}
@@ -1464,7 +1462,7 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 	sql_list = g_slist_reverse (sql_list);
 
 	str = NULL;
-	switch (op->operator) {
+	switch (op->operator_type) {
 	case GDA_SQL_OPERATOR_TYPE_EQ:
 		if (SQL_OPERAND (sql_list->next->data)->is_null) 
 			str = g_strdup_printf ("%s IS NULL", SQL_OPERAND (sql_list->data)->sql);
@@ -1544,7 +1542,7 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 			add_p = FALSE;
 
 		string = g_string_new (SQL_OPERAND (sql_list->data)->sql);
-		if (op->operator == GDA_SQL_OPERATOR_TYPE_IN)
+		if (op->operator_type == GDA_SQL_OPERATOR_TYPE_IN)
 			g_string_append (string, " IN ");
 		else
 			g_string_append (string, " NOT IN ");
@@ -1753,8 +1751,8 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 	g_return_val_if_fail (GDA_SQL_ANY_PART (join)->type == GDA_SQL_ANY_SQL_SELECT_JOIN, NULL);
 
 	/* can't have: 
-	 *  - join->expr && join->using 
-	 *  - (join->type == GDA_SQL_SELECT_JOIN_CROSS) && (join->expr || join->using)
+	 *  - join->expr && join->use 
+	 *  - (join->type == GDA_SQL_SELECT_JOIN_CROSS) && (join->expr || join->use)
 	 */
 	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (join), error)) return NULL;
 
@@ -1809,11 +1807,11 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 		g_free (str);
 		g_string_append_c (string, ')');
 	}
-	else if (join->using) {
+	else if (join->use) {
 		GSList *list;
 		g_string_append (string, " USING (");
-		for (list = join->using; list; list = list->next) {
-			if (list != join->using)
+		for (list = join->use; list; list = list->next) {
+			if (list != join->use)
 				g_string_append (string, ", ");
 			str = context->render_field (GDA_SQL_ANY_PART (list->data), context, error);
 			if (!str) goto err;
