@@ -60,7 +60,8 @@ typedef enum {
         GDA_STATEMENT_SQL_PARAMS_SHORT       = 1 << 2,
         GDA_STATEMENT_SQL_PARAMS_AS_COLON    = 1 << 3,
         GDA_STATEMENT_SQL_PARAMS_AS_DOLLAR   = 1 << 4,
-        GDA_STATEMENT_SQL_PARAMS_AS_QMARK    = 1 << 5 
+        GDA_STATEMENT_SQL_PARAMS_AS_QMARK    = 1 << 5,
+        GDA_STATEMENT_SQL_PARAMS_AS_UQMARK   = 1 << 6
 } GdaStatementSqlFlag;
 
 /* struct for the object's data */
@@ -96,13 +97,8 @@ gchar              *gda_statement_to_sql_extended        (GdaStatement *stmt, Gd
 GdaSqlStatementType gda_statement_get_statement_type     (GdaStatement *stmt);
 gboolean            gda_statement_is_useless             (GdaStatement *stmt);
 gboolean            gda_statement_check_structure        (GdaStatement *stmt, GError **error);
-gboolean            gda_statement_check_connection       (GdaStatement *stmt, GdaConnection *cnc, GError **error);
-
-/*
-GdaObject         *gda_statement_execute              (GdaStatement *stmt, GdaConnection  *cnc, 
-						       GdaParameterList *params,
-						       GdaStatementModelUsage usage, GError **error);
-*/
+gboolean            gda_statement_check_validity         (GdaStatement *stmt, GdaConnection *cnc, GError **error);
+gboolean            gda_statement_normalize              (GdaStatement *stmt, GdaConnection *cnc, GError **error);
 
 G_END_DECLS
 
