@@ -63,58 +63,38 @@ set_from_string (GValue *value, const gchar *as_string)
 	type = G_VALUE_TYPE (value);
 	g_value_reset (value);
 
-	if (g_value_type_transformable (G_TYPE_STRING, type)) {
-		/* use the GLib type transformation function */
-		GValue *string;
-
-                string = g_new0 (GValue, 1);
-                g_value_init (string, G_TYPE_STRING);
-                g_value_set_string (string, as_string);
-
-                g_value_transform (string, value);
-                gda_value_free (string);
-
-                return TRUE;
-	}
-
 	/* custom transform function */
 	retval = FALSE;
 	if (type == G_TYPE_BOOLEAN) {
-		if (g_ascii_strcasecmp (as_string, "true") == 0) {
+		if (((as_string[0] == 't') || (as_string[0] == 'T')) &&
+		    ((as_string[1] == 'r') || (as_string[1] == 'R')) &&
+		    ((as_string[2] == 'u') || (as_string[2] == 'U')) &&
+		    ((as_string[3] == 'e') || (as_string[3] == 'E'))) {
 			g_value_set_boolean (value, TRUE);
 			retval = TRUE;
 		}
-		else {
-			if (g_ascii_strcasecmp (as_string, "false") == 0) {
-				g_value_set_boolean (value, FALSE);
-				retval = TRUE;
-			}
+		else if (((as_string[0] == 'f') || (as_string[0] == 'F')) &&
+			 ((as_string[1] == 'a') || (as_string[1] == 'A')) &&
+			 ((as_string[2] == 'l') || (as_string[2] == 'L')) &&
+			 ((as_string[3] == 's') || (as_string[3] == 'S')) &&
+			 ((as_string[4] == 'e') || (as_string[4] == 'E'))) {
+			g_value_set_boolean (value, FALSE);
+			retval = TRUE;
 		}
 	}
-	else if (type == GDA_TYPE_BINARY) {
-		GdaBinary binary;
-		retval = gda_string_to_binary (as_string, &binary);
-		if (retval)
-			gda_value_set_binary (value, &binary);
-	}
-	else if (type == GDA_TYPE_BLOB) {
-		GdaBlob blob;
-		retval = gda_string_to_blob (as_string, &blob);
-		if (retval)
-			gda_value_set_blob (value, &blob);
-	}
 	else if (type == G_TYPE_INT64) {
-		/* Use g_strtod instead of strtoll */
-		dvalue = g_strtod (as_string, endptr);
+		gint64 i64;
+		i64 = g_ascii_strtoll (as_string, endptr, 10);
 		if (*as_string != '\0' && **endptr == '\0'){
-			g_value_set_int64 (value, (gint64) dvalue);
+			g_value_set_int64 (value, i64);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_UINT64) {
- 	        dvalue = g_strtod (as_string, endptr);
-                if (*as_string!=0 && **endptr==0) {
-			g_value_set_uint64(value,(guint64)dvalue);
+		guint64 ui64;
+		ui64 = g_ascii_strtoull (as_string, endptr, 10);
+		if (*as_string != '\0' && **endptr == '\0'){
+			g_value_set_uint64 (value, ui64);
 			retval = TRUE;
                 }
 	}
@@ -147,28 +127,28 @@ set_from_string (GValue *value, const gchar *as_string)
 		}
 	}
 	else if (type == G_TYPE_CHAR) {
-		lvalue = strtol(as_string, endptr, 10);
+		lvalue = strtol (as_string, endptr, 10);
 		if (*as_string!=0 && **endptr==0) {
 			g_value_set_char(value,(gchar)lvalue);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_UCHAR) {
-		ulvalue = strtoul(as_string,endptr, 10);
+		ulvalue = strtoul (as_string,endptr, 10);
 		if (*as_string!=0 && **endptr==0) {
 			g_value_set_uchar(value,(guchar)ulvalue);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_FLOAT) {
-		dvalue = g_strtod (as_string, endptr);
+		dvalue = g_ascii_strtod (as_string, endptr);
 		if (*as_string != '\0' && **endptr == '\0'){
 			g_value_set_float (value, (gfloat) dvalue);
 			retval = TRUE;
 		}
 	}
 	else if (type == G_TYPE_DOUBLE) {
-		dvalue = g_strtod (as_string, endptr);
+		dvalue = g_ascii_strtod (as_string, endptr);
 		if (*as_string != '\0' && **endptr == '\0'){
 			g_value_set_double (value, dvalue);
 			retval = TRUE;
@@ -176,7 +156,7 @@ set_from_string (GValue *value, const gchar *as_string)
 	}
 	else if (type == GDA_TYPE_NUMERIC) {
 		GdaNumeric numeric;
-		/* FIXME: what test whould i do for numeric? */
+		/* FIXME: what test whould i do for numeric? Use GMP (http://gmplib.org/) ?*/
 		numeric.number = g_strdup (as_string);
 		numeric.precision = 0; /* FIXME */
 		numeric.width = 0; /* FIXME */
@@ -187,14 +167,28 @@ set_from_string (GValue *value, const gchar *as_string)
 	else if (type == G_TYPE_DATE) {
 		GDate *gdate;
 		gdate = g_date_new ();
-		g_date_set_parse (gdate, as_string);
-		if (g_date_valid (gdate)) {
+
+		if (gda_parse_iso8601_date (gdate, as_string)) {
 			g_value_take_boxed (value, gdate);
 			retval = TRUE;
 		}
 		else
 			g_date_free (gdate);
 	}
+	else if (type == GDA_TYPE_TIME) {
+		GdaTime timegda;
+		if (gda_parse_iso8601_time (&timegda, as_string)) {
+			gda_value_set_time (value, &timegda);
+			retval = TRUE;
+		}
+	}
+	else if (type == GDA_TYPE_TIMESTAMP) {
+		GdaTimestamp timestamp;
+		if (gda_parse_iso8601_timestamp (&timestamp, as_string)) {
+			gda_value_set_timestamp (value, &timestamp);
+			retval = TRUE;
+		}
+	}
 	else if (type == GDA_TYPE_NULL) {
 		gda_value_set_null (value);
 		retval = TRUE;
@@ -212,6 +206,33 @@ set_from_string (GValue *value, const gchar *as_string)
 			}
 		}
 	}
+	else if (type == GDA_TYPE_BINARY) {
+		GdaBinary binary;
+		retval = gda_string_to_binary (as_string, &binary);
+		if (retval)
+			gda_value_set_binary (value, &binary);
+	}
+	else if (type == GDA_TYPE_BLOB) {
+		GdaBlob blob;
+		retval = gda_string_to_blob (as_string, &blob);
+		if (retval)
+			gda_value_set_blob (value, &blob);
+	}
+
+
+	if (!retval && g_value_type_transformable (G_TYPE_STRING, type)) {
+		/* use the GLib type transformation function */
+		GValue *string;
+
+                string = g_new0 (GValue, 1);
+                g_value_init (string, G_TYPE_STRING);
+                g_value_set_string (string, as_string);
+
+                g_value_transform (string, value);
+                gda_value_free (string);
+
+                retval = TRUE;
+	}
 
 	return retval;
 }
@@ -1145,7 +1166,12 @@ gda_value_new_timestamp_from_timet (time_t val)
  * @as_string: stringified representation of the value.
  * @type: the new value type.
  *
- * Makes a new #GValue of type @type from its string representation.
+ * Makes a new #GValue of type @type from its string representation. 
+ *
+ * For more information
+ * about the string format, see the gda_value_set_from_string() function.
+ * This function is typically used when reading configuration files or other non-user input that should be locale 
+ * independent.
  *
  * Returns: the newly created #GValue or %NULL if the string representation
  * cannot be converted to the specified @type.
@@ -1172,9 +1198,14 @@ gda_value_new_from_string (const gchar *as_string, GType type)
  *
  * Creates a GValue from an XML representation of it. That XML
  * node corresponds to the following string representation:
- *    &lt;value type="gdatype"&gt;value&lt;/value&gt;
+ * &lt;value type="gdatype"&gt;value&lt;/value&gt;
  *
- * Returns:  the newly created #GValue.
+ * For more information
+ * about the string format, see the gda_value_set_from_string() function.
+ * This function is typically used when reading configuration files or other non-user input that should be locale 
+ * independent.
+ *
+ * Returns: the newly created #GValue.
  */
 GValue *
 gda_value_new_from_xml (const xmlNodePtr node)
@@ -1666,6 +1697,18 @@ gda_value_set_timestamp (GValue *value, const GdaTimestamp *val)
  *
  * Stores the value data from its string representation as @type.
  *
+ * The accepted formats are:
+ * <itemizedlist>
+ *   <listitem><para>G_TYPE_BOOLEAN: a caseless comparison is made with "true" or "false"</para></listitem>
+ *   <listitem><para>numerical types: C locale format (dot as a fraction separator)</para></listitem>
+ *   <listitem><para>G_TYPE_DATE: see <link linkend="gda-parse-iso8601-date">gda_parse_iso8601_date()</link></para></listitem>
+ *   <listitem><para>GDA_TYPE_TIME: see <link linkend="gda-parse-iso8601-time">gda_parse_iso8601_time()</link></para></listitem>
+ *   <listitem><para>GDA_TYPE_TIMESTAMP: see <link linkend="gda-parse-iso8601-timestamp">gda_parse_iso8601_timestamp()</link></para></listitem>
+ * </itemizedlist>
+ *
+ * This function is typically used when reading configuration files or other non-user input that should be locale 
+ * independent.
+ *
  * Returns: %TRUE if the value has been converted to @type from
  * its string representation; it not means that the value is converted 
  * successfully, just that the transformation is avairable. %FALSE otherwise.
@@ -1703,8 +1746,13 @@ gda_value_set_from_value (GValue *value, const GValue *from)
 	g_return_val_if_fail (from, FALSE);
 
 	if (G_IS_VALUE (from)) {
-		g_value_copy (from, value);
-		return g_value_type_compatible (G_VALUE_TYPE (from), G_VALUE_TYPE (value));
+		if (g_value_type_compatible (G_VALUE_TYPE (from), G_VALUE_TYPE (value))) {
+			g_value_reset (value);
+			g_value_copy (from, value);
+			return TRUE;
+		}
+		else
+			return FALSE;
 	}
 	else {
 		l_g_value_unset (value);
@@ -1787,30 +1835,56 @@ gda_value_stringify (const GValue *value)
 }
 	
 /**
- * gda_value_bcompare
+ * gda_value_differ
  * @value1: a #GValue to compare.
  * @value2: the other #GValue to be compared to @value1.
  *
- * Tells if two values are equal or not
+ * Tells if two values are equal or not, by comparing memory representations. Unlike gda_value_compare(),
+ * the returned value is boolean, and gives no idea about ordering.
  *
- * Returns: 0 if @value1 and @value2 are equal, and something else otherwise
+ * The two values must be of the same type, with the exception that a value of any type can be
+ * compared to a GDA_TYPE_NULL value, specifically:
+ * <itemizedlist>
+ *   <listitem><para>if @value1 and @value2 are both GDA_TYPE_NULL values then the returned value is 0</para></listitem>
+ *   <listitem><para>if @value1 is a GDA_TYPE_NULL value and @value2 is of another type then the returned value is 1</para></listitem>
+ *   <listitem><para>if @value1 is of another type and @value2 is a GDA_TYPE_NULL value then the returned value is 1</para></listitem>
+ *   <listitem><para>in all other cases, @value1 and @value2 must be of the same type and their values are compared</para></listitem>
+ * </itemizedlist>
+ *
+ * Returns: a non 0 value if @value1 and @value2 differ, and 0 if they are equal
  */
 gint
-gda_value_bcompare (const GValue *value1, const GValue *value2)
+gda_value_differ (const GValue *value1, const GValue *value2)
 {
 	GType type;
 	g_return_val_if_fail (value1 && value2, FALSE);
 
 	/* blind value comparison */
+	type = G_VALUE_TYPE (value1);
 	if (!bcmp (value1, value2, sizeof (GValue)))
 		return 0;
 
-	type = G_VALUE_TYPE (value1);
+	/* handle GDA_TYPE_NULL comparisons with other types */
+	else if (type == GDA_TYPE_NULL) {
+		if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+			return 0;
+		else
+			return 1;
+	}
+
+	else if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+		return 1;
+
+	g_return_val_if_fail (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2), 1);
+
+	/* general case */
 	if (type == GDA_TYPE_BINARY) {
 		const GdaBinary *binary1 = gda_value_get_binary (value1);
 		const GdaBinary *binary2 = gda_value_get_binary (value2);
 		if (binary1 && binary2 && (binary1->binary_length == binary2->binary_length))
-			return bcmp (binary1->data, binary2->data, binary1->binary_length) ;
+			return bcmp (binary1->data, binary2->data, binary1->binary_length);
+		else
+			return 1;
 	}
 
 	else if (type == GDA_TYPE_BLOB) {
@@ -1821,6 +1895,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 				return bcmp (((GdaBinary *)blob1)->data, ((GdaBinary *)blob2)->data, 
 					     ((GdaBinary *)blob1)->binary_length);
 		}
+		return 1;
 	}
 
 	else if (type == G_TYPE_DATE) {
@@ -1830,6 +1905,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		d2 = (GDate *) g_value_get_boxed (value2);
 		if (d1 && d2)
 			return g_date_compare (d1, d2);
+		return 1;
 	}
 
 	else if (type == GDA_TYPE_GEOMETRIC_POINT) {
@@ -1838,6 +1914,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		p2 = gda_value_get_geometric_point (value2);
 		if (p1 && p2)
 			return bcmp (p1, p2, sizeof (GdaGeometricPoint));
+		return 1;
 	}
 
 	else if (type == G_TYPE_OBJECT) {
@@ -1851,7 +1928,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		GList *l1, *l2;
 		for (l1 = (GList*) gda_value_get_list (value1), l2 = (GList*) gda_value_get_list (value2); 
 		     l1 != NULL && l2 != NULL; l1 = l1->next, l2 = l2->next){
-			if (gda_value_bcompare ((GValue *) l1->data, (GValue *) l2->data))
+			if (gda_value_differ ((GValue *) l1->data, (GValue *) l2->data))
 				return 1;
 		}
 		return 0;
@@ -1863,18 +1940,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		num2 = gda_value_get_numeric (value2);
                 if (num1 && num2)
 			return strcmp (num1->number, num2->number);
-	}
-
-	else if (type == GDA_TYPE_SHORT) {
-		gshort i1 = gda_value_get_short (value1);
-		gshort i2 = gda_value_get_short (value2);
-		return (i1 == i2) ? 0 : 1;
-	}
-	
-	else if (type == GDA_TYPE_USHORT) {
-		gushort i1 = gda_value_get_ushort (value1);
-		gushort i2 = gda_value_get_ushort (value2);
-		return (i1 == i2) ? 0 : 1;
+		return 1;
 	}
 	
 	else if (type == G_TYPE_STRING)	{
@@ -1883,6 +1949,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		str2 = g_value_get_string (value2);
 		if (str1 && str2)
 			return strcmp (str1, str2);
+		return 1;
 	}
 	
 	else if (type == GDA_TYPE_TIME) {
@@ -1891,6 +1958,7 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		t2 = gda_value_get_time (value2);
 		if (t1 && t2)
 			return bcmp (t1, t2, sizeof (GdaTime));
+		return 1;
 	}
 
 	else if (type == GDA_TYPE_TIMESTAMP) {
@@ -1899,17 +1967,44 @@ gda_value_bcompare (const GValue *value1, const GValue *value2)
 		ts2 = gda_value_get_timestamp (value2);
 		if (ts1 && ts2)
 			return bcmp (ts1, ts2, sizeof (GdaTimestamp));
+		return 1;
 	}
 
+	else if ((type == G_TYPE_INT) ||
+		 (type == G_TYPE_UINT) ||
+		 (type == G_TYPE_INT64) ||
+		 (type == G_TYPE_UINT64) ||
+		 (type == GDA_TYPE_SHORT) ||
+		 (type == GDA_TYPE_USHORT) ||
+		 (type == G_TYPE_FLOAT) ||
+		 (type == G_TYPE_DOUBLE) ||
+		 (type == G_TYPE_BOOLEAN) ||
+		 (type == G_TYPE_CHAR) ||
+		 (type == G_TYPE_UCHAR) ||
+		 (type == G_TYPE_LONG) ||
+		 (type == G_TYPE_ULONG))
+		/* values here ARE different because otherwise the bcmp() at the beginning would
+		 * already have retruned */
+		return 1;
+
+	g_warning ("%s() cannot handle values of type %s", __FUNCTION__, g_type_name (G_VALUE_TYPE (value1)));
+
 	return 1;
 }
 
 /**
  * gda_value_compare
- * @value1: a #GValue to compare.
- * @value2: the other #GValue to be compared to @value1.
+ * @value1: a #GValue to compare (not %NULL)
+ * @value2: the other #GValue to be compared to @value1 (not %NULL)
  *
- * Compares two values of the same type.
+ * Compares two values of the same type, with the exception that a value of any type can be
+ * compared to a GDA_TYPE_NULL value, specifically:
+ * <itemizedlist>
+ *   <listitem><para>if @value1 and @value2 are both GDA_TYPE_NULL values then the returned value is 0</para></listitem>
+ *   <listitem><para>if @value1 is a GDA_TYPE_NULL value and @value2 is of another type then the returned value is -1</para></listitem>
+ *   <listitem><para>if @value1 is of another type and @value2 is a GDA_TYPE_NULL value then the returned value is 1</para></listitem>
+ *   <listitem><para>in all other cases, @value1 and @value2 must be of the same type and their values are compared</para></listitem>
+ * </itemizedlist>
  *
  * Returns: if both values have the same type, returns 0 if both contain
  * the same value, an integer less than 0 if @value1 is less than @value2 or
@@ -1923,14 +2018,27 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 	GType type;
 
 	g_return_val_if_fail (value1 && value2, -1);
-	g_return_val_if_fail (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2), -1);
 
 	type = G_VALUE_TYPE (value1);
 	
 	if (value1 == value2)
 		return 0;
 
-	else if (type == G_TYPE_INT64) {
+	/* handle GDA_TYPE_NULL comparisons with other types */
+	else if (type == GDA_TYPE_NULL) {
+		if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+			return 0;
+		else
+			return -1;
+	}
+
+	else if (G_VALUE_TYPE (value2) == GDA_TYPE_NULL)
+		return 1;
+
+	/* general case */
+	g_return_val_if_fail (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2), -1);
+
+	if (type == G_TYPE_INT64) {
 		gint64 i1 = g_value_get_int64 (value1);
 		gint64 i2 = g_value_get_int64 (value2);
 		return (i1 > i2) ? 1 : ((i1 == i2) ? 0 : -1);
@@ -2158,42 +2266,6 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 	return 0;
 }
 
-#define _value_is_null(x) (!(x) || (G_VALUE_TYPE (x) == GDA_TYPE_NULL) || \
-			   ((G_VALUE_TYPE (x) == G_TYPE_STRING) && !g_value_get_string (x)))
-
-/**
- * gda_value_compare_ext
- * @value1: a #GValue to compare.
- * @value2: the other #GValue to be compared to @value1.
- *
- * Like gda_value_compare(), compares two values of the same type, except that NULL values and values
- * of type GDA_TYPE_NULL are considered equals
- *
- * Returns: 0 if both contain the same value, an integer less than 0 if @value1 is less than @value2 or
- * an integer greater than 0 if @value1 is greater than @value2.
- */
-gint
-gda_value_compare_ext (const GValue *value1, const GValue *value2)
-{
-	if (value1 == value2)
-		return 0;
-
-	if (_value_is_null (value1)) {
-		/* value1 represents a NULL value */
-		if (_value_is_null (value2))
-			return 0;
-		else
-			return 1;
-	}
-	else {
-		/* value1 does not represents a NULL value */
-		if (_value_is_null (value2))
-			return -1;
-		else
-			return gda_value_compare (value1, value2);
-	}
-}
-
 /*
  * to_string
  * 
