@@ -42,6 +42,8 @@
 #include <libgda/gda-data-model-dir.h>
 #include <libgda/gda-data-access-wrapper.h>
 #include <libgda/gda-data-proxy.h>
+#include <libgda/gda-data-select.h>
+#include <libgda/gda-lockable.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-quark-list.h>
 #include <libgda/gda-row.h>
@@ -72,9 +74,12 @@
 
 #include <libgda/gda-meta-store.h>
 
+#include <libgda/gda-mutex.h>
+
 G_BEGIN_DECLS
 
 void gda_init (void);
+gchar *gda_get_application_exec_path (const gchar *app_name);
 
 G_END_DECLS
 
