@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007 Vivien Malerba
+ * Copyright (C) 2007 - 2008 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -27,8 +27,12 @@
 gchar *_remove_quotes (gchar *str);
 gchar *_add_quotes (const gchar *str);
 
-gchar *_json_quote_string (const gchar *str);
-gboolean _string_is_identifier (const gchar *str);
+gchar    *_json_quote_string (const gchar *str);
+
+gboolean  _string_is_identifier (const gchar *str);
+gboolean  _identifier_needs_quotes (const gchar *str);
+gboolean  _split_identifier_string (gchar *str, gchar **remain, gchar **last);
+gchar    *_identifier_unquote (gchar *str);
 
 /* to be removed, only here for debug */
 gchar *gda_sql_value_stringify (const GValue *value);
