@@ -26,6 +26,7 @@
 #include "gda-vconnection-data-model.h"
 #include "gda-vconnection-data-model-private.h"
 #include "gda-virtual-provider.h"
+#include <libgda/gda-connection-private.h>
 #include "../gda-sqlite.h"
 
 struct _GdaVconnectionDataModelPrivate {
@@ -74,7 +75,7 @@ gda_vconnection_data_model_init (GdaVconnectionDataModel *cnc, GdaVconnectionDat
 	cnc->priv = g_new (GdaVconnectionDataModelPrivate, 1);
 	cnc->priv->table_data_list = NULL;
 
-	g_object_set (G_OBJECT (cnc), "cnc_string", "_IS_VIRTUAL=TRUE", NULL);
+	g_object_set (G_OBJECT (cnc), "cnc-string", "_IS_VIRTUAL=TRUE", NULL);
 }
 
 static void
@@ -103,6 +104,7 @@ gda_vconnection_data_model_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		if (type == 0) {
 			static GTypeInfo info = {
 				sizeof (GdaVconnectionDataModelClass),
@@ -115,7 +117,10 @@ gda_vconnection_data_model_get_type (void)
 				(GInstanceInitFunc) gda_vconnection_data_model_init
 			};
 			
+		g_static_mutex_lock (&registering);
+		if (type == 0)
 			type = g_type_register_static (GDA_TYPE_VIRTUAL_CONNECTION, "GdaVconnectionDataModel", &info, 0);
+		g_static_mutex_unlock (&registering);
 		}
 	}
 
@@ -156,6 +161,13 @@ gda_vconnection_data_model_get_property (GObject *object,
         }
 }
 
+static void
+spec_destroy_func (GdaVconnectionDataModelSpec *spec)
+{
+	g_object_unref (spec->data_model);
+	g_free (spec);
+}
+
 /**
  * gda_vconnection_data_model_add_model
  * @cnc: a #GdaVconnectionDataModel connection
@@ -180,7 +192,8 @@ gda_vconnection_data_model_add_model (GdaVconnectionDataModel *cnc,
 
 	spec = g_new0 (GdaVconnectionDataModelSpec, 1);
 	spec->data_model = model;
-	retval = gda_vconnection_data_model_add (cnc, spec, g_free, table_name, error);
+	g_object_ref (model);
+	retval = gda_vconnection_data_model_add (cnc, spec, (GDestroyNotify) spec_destroy_func, table_name, error);
 
 	return retval;
 }
@@ -210,7 +223,7 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 	int rc;
 	char *zErrMsg = NULL;
 	gboolean retval = TRUE;
-	SQLITEcnc *scnc;
+	SqliteConnectionData *scnc;
 
 	static gint counter = 0;
 
@@ -221,11 +234,9 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 	if (spec->data_model)
 		g_return_val_if_fail (GDA_IS_DATA_MODEL (spec->data_model), FALSE);
 
-	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
-	if (!scnc) {
-		gda_connection_add_event_string ((GdaConnection *) cnc, _("Invalid SQLite handle"));
+	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data ((GdaConnection *) cnc);
+	if (!scnc) 
 		return FALSE;
-	}
 
 	/* create a new GdaVConnectionTableData structure for this virtual table */
 	GdaVConnectionTableData *td;
@@ -237,7 +248,7 @@ gda_vconnection_data_model_add (GdaVconnectionDataModel *cnc, GdaVconnectionData
 	cnc->priv->table_data_list = g_slist_append (cnc->priv->table_data_list, td);
 
 	/* actually create the virtual table in @cnc */
-	prov = (GdaVirtualProvider *) gda_connection_get_provider_obj (GDA_CONNECTION (cnc));
+	prov = (GdaVirtualProvider *) gda_connection_get_provider (GDA_CONNECTION (cnc));
 	str = g_strdup_printf ("CREATE VIRTUAL TABLE %s USING %s ('%s')", table_name, G_OBJECT_TYPE_NAME (prov), td->unique_name);
 	rc = sqlite3_exec (scnc->connection, str, NULL, 0, &zErrMsg);
 	g_free (str);
@@ -279,16 +290,14 @@ gda_vconnection_data_model_remove (GdaVconnectionDataModel *cnc, const gchar *ta
 	int rc;
 	char *zErrMsg = NULL;
 	gboolean retval = TRUE;
-	SQLITEcnc *scnc;
+	SqliteConnectionData *scnc;
 
 	g_return_val_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc), FALSE);
 	g_return_val_if_fail (table_name && *table_name, FALSE);
 
-	scnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_SQLITE_HANDLE);
-	if (!scnc) {
-		gda_connection_add_event_string ((GdaConnection *) cnc, _("Invalid SQLite handle"));
+	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data ((GdaConnection *) cnc);
+	if (!scnc) 
 		return FALSE;
-	}
 
 	td = gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (!td) {
@@ -297,7 +306,7 @@ gda_vconnection_data_model_remove (GdaVconnectionDataModel *cnc, const gchar *ta
 		return FALSE;
 	}
 	
-	prov = (GdaVirtualProvider *) gda_connection_get_provider_obj (GDA_CONNECTION (cnc));
+	prov = (GdaVirtualProvider *) gda_connection_get_provider (GDA_CONNECTION (cnc));
 	str = g_strdup_printf ("DROP TABLE %s", td->table_name);
 	rc = sqlite3_exec (scnc->connection, str, NULL, 0, &zErrMsg);
 	g_free (str);
@@ -375,7 +384,11 @@ gda_vconnection_data_model_get_table_name (GdaVconnectionDataModel *cnc, GdaData
  * @func: a #GdaVConnectionDataModelFunc function pointer
  * @data: data to pass to @cunc calls
  *
- * Call @func for each #GdaDataModel represented as a table in @cnc.
+ * Call @func for each table in @cnc. 
+ *
+ * Warning: @func will be called for any table present in @cnc even if no data
+ * model represents the contents of the table (which means the 1st argument of @func
+ * may be %NULL)
  */
 void
 gda_vconnection_data_model_foreach (GdaVconnectionDataModel *cnc, 
@@ -392,8 +405,7 @@ gda_vconnection_data_model_foreach (GdaVconnectionDataModel *cnc,
 	while (list) {
 		GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
 		next = list->next;
-		if (td->spec->data_model)
-			func (td->spec->data_model, td->table_name, data);
+		func (td->spec->data_model, td->table_name, data);
 		list = next;
 	}
 }
