@@ -245,7 +245,7 @@ gda_vprovider_data_model_create_connection (GdaServerProvider *provider)
 	GdaConnection *cnc;
 	g_return_val_if_fail (GDA_IS_VPROVIDER_DATA_MODEL (provider), NULL);
 
-	cnc = g_object_new (GDA_TYPE_VCONNECTION_DATA_MODEL, "provider-obj", provider, NULL);
+	cnc = g_object_new (GDA_TYPE_VCONNECTION_DATA_MODEL, "provider", provider, NULL);
 
 	return cnc;
 }
@@ -377,7 +377,7 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 			if (gda_data_model_get_access_flags (td->spec->data_model) & GDA_DATA_MODEL_ACCESS_RANDOM)
 				proxy = g_object_new (GDA_TYPE_DATA_PROXY, 
 						      "model", td->spec->data_model, 
-						      "sample_size", 0, NULL);
+						      "sample-size", 0, NULL);
 			else {
 				/* no random access => use a wrapper */
 				GdaDataModel *wrapper;
@@ -401,7 +401,18 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 		g_object_ref (td->real_model);
 	}
 	else  {
-		columns = td->spec->create_columns_func (td->spec);
+		GError *error = NULL;
+		columns = td->spec->create_columns_func (td->spec, &error);
+		if (! columns) {
+			if (error && error->message) {
+				int len = strlen (error->message) + 1;
+				*pzErr = sqlite3_malloc (sizeof (gchar) * len);
+				memcpy (*pzErr, error->message, len);
+			}
+			else 
+				*pzErr = sqlite3_mprintf (_("Could not compute virtual table's columns"));
+			return SQLITE_ERROR;
+		}
 		ncols = g_list_length (columns);
 	}
 
@@ -412,6 +423,7 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 	for (i = 0; i < ncols; i++) {
 		GdaColumn *column;
 		const gchar *name, *type;
+		GType gtype;
 		gchar *newcolname;
 
 		if (i != 0)
@@ -434,38 +446,32 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 		else
 			newcolname = g_strdup (name);
 
-		type = g_type_name (gda_column_get_g_type (column));
+		gtype = gda_column_get_g_type (column);
+		type = g_type_name (gtype);
 		if (!type) {
 			*pzErr = sqlite3_mprintf (_("Can't get data model's column type or type for column %d"), i);
 			g_string_free (sql, TRUE);
 			return SQLITE_ERROR;
 		}
-		else if (!strcmp (type, "GdaBlob") || !strcmp (type, "GdaBinary"))
+		else if ((gtype == GDA_TYPE_BLOB) || (gtype == GDA_TYPE_BINARY))
 			type = "blob";
-		else if (!strcmp (type, "gchararray"))
+		else if (gtype == G_TYPE_STRING)
 			type = "text";
-		else if (!strncmp (type, "gint", 4))
+		else if ((gtype == G_TYPE_INT) || (gtype == G_TYPE_UINT) || 
+			 (gtype == G_TYPE_INT64) || (gtype == G_TYPE_UINT64) ||
+			 (gtype == GDA_TYPE_SHORT) || (gtype == GDA_TYPE_USHORT) ||
+			 (gtype == G_TYPE_LONG) || (gtype == G_TYPE_ULONG))
 			type = "integer";
-		else if (!strcmp (type, "gdouble"))
+		else if ((gtype == G_TYPE_DOUBLE) || (gtype == G_TYPE_FLOAT))
 			type = "real";
-		else if (!strcmp (type, "gfloat"))
-			type = "real";
-		else {
-			/* use the declared GType */
-			type = g_type_name (gda_column_get_g_type (column));
-			if (!strcmp (type, "GdaBlob") || !strcmp (type, "GdaBinary"))
-				type = "blob";
-			else if (!strcmp (type, "gchararray"))
-				type = "text";
-			else if (!strncmp (type, "gint", 4))
-				type = "integer";
-			else if (!strcmp (type, "gdouble"))
-				type = "real";
-			else if (!strcmp (type, "gfloat"))
-				type = "real";
-			else
-				type = "text";
-		}
+		else if (gtype == G_TYPE_DATE)
+			type = "date";
+		else if (gtype == GDA_TYPE_TIME)
+			type = "time";
+		else if (gtype == GDA_TYPE_TIMESTAMP)
+			type = "timestamp";
+		else 
+			type = "text";
 
 		g_string_append (sql, newcolname);
 		g_free (newcolname);
@@ -613,10 +619,12 @@ virtualNext (sqlite3_vtab_cursor *cur)
 
 	TRACE ();
 
-	gda_data_model_iter_move_next (cursor->iter);
-	while (gda_data_model_iter_is_valid (cursor->iter) &&
-	       gda_data_proxy_row_is_deleted (vtable->proxy, gda_data_model_iter_get_row (cursor->iter)))
-		gda_data_model_iter_move_next (cursor->iter);
+	if (!gda_data_model_iter_move_next (cursor->iter)) {
+		if (gda_data_model_iter_is_valid (cursor->iter))
+			return SQLITE_IOERR;
+		else
+			return SQLITE_OK;
+	}
 	return SQLITE_OK;
 }
 
