@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <glib/gbacktrace.h>
 #include <virtual/gda-vconnection-data-model.h>
+#include <libgda/gda-connection-private.h>
 #include <libgda/gda-server-provider-extra.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-data-model-private.h>
@@ -89,6 +90,7 @@ gda_mdb_provider_class_init (GdaMdbProviderClass *klass)
 static void
 gda_mdb_provider_init (GdaMdbProvider *myprv, GdaMdbProviderClass *klass)
 {
+	mdb_set_date_fmt ("%Y-%m-%d %H:%M:%S");
 }
 
 static void
@@ -231,9 +233,11 @@ gda_mdb_provider_open_connection (GdaServerProvider *provider, GdaConnection *cn
 
 	/* look for parameters */
 	dirname = gda_quark_list_find (params, "DB_DIR");
+	if (!dirname)
+		dirname= "."; /* default to current directory */
 	dbname = gda_quark_list_find (params, "DB_NAME");
 	
-	if (!dirname || !dbname) {
+	if (!dbname) {
                 const gchar *str;
 
                 str = gda_quark_list_find (params, "FILENAME");
@@ -245,6 +249,7 @@ gda_mdb_provider_open_connection (GdaServerProvider *provider, GdaConnection *cn
                 else {
                         gint len = strlen (str);
 			gint elen = strlen (FILE_EXTENSION);
+			dirname = NULL;
 
 			if (g_str_has_suffix (str, FILE_EXTENSION)) {
                                 gchar *ptr;
@@ -401,10 +406,10 @@ table_create_columns_func (LocalSpec *spec)
 		gda_column_set_name (gda_col, tmp);
 		g_free (tmp);
 		gda_column_set_g_type (gda_col, gda_mdb_type_to_gda (mdb_col->col_type));
-		tmp = sanitize_name (g_strdup (mdb_get_coltype_string (spec->cdata->mdb->default_backend, mdb_col->col_type)));
+		tmp = sanitize_name (g_strdup (mdb_get_coltype_string (spec->cdata->mdb->default_backend, 
+								       mdb_col->col_type)));
 		gda_column_set_dbms_type (gda_col, tmp);
 		g_free (tmp);
-                gda_column_set_defined_size (gda_col, mdb_col->col_size);
 		columns = g_list_prepend (columns, gda_col);
 	}
 
@@ -427,7 +432,7 @@ table_create_model_func (LocalSpec *spec)
 	mdb_rewind_table (mdb_table);
 
 	/* prepare data model */
-	g_print ("New data model for table %p\n", mdb_table);
+	/*g_print ("New data model for MDB table %p\n", mdb_table);*/
 	model = gda_data_model_array_new (mdb_table->num_cols);
 	
 	/* prepare column types */
@@ -458,14 +463,15 @@ table_create_model_func (LocalSpec *spec)
 		tmp = sanitize_name (g_strdup (mdb_col->name));
 		gda_column_set_name (gda_col, tmp);
 		gda_column_set_title (gda_col, tmp);
-		gda_column_set_caption (gda_col, tmp);
 		g_free (tmp);
 		tmp = sanitize_name (g_strdup (mdb_get_coltype_string (spec->cdata->mdb->default_backend, mdb_col->col_type)));
 		gda_column_set_dbms_type (gda_col, tmp);
 		g_free (tmp);
                 gda_column_set_g_type (gda_col, coltypes [c]);
-		/*g_print ("col: %s (%s/%s)\n", gda_column_get_name (gda_col), gda_column_get_dbms_type (gda_col),
-		  g_type_name (coltypes [c]));*/
+#ifdef GDA_DEBUG_NO
+		g_print ("col: %s (%s/%s)\n", gda_column_get_name (gda_col), gda_column_get_dbms_type (gda_col),
+			 g_type_name (coltypes [c]));
+#endif
         }
 
 	/* read data */
@@ -493,7 +499,8 @@ table_create_model_func (LocalSpec *spec)
 #endif
 			}
 			else
-				gda_value_set_from_string ((tmpval = gda_value_new (coltypes [c])), bound_values[c], coltypes [c]);
+				gda_value_set_from_string ((tmpval = gda_value_new (coltypes [c])), bound_values[c], 
+							   coltypes [c]);
                         value_list = g_list_append (value_list, tmpval);
 		}
 
@@ -523,7 +530,7 @@ gda_mdb_provider_get_server_version (GdaServerProvider *provider,
 	MdbConnectionData *cdata;
 	
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-        g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+        g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
 	if (!cdata)
@@ -544,7 +551,7 @@ gda_mdb_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
 	MdbConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-        g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+        g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
 	if (!cdata)
