@@ -155,7 +155,7 @@ static GObject             *gda_mysql_provider_statement_execute (GdaServerProvi
 								  GType                           *col_types,
 								  GdaSet                         **last_inserted_row, 
 								  guint                           *task_id,
-								  GdaServerProviderAsyncCallback   async_cb, 
+								  GdaServerProviderExecCallback    async_cb, 
 								  gpointer                         cb_data,
 								  GError                         **error);
 
@@ -306,6 +306,14 @@ gda_mysql_provider_class_init (GdaMysqlProviderClass  *klass)
 	provider_class->xa_funcs->xa_rollback = gda_mysql_provider_xa_rollback;
 	provider_class->xa_funcs->xa_recover = gda_mysql_provider_xa_recover;
 	
+	if (! mysql_thread_safe ()) {
+		gda_log_message ("MySQL was not compiled with the --enable-thread-safe-client flag, "
+				 "only one thread can access the provider");
+		provider_class->limiting_thread = g_thread_self ();
+	}
+	else
+		provider_class->limiting_thread = NULL;
+	
 }
 
 static void
@@ -585,7 +593,7 @@ gda_mysql_provider_close_connection (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	/* Close the connection using the C API */
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -612,7 +620,7 @@ gda_mysql_provider_get_server_version (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -633,7 +641,7 @@ gda_mysql_provider_get_database (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -661,7 +669,7 @@ gda_mysql_provider_supports_operation (GdaServerProvider       *provider,
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
         switch (type) {
@@ -706,7 +714,7 @@ gda_mysql_provider_create_operation (GdaServerProvider       *provider,
 
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
         file = g_utf8_strdown (gda_server_operation_op_type_to_string (type), -1);
@@ -745,7 +753,7 @@ gda_mysql_provider_render_operation (GdaServerProvider   *provider,
 
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	/* test @op's validity */
@@ -815,7 +823,7 @@ gda_mysql_provider_perform_operation (GdaServerProvider               *provider,
 
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
         optype = gda_server_operation_get_op_type (op);
 	switch (optype) {
@@ -840,7 +848,7 @@ gda_mysql_provider_begin_transaction (GdaServerProvider        *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -863,7 +871,7 @@ gda_mysql_provider_commit_transaction (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -886,7 +894,7 @@ gda_mysql_provider_rollback_transaction (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -909,7 +917,7 @@ gda_mysql_provider_add_savepoint (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -932,7 +940,7 @@ gda_mysql_provider_rollback_savepoint (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -955,7 +963,7 @@ gda_mysql_provider_delete_savepoint (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -976,7 +984,7 @@ gda_mysql_provider_supports_feature (GdaServerProvider     *provider,
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	switch (feature) {
@@ -1011,7 +1019,7 @@ gda_mysql_provider_get_data_handler (GdaServerProvider  *provider,
 	GdaDataHandler *dh;
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	if (type == G_TYPE_INVALID) {
@@ -1047,7 +1055,7 @@ gda_mysql_provider_get_default_dbms_type (GdaServerProvider  *provider,
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	TO_IMPLEMENT;
@@ -1132,7 +1140,7 @@ gda_mysql_provider_statement_to_sql (GdaServerProvider    *provider,
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), NULL);
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 	}
 
 	return gda_statement_to_sql_extended (stmt, cnc, params, flags, params_used, error);
@@ -1155,11 +1163,11 @@ gda_mysql_provider_statement_prepare (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), FALSE);
 
 	/* fetch prepares stmt if already done */
-	ps = gda_connection_get_prepared_statement (cnc, stmt);
+	ps = (GdaMysqlPStmt*) gda_connection_get_prepared_statement (cnc, stmt);
 	if (ps)
 		return TRUE;
 
@@ -1201,7 +1209,7 @@ gda_mysql_provider_statement_prepare (GdaServerProvider  *provider,
 		} else {
 			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_PREPARE_STMT_ERROR,
-				     _("Unnamed holder is not allowed in prepared statement."));
+				     _("Unnamed statement parameter is not allowed in prepared statement."));
 			g_slist_foreach (param_ids, (GFunc) g_free, NULL);
 			g_slist_free (param_ids);
 			param_ids = NULL;
@@ -1221,7 +1229,7 @@ gda_mysql_provider_statement_prepare (GdaServerProvider  *provider,
 		_GDA_PSTMT(ps)->param_ids = param_ids;
 		_GDA_PSTMT(ps)->sql = sql;
 		
-		gda_connection_add_prepared_statement (cnc, stmt, ps);
+		gda_connection_add_prepared_statement (cnc, stmt, (GdaPStmt*) ps);
 		return TRUE;
 	}
 	
@@ -1281,7 +1289,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 				      GType                           *col_types,
 				      GdaSet                         **last_inserted_row, 
 				      guint                           *task_id, 
-				      GdaServerProviderAsyncCallback   async_cb,
+				      GdaServerProviderExecCallback    async_cb,
 				      gpointer                         cb_data,
 				      GError                         **error)
 {
@@ -1289,7 +1297,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), NULL);
 
 	/* If asynchronous connection opening is not supported, then exit now */
@@ -1303,9 +1311,11 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 	if (!cdata) 
 		return FALSE;
 
+	if (last_inserted_row)
+		*last_inserted_row = NULL;
 
 	/* get/create new prepared statement */
-	ps = gda_connection_get_prepared_statement (cnc, stmt);
+	ps = (GdaMysqlPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
 	if (!ps) {
 		if (!gda_mysql_provider_statement_prepare (provider, cnc, stmt, NULL)) {
 			/* this case can appear for example if some variables are used in places
@@ -1320,21 +1330,19 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			if (!sql)
 				return NULL;
 			ps = prepare_stmt_simple (cdata, sql, error);
+			g_free (sql);
 			if (!ps)
 				return NULL;
 			
 		}
 		else
-			ps = gda_connection_get_prepared_statement (cnc, stmt);
+			ps = (GdaMysqlPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
 	}
 	g_assert (ps);
 
 	/* optionnally reset the prepared statement if required by the API */
 	// TO_IMPLEMENT;
 	
-	
-	g_print ("   %s: SQL=%s\n", __func__, _GDA_PSTMT(ps)->sql);
-	
 	/* bind statement's parameters */
 	GSList *list;
 	GdaConnectionEvent *event = NULL;
@@ -1344,7 +1352,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 	char **param_values = g_new0 (char *, nb_params + 1);
         int *param_lengths = g_new0 (int, nb_params + 1);
         int *param_formats = g_new0 (int, nb_params + 1);
-	g_print ("NB=%d, SQL=%s\n", nb_params, _GDA_PSTMT(ps)->sql);
+	/* g_print ("NB=%d, SQL=%s\n", nb_params, _GDA_PSTMT(ps)->sql); */
 
 	MYSQL_BIND *mysql_bind_param = g_new0 (MYSQL_BIND, nb_params);
 	
@@ -1379,6 +1387,16 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			g_free (str);
 			break;
 		}
+		if (!gda_holder_is_valid (h)) {
+			gchar *str;
+			str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
+			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+			gda_connection_event_set_description (event, str);
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
+			g_free (str);
+			break;
+		}
 
 		/* actual binding using the C API, for parameter at position @i */
 		const GValue *value = gda_holder_get_value (h);
@@ -1396,7 +1414,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			param_values[i] = gda_handler_time_get_no_locale_str_from_value (handler_time,
 											 value);
 			
-			g_print ("--- TIME=%s\n", param_values[i]);
+			/* g_print ("--- TIME=%s\n", param_values[i]); */
 		} else {
 			GdaDataHandler *data_handler = gda_server_provider_get_data_handler_gtype
 				(provider, cnc, G_VALUE_TYPE(value));
@@ -1405,7 +1423,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			else
 				param_values[i] = gda_data_handler_get_str_from_value (data_handler,
 										       value);
-			g_print ("--- PV=%s\n", param_values[i]);
+			/* g_print ("--- PV=%s\n", param_values[i]); */
 
 			mysql_bind_param[i].buffer_type = MYSQL_TYPE_STRING;
 			mysql_bind_param[i].buffer = g_strdup (param_values[i]);
@@ -1413,11 +1431,6 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			mysql_bind_param[i].length = g_malloc0 (sizeof(unsigned long));
 
 		}
-		
-		gchar *str = gda_value_stringify (value);
-		g_print ("   %s: %s=%s\n", __func__, pname, str);
-		g_free (str);
-		
 
 	}
 		
@@ -1436,6 +1449,24 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 
 		return NULL;
 	}
+
+
+	/* use cursor when retrieving result */
+	if ((model_usage & GDA_STATEMENT_MODEL_RANDOM_ACCESS) == 0 &&
+	    gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT) {
+#if MYSQL_VERSION_ID >= 50002
+		const unsigned long cursor_type = CURSOR_TYPE_READ_ONLY;
+		if (mysql_stmt_attr_set (cdata->mysql_stmt, STMT_ATTR_CURSOR_TYPE, (void *) &cursor_type)) {
+			_gda_mysql_make_error (cnc, NULL, cdata->mysql_stmt, NULL);
+			return NULL;
+		}
+#else
+		model_usage = GDA_STATEMENT_MODEL_RANDOM_ACCESS;
+		g_warning (_("Could not use CURSOR. Mysql version 5.0 at least is required. "
+			     "Using random access anyway."));
+#endif
+	}
+
 	
 	/* add a connection event for the execution */
 	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
@@ -1447,7 +1478,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 	if (mysql_stmt_execute (cdata->mysql_stmt)) {
 		event = _gda_mysql_make_error (cnc, NULL, cdata->mysql_stmt, error);
 	} else {
-		//	
+
 		/* execute prepared statement using C API depending on its kind */
 		if (!g_ascii_strncasecmp (_GDA_PSTMT (ps)->sql, "SELECT", 6) ||
 		    !g_ascii_strncasecmp (_GDA_PSTMT (ps)->sql, "SHOW", 4) ||
@@ -1464,7 +1495,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 				else
 					flags = GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD;
 
-				return_value = (GObject *) gda_mysql_recordset_new (cnc, ps, flags, col_types);
+				return_value = (GObject *) gda_mysql_recordset_new (cnc, ps, params, flags, col_types);
 				gda_connection_internal_statement_executed (cnc, stmt, params, NULL); /* required: help @cnc keep some stats */
 			}
 			
@@ -1512,7 +1543,7 @@ gda_mysql_provider_xa_start (GdaServerProvider         *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -1536,7 +1567,7 @@ gda_mysql_provider_xa_end (GdaServerProvider         *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -1559,7 +1590,7 @@ gda_mysql_provider_xa_prepare (GdaServerProvider         *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -1583,7 +1614,7 @@ gda_mysql_provider_xa_commit (GdaServerProvider         *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -1606,7 +1637,7 @@ gda_mysql_provider_xa_rollback (GdaServerProvider         *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -1630,7 +1661,7 @@ gda_mysql_provider_xa_recover (GdaServerProvider  *provider,
 	MysqlConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
