@@ -33,6 +33,7 @@
 #include <locale.h>
 #endif
 
+extern gchar *gda_numeric_locale;
 
 #define _GDA_PSTMT(x) ((GdaPStmt*)(x))
 
@@ -55,25 +56,25 @@ gda_mysql_recordset_dispose (GObject  *object);
 
 /* virtual methods */
 static gint
-gda_mysql_recordset_fetch_nb_rows (GdaPModel  *model);
+gda_mysql_recordset_fetch_nb_rows (GdaDataSelect  *model);
 static gboolean
-gda_mysql_recordset_fetch_random (GdaPModel  *model,
-				  GdaPRow   **prow,
+gda_mysql_recordset_fetch_random (GdaDataSelect  *model,
+				  GdaRow    **row,
 				  gint        rownum,
 				  GError    **error);
 static gboolean
-gda_mysql_recordset_fetch_next (GdaPModel  *model,
-				GdaPRow   **prow,
+gda_mysql_recordset_fetch_next (GdaDataSelect  *model,
+				GdaRow    **row,
 				gint        rownum,
 				GError    **error);
 static gboolean
-gda_mysql_recordset_fetch_prev (GdaPModel  *model,
-				GdaPRow   **prow,
+gda_mysql_recordset_fetch_prev (GdaDataSelect  *model,
+				GdaRow    **row,
 				gint        rownum,
 				GError    **error);
 static gboolean
-gda_mysql_recordset_fetch_at (GdaPModel  *model,
-			      GdaPRow   **prow,
+gda_mysql_recordset_fetch_at (GdaDataSelect  *model,
+			      GdaRow    **row,
 			      gint        rownum,
 			      GError    **error);
 
@@ -104,12 +105,52 @@ gda_mysql_recordset_init (GdaMysqlRecordset       *recset,
 	/* initialize specific information */
 	// TO_IMPLEMENT;
 	
-	recset->priv->chunk_size = 10;
+	recset->priv->chunk_size = 1;
 	recset->priv->chunks_read = 0;
 	
 }
 
 
+gint
+gda_mysql_recordset_get_chunk_size (GdaMysqlRecordset  *recset)
+{
+	g_return_if_fail (GDA_IS_MYSQL_RECORDSET (recset));
+	return recset->priv->chunk_size;
+}
+
+void
+gda_mysql_recordset_set_chunk_size (GdaMysqlRecordset  *recset,
+				    gint                chunk_size)
+{
+	g_return_if_fail (GDA_IS_MYSQL_RECORDSET (recset));
+
+	if (recset->priv->mysql_stmt == NULL)  // Creation is in progress so it's not set.
+		return;
+
+#if MYSQL_VERSION_ID >= 50002
+	const unsigned long prefetch_rows = chunk_size;
+	if (mysql_stmt_attr_set (recset->priv->mysql_stmt, STMT_ATTR_PREFETCH_ROWS,
+				 (void *) &prefetch_rows)) {
+		_gda_mysql_make_error (recset->priv->cnc, NULL, recset->priv->mysql_stmt, NULL);
+		return;
+	}
+	recset->priv->chunk_size = chunk_size;
+	g_object_notify (G_OBJECT(recset), "chunk-size");
+#else
+	g_warning (_("Could not use CURSOR. Mysql version 5.0 at least is required. "
+		     "Chunk size ignored."));
+#endif
+
+}
+
+gint
+gda_mysql_recordset_get_chunks_read (GdaMysqlRecordset  *recset)
+{
+	g_return_if_fail (GDA_IS_MYSQL_RECORDSET (recset));
+	return recset->priv->chunks_read;
+}
+
+
 static void
 gda_mysql_recordset_set_property (GObject       *object,
 				  guint          param_id,
@@ -126,7 +167,8 @@ gda_mysql_recordset_set_property (GObject       *object,
 
 	switch (param_id) {
 	case PROP_CHUNK_SIZE:
-		record_set->priv->chunk_size = g_value_get_int (value);
+		gda_mysql_recordset_set_chunk_size (record_set,
+						    g_value_get_int (value));
 		break;
 	case PROP_CHUNKS_READ:
 		break;
@@ -166,7 +208,7 @@ static void
 gda_mysql_recordset_class_init (GdaMysqlRecordsetClass  *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaPModelClass *pmodel_class = GDA_PMODEL_CLASS (klass);
+	GdaDataSelectClass *pmodel_class = GDA_DATA_SELECT_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -187,7 +229,7 @@ gda_mysql_recordset_class_init (GdaMysqlRecordsetClass  *klass)
 		 PROP_CHUNK_SIZE,
 		 g_param_spec_int ("chunk-size", _("Number of rows fetched at a time"),
 				   NULL,
-				   1, G_MAXINT - 1, 10,
+				   1, G_MAXINT - 1, 1,
 				   (G_PARAM_CONSTRUCT | G_PARAM_WRITABLE | G_PARAM_READABLE)));
 
 	g_object_class_install_property
@@ -245,7 +287,7 @@ gda_mysql_recordset_get_type (void)
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
-			type = g_type_register_static (GDA_TYPE_PMODEL, "GdaMysqlRecordset", &info, 0);
+			type = g_type_register_static (GDA_TYPE_DATA_SELECT, "GdaMysqlRecordset", &info, 0);
 		g_static_mutex_unlock (&registering);
 	}
 
@@ -259,6 +301,7 @@ gda_mysql_recordset_get_type (void)
 GdaDataModel *
 gda_mysql_recordset_new (GdaConnection            *cnc,
 			 GdaMysqlPStmt            *ps,
+			 GdaSet                   *exec_params,
 			 GdaDataModelAccessFlags   flags,
 			 GType                    *col_types)
 {
@@ -300,7 +343,7 @@ gda_mysql_recordset_new (GdaConnection            *cnc,
 					if (col_types [i] == G_TYPE_NONE)
 						break;
 					if (i >= _GDA_PSTMT (ps)->ncols)
-						g_warning (_("Column %d is out of range (0-%d), ignoring its specified type"), i,
+						g_warning (_("Column %d out of range (0-%d), ignoring its specified type"), i,
 							   _GDA_PSTMT (ps)->ncols - 1);
 					else
 						_GDA_PSTMT (ps)->types [i] = col_types [i];
@@ -335,20 +378,6 @@ gda_mysql_recordset_new (GdaConnection            *cnc,
 			gda_column_set_g_type (column, gtype);
 			gda_column_set_name (column, field->name);
 			gda_column_set_title (column, field->name);
-			gda_column_set_scale (column, (gtype == G_TYPE_DOUBLE) ? DBL_DIG :
-					      (gtype == G_TYPE_FLOAT) ? FLT_DIG : 0);
-			gda_column_set_defined_size (column, field->length);
-			gda_column_set_references (column, "");
-
-			/* Use @cnc's associate GdaMetaStore to get the following information:
-			gda_column_set_references (column, ...);
-			gda_column_set_table (column, ...);
-			gda_column_set_primary_key (column, ...);
-			gda_column_set_unique_key (column, ...);
-			gda_column_set_allow_null (column, ...);
-			gda_column_set_auto_increment (column, ...);
-			*/
-
 			
 			mysql_bind_result[i].buffer_type = field->type;
 			switch (mysql_bind_result[i].buffer_type) {
@@ -417,6 +446,7 @@ gda_mysql_recordset_new (GdaConnection            *cnc,
         model = g_object_new (GDA_TYPE_MYSQL_RECORDSET,
 			      "prepared-stmt", ps,
 			      "model-usage", rflags,
+			      "exec-params", exec_params, 
 			      NULL);
         model->priv->cnc = cnc;
 	g_object_ref (G_OBJECT(cnc));
@@ -426,7 +456,7 @@ gda_mysql_recordset_new (GdaConnection            *cnc,
 	
 	model->priv->mysql_stmt = cdata->mysql_stmt;
 
-	((GdaPModel *) model)->advertized_nrows = mysql_stmt_affected_rows (cdata->mysql_stmt);
+	((GdaDataSelect *) model)->advertized_nrows = mysql_stmt_affected_rows (cdata->mysql_stmt);
 	
 
         return GDA_DATA_MODEL (model);
@@ -437,7 +467,7 @@ gda_mysql_recordset_new (GdaConnection            *cnc,
  * Get the number of rows in @model, if possible
  */
 static gint
-gda_mysql_recordset_fetch_nb_rows (GdaPModel *model)
+gda_mysql_recordset_fetch_nb_rows (GdaDataSelect *model)
 {
 	GdaMysqlRecordset *imodel;
 
@@ -455,23 +485,23 @@ gda_mysql_recordset_fetch_nb_rows (GdaPModel *model)
 }
 
 
-static GdaPRow *
+static GdaRow *
 new_row_from_mysql_stmt (GdaMysqlRecordset  *imodel,
 			 gint                rownum)
 {
-	g_print ("*** %s -- %d -- %d\n", __func__, ((GdaPModel *) imodel)->prep_stmt->ncols, rownum);
+	/* g_print ("*** %s -- %d -- %d\n", __func__, ((GdaDataSelect *) imodel)->prep_stmt->ncols, rownum); */
 	
-	MYSQL_BIND *mysql_bind_result = ((GdaMysqlPStmt *) ((GdaPModel *) imodel)->prep_stmt)->mysql_bind_result;
+	MYSQL_BIND *mysql_bind_result = ((GdaMysqlPStmt *) ((GdaDataSelect *) imodel)->prep_stmt)->mysql_bind_result;
 	g_assert (mysql_bind_result);
 	
-	GdaPRow *prow = gda_prow_new (((GdaPModel *) imodel)->prep_stmt->ncols);
+	GdaRow *row = gda_row_new (((GdaDataSelect *) imodel)->prep_stmt->ncols);
 	gint col;
-	for (col = 0; col < ((GdaPModel *) imodel)->prep_stmt->ncols; ++col) {
+	for (col = 0; col < ((GdaDataSelect *) imodel)->prep_stmt->ncols; ++col) {
 		
 		gint i = col;
 		
-		GValue *value = gda_prow_get_value (prow, i);
-		GType type = ((GdaPModel *) imodel)->prep_stmt->types[i];
+		GValue *value = gda_row_get_value (row, i);
+		GType type = ((GdaDataSelect *) imodel)->prep_stmt->types[i];
 		gda_value_reset_with_type (value, type);
 		
 		int intvalue = 0;
@@ -561,7 +591,7 @@ new_row_from_mysql_stmt (GdaMysqlRecordset  *imodel,
 				g_warning (_("Type %s not mapped for value %f"),
 					   g_type_name (type), intvalue);
 			}
-			setlocale (LC_NUMERIC, "");
+			setlocale (LC_NUMERIC, gda_numeric_locale);
 			
 			break;
 		case MYSQL_TYPE_STRING:
@@ -592,7 +622,7 @@ new_row_from_mysql_stmt (GdaMysqlRecordset  *imodel,
 			} else if (type == G_TYPE_DOUBLE) {
 				setlocale (LC_NUMERIC, "C");
 				g_value_set_double (value, atof (strvalue));
-				setlocale (LC_NUMERIC, "");
+				setlocale (LC_NUMERIC, gda_numeric_locale);
 			} else {
 				g_warning (_("Type %s not mapped for value %p"),
 					   g_type_name (type), strvalue);
@@ -607,32 +637,32 @@ new_row_from_mysql_stmt (GdaMysqlRecordset  *imodel,
 		g_free (strvalue);
 		
 		/* gchar *str = gda_value_stringify (value); */
-		/* g_print ("   V%d=%s\n", i, str); */
+		/* g_print ("***V%d=%s\n", i, str); */
 		/* g_free (str); */
-		//		
+
 	}
-	return prow;
+	return row;
 }
 
 
 /*
- * Create a new filled #GdaPRow object for the row at position @rownum, and put it into *prow.
+ * Create a new filled #GdaRow object for the row at position @rownum, and put it into *row.
  *
- * WARNING: @prow will NOT be NULL, but *prow may or may not be NULL:
- *  -  If *prow is NULL then a new #GdaPRow object has to be created, 
- *  -  and otherwise *prow contains a #GdaPRow object which has already been created 
+ * WARNING: @row will NOT be NULL, but *row may or may not be NULL:
+ *  -  If *row is NULL then a new #GdaRow object has to be created, 
+ *  -  and otherwise *row contains a #GdaRow object which has already been created 
  *     (through a call to this very function), and in this case it should not be modified
  *     but the function may return FALSE if an error occurred.
  *
- * Memory management for that new GdaPRow object is left to the implementation, which
- * can use gda_pmodel_take_row(). If new row objects are "given" to the GdaPModel implemantation
+ * Memory management for that new GdaRow object is left to the implementation, which
+ * can use gda_data_select_take_row(). If new row objects are "given" to the GdaDataSelect implemantation
  * using that method, then this method should detect when all the data model rows have been analysed
  * (when model->nb_stored_rows == model->advertized_nrows) and then possibly discard the API handle
  * as it won't be used anymore to fetch rows.
  */
 static gboolean 
-gda_mysql_recordset_fetch_random (GdaPModel  *model,
-				  GdaPRow   **prow,
+gda_mysql_recordset_fetch_random (GdaDataSelect  *model,
+				  GdaRow    **row,
 				  gint        rownum,
 				  GError    **error)
 {
@@ -642,7 +672,7 @@ gda_mysql_recordset_fetch_random (GdaPModel  *model,
 
 	// TO_IMPLEMENT;
 	
-	if (*prow)
+	if (*row)
 		return TRUE;
 
 	if (imodel->priv->mysql_stmt == NULL) {
@@ -655,22 +685,24 @@ gda_mysql_recordset_fetch_random (GdaPModel  *model,
 	if (mysql_stmt_fetch (imodel->priv->mysql_stmt))
 		return FALSE;
 	
-	*prow = new_row_from_mysql_stmt (imodel, rownum);
-	gda_pmodel_take_row (model, *prow, rownum);
-	
-	/* if (model->nb_stored_rows == model->advertized_nrows) { */
-	/* 	g_print ("  All the row have been converted..."); */
-	/* } */
+	*row = new_row_from_mysql_stmt (imodel, rownum);
+	gda_data_select_take_row (model, *row, rownum);
 	
+	if (model->nb_stored_rows == model->advertized_nrows) {
+		/* All the row have been converted.  We could free result now */
+		/* but it was done before provided no field-based API functions */
+		/* that process result set meta data was needed in the middle. */
+	}
 
 	return TRUE;
 }
 
 /*
- * Create and "give" filled #GdaPRow object for all the rows in the model
+ * Create and "give" filled #GdaRow object for all the rows in the model
  */
 static gboolean
-gda_mysql_recordset_store_all (GdaPModel *model, GError **error)
+gda_mysql_recordset_store_all (GdaDataSelect  *model,
+			       GError    **error)
 {
 	GdaMysqlRecordset *imodel;
 	gint i;
@@ -679,27 +711,57 @@ gda_mysql_recordset_store_all (GdaPModel *model, GError **error)
 
 	/* default implementation */
 	for (i = 0; i < model->advertized_nrows; i++) {
-		GdaPRow *prow;
-		if (! gda_mysql_recordset_fetch_random (model, &prow, i, error))
+		GdaRow *row;
+		if (! gda_mysql_recordset_fetch_random (model, &row, i, error))
 			return FALSE;
 	}
 	return TRUE;
 }
 
 /*
- * Create a new filled #GdaPRow object for the next cursor row, and put it into *prow.
+ * Create a new filled #GdaRow object for the next cursor row, and put it into *row.
  *
- * WARNING: @prow will NOT be NULL, but *prow may or may not be NULL:
- *  -  If *prow is NULL then a new #GdaPRow object has to be created, 
- *  -  and otherwise *prow contains a #GdaPRow object which has already been created 
+ * WARNING: @row will NOT be NULL, but *row may or may not be NULL:
+ *  -  If *row is NULL then a new #GdaRow object has to be created, 
+ *  -  and otherwise *row contains a #GdaRow object which has already been created 
  *     (through a call to this very function), and in this case it should not be modified
  *     but the function may return FALSE if an error occurred.
  *
- * Memory management for that new GdaPRow object is left to the implementation, which
- * can use gda_pmodel_take_row().
+ * Memory management for that new GdaRow object is left to the implementation, which
+ * can use gda_data_select_take_row().
  */
 static gboolean 
-gda_mysql_recordset_fetch_next (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
+gda_mysql_recordset_fetch_next (GdaDataSelect  *model,
+				GdaRow    **row,
+				gint        rownum,
+				GError    **error)
+{
+	GdaMysqlRecordset *imodel = (GdaMysqlRecordset*) model;
+
+	// TO_IMPLEMENT;
+	//
+
+	// gda_data_select_iter_next increments rownum
+	return /* TRUE */ gda_mysql_recordset_fetch_random (model, row, rownum, error);
+}
+
+/*
+ * Create a new filled #GdaRow object for the previous cursor row, and put it into *row.
+ *
+ * WARNING: @row will NOT be NULL, but *row may or may not be NULL:
+ *  -  If *row is NULL then a new #GdaRow object has to be created, 
+ *  -  and otherwise *row contains a #GdaRow object which has already been created 
+ *     (through a call to this very function), and in this case it should not be modified
+ *     but the function may return FALSE if an error occurred.
+ *
+ * Memory management for that new GdaRow object is left to the implementation, which
+ * can use gda_data_select_take_row().
+ */
+static gboolean 
+gda_mysql_recordset_fetch_prev (GdaDataSelect  *model,
+				GdaRow    **row,
+				gint        rownum,
+				GError    **error)
 {
 	GdaMysqlRecordset *imodel = (GdaMysqlRecordset*) model;
 
@@ -709,41 +771,22 @@ gda_mysql_recordset_fetch_next (GdaPModel *model, GdaPRow **prow, gint rownum, G
 }
 
 /*
- * Create a new filled #GdaPRow object for the previous cursor row, and put it into *prow.
+ * Create a new filled #GdaRow object for the cursor row at position @rownum, and put it into *row.
  *
- * WARNING: @prow will NOT be NULL, but *prow may or may not be NULL:
- *  -  If *prow is NULL then a new #GdaPRow object has to be created, 
- *  -  and otherwise *prow contains a #GdaPRow object which has already been created 
+ * WARNING: @row will NOT be NULL, but *row may or may not be NULL:
+ *  -  If *row is NULL then a new #GdaRow object has to be created, 
+ *  -  and otherwise *row contains a #GdaRow object which has already been created 
  *     (through a call to this very function), and in this case it should not be modified
  *     but the function may return FALSE if an error occurred.
  *
- * Memory management for that new GdaPRow object is left to the implementation, which
- * can use gda_pmodel_take_row().
+ * Memory management for that new GdaRow object is left to the implementation, which
+ * can use gda_data_select_take_row().
  */
 static gboolean 
-gda_mysql_recordset_fetch_prev (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
-{
-	GdaMysqlRecordset *imodel = (GdaMysqlRecordset*) model;
-
-	TO_IMPLEMENT;
-
-	return TRUE;
-}
-
-/*
- * Create a new filled #GdaPRow object for the cursor row at position @rownum, and put it into *prow.
- *
- * WARNING: @prow will NOT be NULL, but *prow may or may not be NULL:
- *  -  If *prow is NULL then a new #GdaPRow object has to be created, 
- *  -  and otherwise *prow contains a #GdaPRow object which has already been created 
- *     (through a call to this very function), and in this case it should not be modified
- *     but the function may return FALSE if an error occurred.
- *
- * Memory management for that new GdaPRow object is left to the implementation, which
- * can use gda_pmodel_take_row().
- */
-static gboolean 
-gda_mysql_recordset_fetch_at (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
+gda_mysql_recordset_fetch_at (GdaDataSelect  *model,
+			      GdaRow    **row,
+			      gint        rownum,
+			      GError    **error)
 {
 	GdaMysqlRecordset *imodel = (GdaMysqlRecordset*) model;
 	
