@@ -116,7 +116,7 @@ static GObject             *gda_postgres_provider_statement_execute (GdaServerPr
 								     GdaStatement *stmt, GdaSet *params,
 								     GdaStatementModelUsage model_usage, 
 								     GType *col_types, GdaSet **last_inserted_row, 
-								     guint *task_id, GdaServerProviderAsyncCallback async_cb, 
+								     guint *task_id, GdaServerProviderExecCallback async_cb, 
 								     gpointer cb_data, GError **error);
 
 /* distributed transactions */
@@ -147,7 +147,7 @@ static void gda_postgres_free_cnc_data (PostgresConnectionData *cdata);
  * TO_ADD: any prepared statement to be used internally by the provider should be
  *         declared here, as constants and as SQL statements
  */
-GdaStatement **internal_stmt;
+static GdaStatement **internal_stmt;
 
 typedef enum {
 	I_STMT_BEGIN,
@@ -159,7 +159,7 @@ typedef enum {
 	I_STMT_XA_RECOVER
 } InternalStatementItem;
 
-gchar *internal_sql[] = {
+static gchar *internal_sql[] = {
 	"BEGIN",
 	"COMMIT",
 	"ROLLBACK",
@@ -260,6 +260,16 @@ gda_postgres_provider_class_init (GdaPostgresProviderClass *klass)
 	provider_class->xa_funcs->xa_commit = gda_postgres_provider_xa_commit;
 	provider_class->xa_funcs->xa_rollback = gda_postgres_provider_xa_rollback;
 	provider_class->xa_funcs->xa_recover = gda_postgres_provider_xa_recover;
+
+	/* If PostgreSQL was not compiled with the --enable-thread-safety flag, then libPQ is not
+	 * considered thread safe, and we limit the usage of the provider from the current thread */
+	if (! PQisthreadsafe ()) {
+		gda_log_message ("PostgreSQL was not compiled with the --enable-thread-safety flag, "
+				 "only one thread can access the provider");
+		provider_class->limiting_thread = g_thread_self ();
+	}
+	else
+		provider_class->limiting_thread = NULL;
 }
 
 static void
@@ -774,7 +784,7 @@ gda_postgres_provider_close_connection (GdaServerProvider *provider, GdaConnecti
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	/* Close the connection using the C API */
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
@@ -800,7 +810,7 @@ gda_postgres_provider_get_server_version (GdaServerProvider *provider, GdaConnec
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -820,7 +830,7 @@ gda_postgres_provider_get_database (GdaServerProvider *provider, GdaConnection *
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -846,7 +856,7 @@ gda_postgres_provider_supports_operation (GdaServerProvider *provider, GdaConnec
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	switch (type) {
@@ -888,7 +898,7 @@ gda_postgres_provider_create_operation (GdaServerProvider *provider, GdaConnecti
 
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
         file = g_utf8_strdown (gda_server_operation_op_type_to_string (type), -1);
@@ -925,7 +935,7 @@ gda_postgres_provider_render_operation (GdaServerProvider *provider, GdaConnecti
 
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	/* test @op's validity */
@@ -1008,7 +1018,7 @@ gda_postgres_provider_perform_operation (GdaServerProvider *provider, GdaConnect
 
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
         optype = gda_server_operation_get_op_type (op);
 	if (!cnc && 
@@ -1114,7 +1124,7 @@ gda_postgres_provider_begin_transaction (GdaServerProvider *provider, GdaConnect
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1198,7 +1208,7 @@ gda_postgres_provider_commit_transaction (GdaServerProvider *provider, GdaConnec
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1222,7 +1232,7 @@ gda_postgres_provider_rollback_transaction (GdaServerProvider *provider,
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1245,7 +1255,7 @@ gda_postgres_provider_add_savepoint (GdaServerProvider *provider, GdaConnection
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1298,7 +1308,7 @@ gda_postgres_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnec
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1351,7 +1361,7 @@ gda_postgres_provider_delete_savepoint (GdaServerProvider *provider, GdaConnecti
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 
 	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1402,7 +1412,7 @@ gda_postgres_provider_supports_feature (GdaServerProvider *provider, GdaConnecti
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	switch (feature) {
@@ -1462,7 +1472,7 @@ gda_postgres_provider_get_data_handler (GdaServerProvider *provider, GdaConnecti
 	GdaDataHandler *dh;
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	if (type == G_TYPE_INVALID) {
@@ -1511,7 +1521,7 @@ gda_postgres_provider_get_default_dbms_type (GdaServerProvider *provider, GdaCon
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	}
 
 	if (type == G_TYPE_INT64)
@@ -1594,7 +1604,7 @@ gda_postgres_provider_statement_to_sql (GdaServerProvider *provider, GdaConnecti
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), NULL);
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 	}
 
 	return gda_statement_to_sql_extended (stmt, cnc, params, flags, params_used, error);
@@ -1615,11 +1625,11 @@ gda_postgres_provider_statement_prepare (GdaServerProvider *provider, GdaConnect
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), FALSE);
 
 	/* fetch prepares stmt if already done */
-	ps = gda_connection_get_prepared_statement (cnc, stmt);
+	ps = (GdaPostgresPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
 	if (ps)
 		return TRUE;
 
@@ -1684,7 +1694,7 @@ gda_postgres_provider_statement_prepare (GdaServerProvider *provider, GdaConnect
         _GDA_PSTMT (ps)->param_ids = param_ids;
         _GDA_PSTMT (ps)->sql = sql;
 	
-	gda_connection_add_prepared_statement (cnc, stmt, ps);
+	gda_connection_add_prepared_statement (cnc, stmt, (GdaPStmt *) ps);
 	return TRUE;
 
  out_err:
@@ -1791,7 +1801,7 @@ make_last_inserted_set (GdaConnection *cnc, GdaStatement *stmt, Oid last_id)
 	where = gda_sql_expr_new (GDA_SQL_ANY_PART (select));
 	cond = gda_sql_operation_new (GDA_SQL_ANY_PART (where));
 	where->cond = cond;
-	cond->operator = GDA_SQL_OPERATOR_TYPE_EQ;
+	cond->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
 	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (cond));
 	g_value_set_string ((value = gda_value_new (G_TYPE_STRING)), "oid");
 	expr->value = value;
@@ -1828,7 +1838,7 @@ make_last_inserted_set (GdaConnection *cnc, GdaStatement *stmt, Oid last_id)
 		return NULL;
         }
 	else {
-		GdaSet *set;
+		GdaSet *set = NULL;
 		GSList *holders = NULL;
 		gint nrows, ncols, i;
 
@@ -1847,20 +1857,33 @@ make_last_inserted_set (GdaConnection *cnc, GdaStatement *stmt, Oid last_id)
 			GdaHolder *h;
 			GdaColumn *col;
 			gchar *id;
+			const GValue *cvalue;
+
 			col = gda_data_model_describe_column (model, i);
 			h = gda_holder_new (gda_column_get_g_type (col));
 			id = g_strdup_printf ("+%d", i);
-			g_object_set (G_OBJECT (h), "id", id, "not-null", FALSE, NULL);
+			g_object_set (G_OBJECT (h), "id", id, "not-null", FALSE,
+				      "name", gda_column_get_name (col), NULL);
 			g_free (id);
-			gda_holder_set_value (h, gda_data_model_get_value_at (model, i, 0));
+			cvalue = gda_data_model_get_value_at (model, i, 0, NULL);
+			if (!cvalue || !gda_holder_set_value (h, cvalue, NULL)) {
+				if (holders) {
+					g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
+					g_slist_free (holders);
+					holders = NULL;
+				}
+				break;
+			}
 			holders = g_slist_prepend (holders, h);
 		}
 		g_object_unref (model);
 
-		holders = g_slist_reverse (holders);
-		set = gda_set_new (holders);
-		g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
-		g_slist_free (holders);
+		if (holders) {
+			holders = g_slist_reverse (holders);
+			set = gda_set_new (holders);
+			g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
+			g_slist_free (holders);
+		}
 
 		return set;
 	}
@@ -1885,13 +1908,13 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 					 GdaStatementModelUsage model_usage, 
 					 GType *col_types, GdaSet **last_inserted_row, 
 					 guint *task_id, 
-					 GdaServerProviderAsyncCallback async_cb, gpointer cb_data, GError **error)
+					 GdaServerProviderExecCallback async_cb, gpointer cb_data, GError **error)
 {
 	GdaPostgresPStmt *ps;
 	PostgresConnectionData *cdata;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), NULL);
 
 	/* If asynchronous connection opening is not supported, then exit now */
@@ -1952,13 +1975,13 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 		PQclear (pg_res);
 		
 		/* create data model in CURSOR mode */
-		recset = gda_postgres_recordset_new_cursor (cnc, ps, cursor_name, col_types);
+		recset = gda_postgres_recordset_new_cursor (cnc, ps, params, cursor_name, col_types);
 		gda_connection_internal_statement_executed (cnc, stmt, params, NULL); /* required: help @cnc keep some stats */
 		return (GObject*) recset;
 	}
 
 	/* get/create new prepared statement */
-	ps = gda_connection_get_prepared_statement (cnc, stmt);
+	ps = (GdaPostgresPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
 	if (!ps) {
 		if (!gda_postgres_provider_statement_prepare (provider, cnc, stmt, NULL)) {
 			/* this case can appear for example if some variables are used in places
@@ -1977,7 +2000,7 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 				return NULL;
 		}
 		else
-			ps = gda_connection_get_prepared_statement (cnc, stmt);
+			ps = (GdaPostgresPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
 	}
 	g_assert (ps);
 
@@ -2027,6 +2050,16 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 			g_free (str);
 			break;
 		}
+		if (!gda_holder_is_valid (h)) {
+			gchar *str;
+			str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
+			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+			gda_connection_event_set_description (event, str);
+			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
+			g_free (str);
+			break;
+		}
 
 		/* actual binding using the C API, for parameter at position @i */
 		const GValue *value = gda_holder_get_value (h);
@@ -2134,7 +2167,7 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
                                 PQclear (pg_res);
                         }
                         else if (status == PGRES_TUPLES_OK) 
-				retval = (GObject*) gda_postgres_recordset_new_random (cnc, ps, pg_res, col_types);
+				retval = (GObject*) gda_postgres_recordset_new_random (cnc, ps, params, pg_res, col_types);
                         else {
                                 PQclear (pg_res);
                                 retval = (GObject *) gda_data_model_array_new (0);
@@ -2156,7 +2189,7 @@ gda_postgres_provider_xa_start (GdaServerProvider *provider, GdaConnection *cnc,
 				const GdaXaTransactionId *xid, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	return gda_postgres_provider_begin_transaction (provider, cnc, NULL, 
@@ -2172,7 +2205,7 @@ gda_postgres_provider_xa_end (GdaServerProvider *provider, GdaConnection *cnc,
 			      const GdaXaTransactionId *xid, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	/* nothing to do for PostgreSQL here */
@@ -2191,13 +2224,13 @@ gda_postgres_provider_xa_prepare (GdaServerProvider *provider, GdaConnection *cn
 	gchar *str;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	if (!gda_statement_get_parameters (internal_stmt [I_STMT_XA_PREPARE], &params, error))
 		return FALSE;
 	str = gda_xa_transaction_id_to_string (xid);
-	if (!gda_set_set_holder_value (params, "xid", str)) {
+	if (!gda_set_set_holder_value (params, NULL, "xid", str)) {
 		g_free (str);
 		g_object_unref (params);
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_INTERNAL_ERROR,
@@ -2227,13 +2260,13 @@ gda_postgres_provider_xa_commit (GdaServerProvider *provider, GdaConnection *cnc
 	gchar *str;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	if (!gda_statement_get_parameters (internal_stmt [I_STMT_XA_PREPARE], &params, error))
 		return FALSE;
 	str = gda_xa_transaction_id_to_string (xid);
-	if (!gda_set_set_holder_value (params, "xid", str)) {
+	if (!gda_set_set_holder_value (params, NULL, "xid", str)) {
 		g_free (str);
 		g_object_unref (params);
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_INTERNAL_ERROR,
@@ -2262,13 +2295,13 @@ gda_postgres_provider_xa_rollback (GdaServerProvider *provider, GdaConnection *c
 	gchar *str;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
 	g_return_val_if_fail (xid, FALSE);
 
 	if (!gda_statement_get_parameters (internal_stmt [I_STMT_XA_PREPARE], &params, error))
 		return FALSE;
 	str = gda_xa_transaction_id_to_string (xid);
-	if (!gda_set_set_holder_value (params, "xid", str)) {
+	if (!gda_set_set_holder_value (params, NULL, "xid", str)) {
 		g_free (str);
 		g_object_unref (params);
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_INTERNAL_ERROR,
@@ -2297,7 +2330,7 @@ gda_postgres_provider_xa_recover (GdaServerProvider *provider, GdaConnection *cn
 	GdaDataModel *model;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
 
 	model = gda_connection_statement_execute_select (cnc, internal_stmt [I_STMT_XA_RECOVER], NULL, error);
 	if (!model)
@@ -2308,7 +2341,7 @@ gda_postgres_provider_xa_recover (GdaServerProvider *provider, GdaConnection *cn
 		
 		nrows = gda_data_model_get_n_rows (model);
 		for (i = 0; i < nrows; i++) {
-			const GValue *cvalue = gda_data_model_get_value_at (model, 0, i);
+			const GValue *cvalue = gda_data_model_get_value_at (model, 0, i, NULL);
 			if (cvalue && !gda_value_is_null (cvalue))
 				list = g_list_prepend (list, 
 						       gda_xa_transaction_string_to_id (g_value_get_string (cvalue)));
