@@ -34,6 +34,8 @@
 #include <locale.h>
 #endif
 
+extern gchar *gda_numeric_locale;
+
 #define _GDA_PSTMT(x) ((GdaPStmt*)(x))
 
 static void gda_postgres_recordset_class_init (GdaPostgresRecordsetClass *klass);
@@ -51,21 +53,19 @@ static void gda_postgres_recordset_get_property (GObject *object,
 						 GParamSpec *pspec);
 
 /* virtual methods */
-static gint     gda_postgres_recordset_fetch_nb_rows (GdaPModel *model);
-static gboolean gda_postgres_recordset_fetch_random (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error);
-static gboolean gda_postgres_recordset_store_all (GdaPModel *model, GError **error);
-static gboolean gda_postgres_recordset_fetch_next (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error);
-static gboolean gda_postgres_recordset_fetch_prev (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error);
-static gboolean gda_postgres_recordset_fetch_at (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error);
+static gint     gda_postgres_recordset_fetch_nb_rows (GdaDataSelect *model);
+static gboolean gda_postgres_recordset_fetch_random (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error);
+static gboolean gda_postgres_recordset_store_all (GdaDataSelect *model, GError **error);
+static gboolean gda_postgres_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error);
+static gboolean gda_postgres_recordset_fetch_prev (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error);
+static gboolean gda_postgres_recordset_fetch_at (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error);
 
 /* static helper functions */
 static void make_point (GdaGeometricPoint *point, const gchar *value);
-static void make_time (GdaTime *timegda, const gchar *value);
-static void make_timestamp (GdaTimestamp *timestamp, const gchar *value);
 static void set_value (GdaConnection *cnc, GValue *value, GType type, const gchar *thevalue, gint length);
 
-static void     set_prow_with_pg_res (GdaPostgresRecordset *imodel, GdaPRow *prow, gint pg_res_rownum);
-static GdaPRow *new_row_from_pg_res (GdaPostgresRecordset *imodel, gint pg_res_rownum);
+static void     set_prow_with_pg_res (GdaPostgresRecordset *imodel, GdaRow *prow, gint pg_res_rownum);
+static GdaRow *new_row_from_pg_res (GdaPostgresRecordset *imodel, gint pg_res_rownum);
 static gboolean row_is_in_current_pg_res (GdaPostgresRecordset *model, gint row);
 static gboolean fetch_next_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **error);
 static gboolean fetch_prev_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **error);
@@ -77,7 +77,7 @@ struct _GdaPostgresRecordsetPrivate {
 	PGresult         *pg_res;
 
 	/* cursor access attributes */
-	GdaPRow          *tmp_row; /* used to store a reference to the last #GdaPRow returned */
+	GdaRow           *tmp_row; /* used to store a reference to the last #GdaRow returned */
 	gchar            *cursor_name;
 	PGconn           *pconn;
 	gint              chunk_size; /* Number of rows to fetch at a time when iterating forward or backwards. */
@@ -119,7 +119,7 @@ static void
 gda_postgres_recordset_class_init (GdaPostgresRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaPModelClass *pmodel_class = GDA_PMODEL_CLASS (klass);
+	GdaDataSelectClass *pmodel_class = GDA_DATA_SELECT_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -136,11 +136,11 @@ gda_postgres_recordset_class_init (GdaPostgresRecordsetClass *klass)
         object_class->set_property = gda_postgres_recordset_set_property;
         object_class->get_property = gda_postgres_recordset_get_property;
         g_object_class_install_property (object_class, PROP_CHUNCK_SIZE,
-                                         g_param_spec_int ("chunk_size", _("Number of rows fetched at a time"), NULL,
+                                         g_param_spec_int ("chunk-size", _("Number of rows fetched at a time"), NULL,
                                                            1, G_MAXINT - 1, 10,
                                                            G_PARAM_CONSTRUCT | G_PARAM_READABLE | G_PARAM_WRITABLE));
         g_object_class_install_property (object_class, PROP_CHUNCKS_READ,
-                                         g_param_spec_int ("chunks_read",
+                                         g_param_spec_int ("chunks-read",
                                                            _("Number of rows chunks read since the object creation"), NULL,
                                                            0, G_MAXINT - 1, 0,
                                                            G_PARAM_READABLE));
@@ -240,7 +240,7 @@ gda_postgres_recordset_get_type (void)
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
-			type = g_type_register_static (GDA_TYPE_PMODEL, "GdaPostgresRecordset", &info, 0);
+			type = g_type_register_static (GDA_TYPE_DATA_SELECT, "GdaPostgresRecordset", &info, 0);
 		g_static_mutex_unlock (&registering);
 	}
 
@@ -276,7 +276,7 @@ finish_prep_stmt_init (PostgresConnectionData *cdata, GdaPostgresPStmt *ps, PGre
 					if (col_types [i] == G_TYPE_NONE)
 						break;
 					if (i >= _GDA_PSTMT (ps)->ncols)
-						g_warning (_("Column %d is out of range (0-%d), ignoring its specified type"), i,
+						g_warning (_("Column %d out of range (0-%d), ignoring its specified type"), i,
 							   _GDA_PSTMT (ps)->ncols - 1);
 					else
 						_GDA_PSTMT (ps)->types [i] = col_types [i];
@@ -302,26 +302,13 @@ finish_prep_stmt_init (PostgresConnectionData *cdata, GdaPostgresPStmt *ps, PGre
 			gda_column_set_g_type (column, gtype);
 			gda_column_set_name (column, PQfname (pg_res, i));
 			gda_column_set_title (column, PQfname (pg_res, i));
-			gda_column_set_scale (column, (gtype == G_TYPE_DOUBLE) ? DBL_DIG :
-					      (gtype == G_TYPE_FLOAT) ? FLT_DIG : 0);
-			gda_column_set_defined_size (column, PQfsize (pg_res, i));
-			gda_column_set_references (column, "");
-
-			/*
-			  FIXME: Use @cnc's associated GdaMetaStore to get the following information:
-			  gda_column_set_references (column, "");
-			  gda_column_set_table (column, ...);
-			  gda_column_set_primary_key (column, ...);
-			  gda_column_set_unique_key (column, ...);
-			  gda_column_set_allow_null (column, ...);
-			  gda_column_set_auto_increment (column, ...);
-			*/
 		}
         }
 }
 
 GdaDataModel *
-gda_postgres_recordset_new_random (GdaConnection *cnc, GdaPostgresPStmt *ps, PGresult *pg_res, GType *col_types)
+gda_postgres_recordset_new_random (GdaConnection *cnc, GdaPostgresPStmt *ps, GdaSet *exec_params, 
+				   PGresult *pg_res, GType *col_types)
 {
 	GdaPostgresRecordset *model;
         PostgresConnectionData *cdata;
@@ -339,9 +326,10 @@ gda_postgres_recordset_new_random (GdaConnection *cnc, GdaPostgresPStmt *ps, PGr
 	/* create data model */
         model = g_object_new (GDA_TYPE_POSTGRES_RECORDSET, "connection", cnc, 
 			      "prepared-stmt", ps, 
-			      "model-usage", GDA_DATA_MODEL_ACCESS_RANDOM, NULL);
+			      "model-usage", GDA_DATA_MODEL_ACCESS_RANDOM, 
+			      "exec-params", exec_params, NULL);
 	model->priv->pg_res = pg_res;
-	((GdaPModel*) model)->advertized_nrows = PQntuples (model->priv->pg_res);
+	((GdaDataSelect*) model)->advertized_nrows = PQntuples (model->priv->pg_res);
 
         return GDA_DATA_MODEL (model);
 }
@@ -350,7 +338,8 @@ gda_postgres_recordset_new_random (GdaConnection *cnc, GdaPostgresPStmt *ps, PGr
  * Takes ownership of @cursor_name
  */
 GdaDataModel *
-gda_postgres_recordset_new_cursor (GdaConnection *cnc, GdaPostgresPStmt *ps, gchar *cursor_name, GType *col_types)
+gda_postgres_recordset_new_cursor (GdaConnection *cnc, GdaPostgresPStmt *ps, GdaSet *exec_params,
+				   gchar *cursor_name, GType *col_types)
 {
 	GdaPostgresRecordset *model;
         PostgresConnectionData *cdata;
@@ -395,7 +384,8 @@ gda_postgres_recordset_new_cursor (GdaConnection *cnc, GdaPostgresPStmt *ps, gch
 	/* create model */
 	model = g_object_new (GDA_TYPE_POSTGRES_RECORDSET, "connection", cnc, 
 			      "prepared-stmt", ps, "model-usage", 
-			      GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD | GDA_DATA_MODEL_ACCESS_CURSOR_BACKWARD, NULL);
+			      GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD | GDA_DATA_MODEL_ACCESS_CURSOR_BACKWARD, 
+			      "exec-params", exec_params, NULL);
 	model->priv->pconn = cdata->pconn;
 	model->priv->cursor_name = cursor_name;
 	gboolean fetch_error;
@@ -408,7 +398,7 @@ gda_postgres_recordset_new_cursor (GdaConnection *cnc, GdaPostgresPStmt *ps, gch
  * Get the number of rows in @model, if possible
  */
 static gint
-gda_postgres_recordset_fetch_nb_rows (GdaPModel *model)
+gda_postgres_recordset_fetch_nb_rows (GdaDataSelect *model)
 {
 	GdaPostgresRecordset *imodel;
 
@@ -424,12 +414,12 @@ gda_postgres_recordset_fetch_nb_rows (GdaPModel *model)
 }
 
 /*
- * Create a new filled #GdaPRow object for the row at position @rownum.
+ * Create a new filled #GdaRow object for the row at position @rownum.
  *
- * Each new #GdaPRow created is "given" to the #GdaPModel implementation using gda_pmodel_take_row ().
+ * Each new #GdaRow created is "given" to the #GdaDataSelect implementation using gda_data_select_take_row ().
  */
 static gboolean
-gda_postgres_recordset_fetch_random (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
+gda_postgres_recordset_fetch_random (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset *) model;
 
@@ -443,10 +433,10 @@ gda_postgres_recordset_fetch_random (GdaPModel *model, GdaPRow **prow, gint rown
 	}
 
 	*prow = new_row_from_pg_res (imodel, rownum);
-	gda_pmodel_take_row (model, *prow, rownum);
+	gda_data_select_take_row (model, *prow, rownum);
 
 	if (model->nb_stored_rows == model->advertized_nrows) {
-		/* all the rows have been converted from PGresult to GdaPRow objects => we can
+		/* all the rows have been converted from PGresult to GdaRow objects => we can
 		 * discard the PGresult */
 		PQclear (imodel->priv->pg_res);
 		imodel->priv->pg_res = NULL;
@@ -456,10 +446,10 @@ gda_postgres_recordset_fetch_random (GdaPModel *model, GdaPRow **prow, gint rown
 }
 
 /*
- * Create and "give" filled #GdaPRow object for all the rows in the model
+ * Create and "give" filled #GdaRow object for all the rows in the model
  */
 static gboolean
-gda_postgres_recordset_store_all (GdaPModel *model, GError **error)
+gda_postgres_recordset_store_all (GdaDataSelect *model, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 	gint i;
@@ -471,7 +461,7 @@ gda_postgres_recordset_store_all (GdaPModel *model, GError **error)
 	}
 
 	for (i = 0; i < model->advertized_nrows; i++) {
-		GdaPRow *prow;
+		GdaRow *prow;
 		if (! gda_postgres_recordset_fetch_random (model, &prow, i, error))
 			return FALSE;
 	}
@@ -479,13 +469,13 @@ gda_postgres_recordset_store_all (GdaPModel *model, GError **error)
 }
 
 /*
- * Create a new filled #GdaPRow object for the next cursor row
+ * Create a new filled #GdaRow object for the next cursor row
  *
- * Each new #GdaPRow created is referenced only by imodel->priv->tmp_row (the #GdaPModel implementation
- * never keeps a reference to it). Before a new #GdaPRow gets created, the previous one, if set, is discarded.
+ * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
+ * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
  */
 static gboolean
-gda_postgres_recordset_fetch_next (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
+gda_postgres_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
@@ -516,13 +506,13 @@ gda_postgres_recordset_fetch_next (GdaPModel *model, GdaPRow **prow, gint rownum
 }
 
 /*
- * Create a new filled #GdaPRow object for the previous cursor row
+ * Create a new filled #GdaRow object for the previous cursor row
  *
- * Each new #GdaPRow created is referenced only by imodel->priv->tmp_row (the #GdaPModel implementation
- * never keeps a reference to it). Before a new #GdaPRow gets created, the previous one, if set, is discarded.
+ * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
+ * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
  */
 static gboolean
-gda_postgres_recordset_fetch_prev (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
+gda_postgres_recordset_fetch_prev (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
@@ -553,13 +543,13 @@ gda_postgres_recordset_fetch_prev (GdaPModel *model, GdaPRow **prow, gint rownum
 }
 
 /*
- * Create a new filled #GdaPRow object for the cursor row at position @rownum
+ * Create a new filled #GdaRow object for the cursor row at position @rownum
  *
- * Each new #GdaPRow created is referenced only by imodel->priv->tmp_row (the #GdaPModel implementation
- * never keeps a reference to it). Before a new #GdaPRow gets created, the previous one, if set, is discarded.
+ * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
+ * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
  */
 static gboolean
-gda_postgres_recordset_fetch_at (GdaPModel *model, GdaPRow **prow, gint rownum, GError **error)
+gda_postgres_recordset_fetch_at (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaPostgresRecordset *imodel = (GdaPostgresRecordset*) model;
 
@@ -608,72 +598,6 @@ make_point (GdaGeometricPoint *point, const gchar *value)
 	point->y = atof (value);
 }
 
-/* Makes a GdaTime from a string like "12:30:15+01" */
-static void
-make_time (GdaTime *timegda, const gchar *value)
-{
-	timegda->hour = atoi (value);
-	value += 3;
-	timegda->minute = atoi (value);
-	value += 3;
-	timegda->second = atoi (value);
-	value += 2;
-	if (*value)
-		timegda->timezone = atoi (value);
-	else
-		timegda->timezone = GDA_TIMEZONE_INVALID;
-}
-
-/* Makes a GdaTimestamp from a string like "2003-12-13 13:12:01.12+01" */
-static void
-make_timestamp (GdaTimestamp *timestamp, const gchar *value)
-{
-	timestamp->year = atoi (value);
-	value += 5;
-	timestamp->month = atoi (value);
-	value += 3;
-	timestamp->day = atoi (value);
-	value += 3;
-	timestamp->hour = atoi (value);
-	value += 3;
-	timestamp->minute = atoi (value);
-	value += 3;
-	timestamp->second = atoi (value);
-	value += 2;
-	if (*value != '.') {
-		timestamp->fraction = 0;
-	} else {
-		gint ndigits = 0;
-		gint64 fraction;
-
-		value++;
-		fraction = atol (value);
-		while (*value && *value != '+') {
-			value++;
-			ndigits++;
-		}
-
-		while (ndigits < 3) {
-			fraction *= 10;
-			ndigits++;
-		}
-
-		while (fraction > 0 && ndigits > 3) {
-			fraction /= 10;
-			ndigits--;
-		}
-		
-		timestamp->fraction = fraction;
-	}
-
-	if (*value != '+') {
-		timestamp->timezone = 0;
-	} else {
-		value++;
-		timestamp->timezone = atol (value) * 60 * 60;
-	}
-}
-
 static void
 set_value (GdaConnection *cnc, GValue *value, GType type,
 	   const gchar *thevalue, gint length)
@@ -697,12 +621,12 @@ set_value (GdaConnection *cnc, GValue *value, GType type,
 	else if (type == G_TYPE_FLOAT) {
 		setlocale (LC_NUMERIC, "C");
 		g_value_set_float (value, atof (thevalue));
-		setlocale (LC_NUMERIC, "");
+		setlocale (LC_NUMERIC, gda_numeric_locale);
 	}
 	else if (type == G_TYPE_DOUBLE) {
 		setlocale (LC_NUMERIC, "C");
 		g_value_set_double (value, atof (thevalue));
-		setlocale (LC_NUMERIC, "");
+		setlocale (LC_NUMERIC, gda_numeric_locale);
 	}
 	else if (type == GDA_TYPE_NUMERIC) {
 		GdaNumeric numeric;
@@ -713,15 +637,11 @@ set_value (GdaConnection *cnc, GValue *value, GType type,
 		g_free (numeric.number);
 	}
 	else if (type == G_TYPE_DATE) {
-		GDate *gdate;
-		gdate = g_date_new ();
-		g_date_set_parse (gdate, thevalue);
-		if (!g_date_valid (gdate)) {
-			g_warning (_("Could not parse date '%s', assuming 01/01/0001"), thevalue);
-			g_date_clear (gdate, 1);
-			g_date_set_dmy (gdate, 1, 1, 1);
-		}
-		g_value_take_boxed (value, gdate);
+		GDate date;
+		if (!gda_parse_iso8601_date (&date, thevalue)) 
+			g_warning (_("Invalid date '%s' (date format should be YYYY-MM-DD)"), thevalue);
+		else
+			g_value_set_boxed (value, &date);
 	}
 	else if (type == GDA_TYPE_GEOMETRIC_POINT) {
 		GdaGeometricPoint point;
@@ -730,13 +650,18 @@ set_value (GdaConnection *cnc, GValue *value, GType type,
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
 		GdaTimestamp timestamp;
-		make_timestamp (&timestamp, thevalue);
-		gda_value_set_timestamp (value, &timestamp);
+		if (! gda_parse_iso8601_timestamp (&timestamp, thevalue))
+			g_warning (_("Invalid timestamp '%s' (format should be YYYY-MM-DD HH:MM:SS[.ms])"), 
+				   thevalue);
+		else
+			gda_value_set_timestamp (value, &timestamp);
 	}
 	else if (type == GDA_TYPE_TIME) {
 		GdaTime timegda;
-		make_time (&timegda, thevalue);
-		gda_value_set_time (value, &timegda);
+		if (!gda_parse_iso8601_time (&timegda, thevalue)) 
+			g_warning (_("Invalid time '%s' (time format should be HH:MM:SS[.ms])"), thevalue);
+		else
+			gda_value_set_time (value, &timegda);
 	}
 	else if (type == GDA_TYPE_BINARY) {
 		/*
@@ -784,30 +709,30 @@ row_is_in_current_pg_res (GdaPostgresRecordset *model, gint row)
 }
 
 static void
-set_prow_with_pg_res (GdaPostgresRecordset *imodel, GdaPRow *prow, gint pg_res_rownum)
+set_prow_with_pg_res (GdaPostgresRecordset *imodel, GdaRow *prow, gint pg_res_rownum)
 {
 	gchar *thevalue;
 	gint col;
 
-	for (col = 0; col < ((GdaPModel*) imodel)->prep_stmt->ncols; col++) {
+	for (col = 0; col < ((GdaDataSelect*) imodel)->prep_stmt->ncols; col++) {
 		thevalue = PQgetvalue (imodel->priv->pg_res, pg_res_rownum, col);
 		if (thevalue && (*thevalue != '\0' ? FALSE : PQgetisnull (imodel->priv->pg_res, pg_res_rownum, col)))
-			gda_value_set_null (gda_prow_get_value (prow, col));
+			gda_value_set_null (gda_row_get_value (prow, col));
 		else
-			set_value (((GdaPModel*) imodel)->cnc,
-				   gda_prow_get_value (prow, col), 
-				   ((GdaPModel*) imodel)->prep_stmt->types [col], 
+			set_value (gda_data_select_get_connection ((GdaDataSelect*) imodel),
+				   gda_row_get_value (prow, col), 
+				   ((GdaDataSelect*) imodel)->prep_stmt->types [col], 
 				   thevalue, 
 				   PQgetlength (imodel->priv->pg_res, pg_res_rownum, col));
 	}
 }
 
-static GdaPRow *
+static GdaRow *
 new_row_from_pg_res (GdaPostgresRecordset *imodel, gint pg_res_rownum)
 {
-	GdaPRow *prow;
+	GdaRow *prow;
 
-	prow = gda_prow_new (((GdaPModel*) imodel)->prep_stmt->ncols);
+	prow = gda_row_new (((GdaDataSelect*) imodel)->prep_stmt->ncols);
 	set_prow_with_pg_res (imodel, prow, pg_res_rownum);
 	return prow;
 }
@@ -838,7 +763,7 @@ fetch_next_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
         status = PQresultStatus (model->priv->pg_res);
 	model->priv->chunks_read ++;
         if (status != PGRES_TUPLES_OK) {
-		_gda_postgres_make_error (((GdaPModel*) model)->cnc, 
+		_gda_postgres_make_error (gda_data_select_get_connection ((GdaDataSelect*) model), 
 					  model->priv->pconn, model->priv->pg_res, error);
                 PQclear (model->priv->pg_res);
                 model->priv->pg_res = NULL;
@@ -862,12 +787,12 @@ fetch_next_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
 			else
 				model->priv->pg_res_inf = model->priv->pg_pos + 1;
 
-			/* GDA_PMODEL (model)->advertized_nrows and model->priv->pg_pos */
+			/* GDA_DATA_SELECT (model)->advertized_nrows and model->priv->pg_pos */
 			if (nbtuples < model->priv->chunk_size) {
 				if (model->priv->pg_pos == G_MININT) 
-					GDA_PMODEL (model)->advertized_nrows = nbtuples;
+					GDA_DATA_SELECT (model)->advertized_nrows = nbtuples;
 				else
-					GDA_PMODEL (model)->advertized_nrows = model->priv->pg_pos + nbtuples + 1;
+					GDA_DATA_SELECT (model)->advertized_nrows = model->priv->pg_pos + nbtuples + 1;
 
 				model->priv->pg_pos = G_MAXINT;				
 			}
@@ -880,9 +805,9 @@ fetch_next_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
 		}
 		else {
 			if (model->priv->pg_pos == G_MININT)
-				GDA_PMODEL (model)->advertized_nrows = 0;
+				GDA_DATA_SELECT (model)->advertized_nrows = 0;
 			else
-				GDA_PMODEL (model)->advertized_nrows = model->priv->pg_pos + 1; /* total number of rows */
+				GDA_DATA_SELECT (model)->advertized_nrows = model->priv->pg_pos + 1; /* total number of rows */
 			model->priv->pg_pos = G_MAXINT;
 			retval = FALSE;
 		}
@@ -890,7 +815,7 @@ fetch_next_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
 
 #ifdef GDA_PG_DEBUG
 	g_print ("--> SIZE = %d (inf = %d) nrows = %d, pg_pos = %d\n", model->priv->pg_res_size, model->priv->pg_res_inf,
-		 GDA_PMODEL (model)->advertized_nrows, model->priv->pg_pos);
+		 GDA_DATA_SELECT (model)->advertized_nrows, model->priv->pg_pos);
 #endif
 
 	return retval;
@@ -908,7 +833,7 @@ fetch_prev_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
 	if (model->priv->pg_pos == G_MININT) 
 		return FALSE;
 	else if (model->priv->pg_pos == G_MAXINT) 
-		g_assert (GDA_PMODEL (model)->advertized_nrows >= 0); /* total number of rows MUST be known at this point */
+		g_assert (GDA_DATA_SELECT (model)->advertized_nrows >= 0); /* total number of rows MUST be known at this point */
 
 	gchar *str;
 	gboolean retval = TRUE;
@@ -930,7 +855,7 @@ fetch_prev_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
         status = PQresultStatus (model->priv->pg_res);
 	model->priv->chunks_read ++;
         if (status != PGRES_TUPLES_OK) {
-		_gda_postgres_make_error (((GdaPModel*) model)->cnc,
+		_gda_postgres_make_error (gda_data_select_get_connection ((GdaDataSelect*) model),
 					  model->priv->pconn, model->priv->pg_res, error);
                 PQclear (model->priv->pg_res);
                 model->priv->pg_res = NULL;
@@ -950,7 +875,7 @@ fetch_prev_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
                 if (nbtuples > 0) {
 			/* model->priv->pg_res_inf */
 			if (model->priv->pg_pos == G_MAXINT)
-				model->priv->pg_res_inf = GDA_PMODEL (model)->advertized_nrows - nbtuples;
+				model->priv->pg_res_inf = GDA_DATA_SELECT (model)->advertized_nrows - nbtuples;
 			else
 				model->priv->pg_res_inf = 
 					MAX (model->priv->pg_res_inf - (noffset - model->priv->chunk_size), 0);
@@ -961,7 +886,7 @@ fetch_prev_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
 			}
 			else {
 				if (model->priv->pg_pos == G_MAXINT)
-					model->priv->pg_pos = GDA_PMODEL (model)->advertized_nrows - 1;
+					model->priv->pg_pos = GDA_DATA_SELECT (model)->advertized_nrows - 1;
 				else
 					model->priv->pg_pos = MAX (model->priv->pg_pos - noffset, -1) + nbtuples;
 			}
@@ -974,7 +899,7 @@ fetch_prev_chunk (GdaPostgresRecordset *model, gboolean *fetch_error, GError **e
 
 #ifdef GDA_PG_DEBUG
 	g_print ("<-- SIZE = %d (inf = %d) nrows = %d, pg_pos = %d\n", model->priv->pg_res_size, model->priv->pg_res_inf,
-		 GDA_PMODEL (model)->advertized_nrows, model->priv->pg_pos);
+		 GDA_DATA_SELECT (model)->advertized_nrows, model->priv->pg_pos);
 #endif
 
 	return retval;
@@ -1004,7 +929,7 @@ fetch_row_number_chunk (GdaPostgresRecordset *model, int row_index, gboolean *fe
         status = PQresultStatus (model->priv->pg_res);
         model->priv->chunks_read ++; /* Not really correct, because we are only fetching 1 row, not a whole chunk of rows. */
         if (status != PGRES_TUPLES_OK) {
-		_gda_postgres_make_error (((GdaPModel*) model)->cnc, 
+		_gda_postgres_make_error (gda_data_select_get_connection ((GdaDataSelect*) model), 
 					  model->priv->pconn, model->priv->pg_res, error);
                 PQclear (model->priv->pg_res);
                 model->priv->pg_res = NULL;
