@@ -51,6 +51,7 @@ gda_capi_blob_op_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaCapiBlobOpClass),
 			(GBaseInitFunc) NULL,
@@ -62,7 +63,10 @@ gda_capi_blob_op_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_capi_blob_op_init
 		};
-		type = g_type_register_static (GDA_TYPE_BLOB_OP, "GdaCapiBlobOp", &info, 0);
+		g_static_mutex_lock (&registering);
+		if (type == 0)
+			type = g_type_register_static (GDA_TYPE_BLOB_OP, "GdaCapiBlobOp", &info, 0);
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -159,7 +163,7 @@ gda_capi_blob_op_read (GdaBlobOp *op, GdaBlob *blob, glong offset, glong size)
 	bin = (GdaBinary *) blob;
 	if (bin->data) 
 		g_free (bin->data);
-	bin->data = g_new0 (gchar, size);
+	bin->data = g_new0 (guchar, size);
 	bin->binary_length = 0;
 
 	/* fetch blob data using C API into bin->data, and set bin->binary_length */
