@@ -52,6 +52,9 @@ main (int argc, char **argv)
 {
 	int number_failed = 0;
 
+	/* set up test environment */
+        g_setenv ("GDA_TOP_BUILD_DIR", TOP_BUILD_DIR, 0);
+        g_setenv ("GDA_TOP_SRC_DIR", TOP_SRC_DIR, TRUE);
 	gda_init ();
 
 	prepend_null_row = FALSE;
@@ -154,7 +157,7 @@ do_test_read_direct_1 ()
 		return FALSE;
 	}
 
-	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "model", import, "sample_size", 0, NULL);
+	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "model", import, "sample-size", 0, NULL);
 	g_object_unref (import);
 	if (!proxy) {
 #ifdef CHECK_EXTRA_INFO
@@ -162,7 +165,7 @@ do_test_read_direct_1 ()
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	return do_test_common_read (proxy);
 }
 
@@ -197,7 +200,7 @@ do_test_read_direct_2 ()
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	return do_test_common_read (proxy);
 }
 
@@ -244,7 +247,7 @@ do_test_array ()
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	return do_test_common_write (proxy);
 }
 
@@ -291,19 +294,19 @@ do_test_prop_change (void)
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 0);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, NULL);
 
 	gboolean retval = FALSE;
 
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	/*
@@ -389,20 +392,20 @@ do_test_proxied_model_modif (void)
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 0);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, 
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
 		      "prepend-null-entry", prepend_null_row, NULL);
 
 	gboolean retval = FALSE;
 
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	
@@ -433,7 +436,7 @@ do_test_proxied_model_modif (void)
 	clean_expected_signals (proxy);
 	if (!check_data_model_n_rows (proxy, 159)) goto out;
 
-	values = make_values_list (0, G_TYPE_STRING, "BigCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "567890", 0);
+	values = make_values_list (0, G_TYPE_STRING, "BigCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "567890", (GType) 0);
 	declare_expected_signals ("U158", "Set values of new proxied row - 1");
 	if (!check_data_model_set_values (model, 158, values)) goto out;
 	free_values_list (values);
@@ -499,16 +502,16 @@ do_test_proxied_model_modif (void)
 	/*
 	 * change new proxied's rows values
 	 */
-	values = make_values_list (0, G_TYPE_STRING, "SmallCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "4907", 0);
+	values = make_values_list (0, G_TYPE_STRING, "SmallCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "4907", (GType) 0);
 	declare_expected_signals ("U1", "Set values of new proxied row - 2");
 	if (!check_data_model_set_values (model, 21, values)) goto out;
 	free_values_list (values);
 	clean_expected_signals (proxy);
 	if (!check_data_model_value (proxy, 1, 0, G_TYPE_STRING, "SmallCity2")) goto out;
-	if (!check_data_model_value (proxy, 1, 1, G_TYPE_STRING, NULL)) goto out;
+	if (!check_data_model_value (proxy, 1, 1, G_TYPE_STRING, "ARG")) goto out;
 	if (!check_data_model_value (proxy, 1, 2, G_TYPE_STRING, "4907")) goto out;
 	if (!check_data_model_value (proxy, 1, 3, G_TYPE_STRING, "SmallCity2")) goto out;
-	if (!check_data_model_value (proxy, 1, 4, G_TYPE_STRING, NULL)) goto out;
+	if (!check_data_model_value (proxy, 1, 4, G_TYPE_STRING, "ARG")) goto out;
 	if (!check_data_model_value (proxy, 1, 5, G_TYPE_STRING, "4907")) goto out;
 
 	/* 
@@ -524,7 +527,7 @@ do_test_proxied_model_modif (void)
 	/* 
 	 * delete a proxy row, in current proxy's chunk
 	 */
-	declare_expected_signals ("R5", "Delete a proxy row - 3");
+	declare_expected_signals ("R6", "Delete a proxy row - 3");
 	if (!check_data_model_remove_row (model, 25)) goto out;
 	clean_expected_signals (proxy);
 	gda_data_model_dump (proxy, stdout);
@@ -543,16 +546,16 @@ do_test_common_read (GdaDataModel *proxy)
 	gboolean retval = FALSE;
 
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 10);
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, 
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
 		      "prepend-null-entry", prepend_null_row, NULL);
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	/* 
@@ -665,16 +668,16 @@ do_test_common_write (GdaDataModel *proxy)
 	pmodel = gda_data_proxy_get_proxied_model (GDA_DATA_PROXY (proxy));
 
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 10);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, 
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
 		      "prepend-null-entry", prepend_null_row, NULL);
 
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	/*
@@ -728,7 +731,7 @@ do_test_common_write (GdaDataModel *proxy)
 	/*
 	 * set several values for the new row 
 	 */
-	values = make_values_list (0, G_TYPE_STRING, "MyCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "12345", 0);
+	values = make_values_list (0, G_TYPE_STRING, "MyCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "12345", (GType) 0);
 	declare_expected_signals ("U159", "Set values of new row - new row");
 	if (!check_data_model_set_values (proxy, 159, values)) goto out;
 	free_values_list (values);
@@ -741,7 +744,7 @@ do_test_common_write (GdaDataModel *proxy)
 	if (!check_data_model_value (proxy, 159, 5, G_TYPE_STRING, NULL)) goto out;
 	if (!check_data_model_n_rows (proxy, 160)) goto out;
 
-	values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", 0);
+	values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", (GType) 0);
 	declare_expected_signals ("U157", "Set values of new row - existing row");
 	if (!check_data_model_set_values (proxy, 157, values)) goto out;
 	free_values_list (values);
@@ -838,16 +841,16 @@ do_test_common_write (GdaDataModel *proxy)
 	 * Set values for those 3 new rows
 	 */
 	declare_expected_signals ("4U50", "Set values of 3 new rows");
-	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", (GType) 0);
 	if (!check_data_model_set_values (proxy, 50, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", (GType) 0);
 	if (!check_data_model_set_values (proxy, 51, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", (GType) 0);
 	if (!check_data_model_set_values (proxy, 52, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", (GType) 0);
 	if (!check_data_model_set_values (proxy, 53, values)) goto out;
 	free_values_list (values);
 	clean_expected_signals (proxy);
@@ -944,16 +947,16 @@ do_test_common_write (GdaDataModel *proxy)
 	 * Set values for those 4 new rows
 	 */
 	declare_expected_signals ("4U159", "Set values of 4 new rows");
-	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", (GType) 0);
 	if (!check_data_model_set_values (proxy, 159, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", (GType) 0);
 	if (!check_data_model_set_values (proxy, 160, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", (GType) 0);
 	if (!check_data_model_set_values (proxy, 161, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", (GType) 0);
 	if (!check_data_model_set_values (proxy, 162, values)) goto out;
 	free_values_list (values);
 	clean_expected_signals (proxy);
@@ -1045,7 +1048,7 @@ do_test_common_write (GdaDataModel *proxy)
 		/*
 		 * Try to alter first row
 		 */
-		values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", 0);
+		values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", (GType) 0);
 		if (gda_data_model_set_values (proxy, 0, values, NULL)) {
 #ifdef CHECK_EXTRA_INFO
 			g_print ("ERROR: Should not be able to alter row 0 (as it is the prepended row)\n");
@@ -1212,8 +1215,16 @@ clean_expected_signals (GdaDataModel *model)
 		}
 		ncols = gda_data_model_get_n_columns (model);
 		for (col = 0; col < ncols; col++) {
-			const GValue *value = gda_data_model_get_value_at (model, col, 0);
-			if (value) {
+			GError *lerror = NULL;
+			const GValue *value = gda_data_model_get_value_at (model, col, 0, &lerror);
+			if (!value) {
+#ifdef CHECK_EXTRA_INFO
+				g_print ("Can't get data model's value: %s\n",
+					 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+				exit (1);
+			}
+			else if (! gda_value_is_null (value)) {
 #ifdef CHECK_EXTRA_INFO
 				g_print ("ERROR: As the \"prepend-null-entry\" property is set, there should always be a first "
 					 "all-NULL entry, got '%s' at column %d\n", gda_value_stringify (value), col);
@@ -1322,6 +1333,7 @@ check_data_model_value (GdaDataModel *model, gint row, gint col, GType type, con
 	GValue *gvalue;
 	const GValue *mvalue;
 	gboolean retval;
+	GError *lerror = NULL;
 
 	row = ADJUST_ROW_FOR_MODEL (model, row);
 
@@ -1335,8 +1347,15 @@ check_data_model_value (GdaDataModel *model, gint row, gint col, GType type, con
 	else
 		gvalue = gda_value_new_null ();	
 
-	mvalue = gda_data_model_get_value_at (model, col, row); 
-	retval = !gda_value_compare_ext (gvalue, mvalue);
+	mvalue = gda_data_model_get_value_at (model, col, row, &lerror);
+	if (!mvalue) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("Can't get data model's value: %s\n",
+			 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+		return FALSE;
+	}
+	retval = !gda_value_compare (gvalue, mvalue);
 	gda_value_free (gvalue);
 
 #ifdef CHECK_EXTRA_INFO
@@ -1366,7 +1385,7 @@ check_data_model_n_rows (GdaDataModel *model, gint nrows)
 		return FALSE;
 	}
 
-	value = gda_data_model_get_value_at (model, 0, nrows);
+	value = gda_data_model_get_value_at (model, 0, nrows, NULL);
 	if (value) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("ERROR: Got a value at row %d, should not get any as it is out of bounds\n", nrows);
@@ -1443,8 +1462,16 @@ check_data_model_set_values (GdaDataModel *model, gint row, GList *values)
 	GList *list;
 	gint col;
 	for (col = 0, list = values; list; col++, list = list->next) {
-		const GValue *cvalue = gda_data_model_get_value_at (model, col, row);
-		if (gda_value_compare_ext (cvalue, (GValue *) list->data)) {
+		GError *lerror = NULL;
+		const GValue *cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
+		if (!cvalue) {
+#ifdef CHECK_EXTRA_INFO
+			g_print ("Can't get data model's value: %s\n",
+				 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+			return FALSE;
+		}
+		if (list->data && gda_value_compare (cvalue, (GValue *) list->data)) {
 #ifdef CHECK_EXTRA_INFO
 			g_print ("ERROR: Read value is not equal to set value: got '%s' and expected '%s'\n",
 				 gda_value_stringify (cvalue), gda_value_stringify ((GValue *) list->data));
