@@ -24,7 +24,7 @@ void
 common_declare_meta_store (GdaMetaStore *store)
 {
 	g_signal_connect (store, "meta-changed", G_CALLBACK (meta_changed_cb), NULL);
-	g_signal_connect (store, "suggest_update", G_CALLBACK (suggest_update_cb), NULL);
+	g_signal_connect (store, "suggest-update", G_CALLBACK (suggest_update_cb), NULL);
 }
 
 static void
@@ -107,9 +107,9 @@ suggest_update_cb (GdaMetaStore *store, GdaMetaContext *context, gpointer data)
 
 /*
  * Loading a CSV file
- * ... is a (-1 terminated) list of triplets composed of:
+ * ... is a (-1 terminated) list of pairs composed of:
  *   - a column number (gint)
- *   - the column (gchar *)
+ *   - the column type (gchar *)
  */
 GdaDataModel *
 common_load_csv_file (const gchar *data_file, ...)
@@ -135,7 +135,12 @@ common_load_csv_file (const gchar *data_file, ...)
 		
 		v = gda_value_new (G_TYPE_ULONG);
 		g_value_set_ulong (v, gda_g_type_from_string (va_arg (args, gchar*)));
-		gda_holder_take_value (holder, v);
+		if (! gda_holder_take_value (holder, v, NULL)) {
+			va_end (args);
+			g_object_unref (holder);
+			g_object_unref (options);
+			return NULL;
+		}
 		
 		gda_set_add_holder (options, holder);
 		g_object_unref (holder);
@@ -226,8 +231,14 @@ common_declare_expected_insertions_from_model (const gchar *table_name, GdaDataM
 		for (j = 0; j < ncols; j++) {
 			gchar *str;
 			const GValue *value;
+			GError *lerror = NULL;
 			str = g_strdup_printf ("+%d", j);
-			value = gda_data_model_get_value_at (model, j, i);
+			value = gda_data_model_get_value_at (model, j, i, &lerror);
+			if (!value) {
+				g_print ("Can't get data model's value: %s",
+					 lerror && lerror->message ? lerror->message : "No detail");
+				exit (1);
+			}
 			g_hash_table_insert (ac->keys, str, gda_value_copy (value));
 		}
 
@@ -511,15 +522,15 @@ test_routines (GdaMetaStore *store)
 	TEST_HEADER;
 
 	/* load CSV file */
-	import = common_load_csv_file ("data_routines.csv", 8, "boolean", 14, "boolean", 16, "boolean", -1);
+	import = common_load_csv_file ("data_routines.csv", 8, "boolean", 9, "gint", 15, "boolean", 17, "boolean", -1);
 	common_declare_expected_insertions_from_model (TNAME, import);
 	TEST_MODIFY (store, TNAME, import, NULL, &error, NULL);
 	TEST_END (import);
 
 	/* remove some lines */
-	DECL_CHANGE (TNAME, GDA_META_STORE_REMOVE, "-0", "meta", "-1", "information_schema", "-2", "_pg_numeric_precision_radix_10632", NULL);
+	DECL_CHANGE (TNAME, GDA_META_STORE_REMOVE, "-0", "meta", "-1", "information_schema", "-2", "_pg_numeric_precision_radix_11324", NULL);
 	TEST_MODIFY (store, TNAME, NULL, 
-		     "specific_name='_pg_numeric_precision_radix_10632'", &error, NULL);
+		     "specific_name='_pg_numeric_precision_radix_11324'", &error, NULL);
 	TEST_END (NULL);
 #undef TNAME
 }
