@@ -30,6 +30,7 @@ static gboolean test6 (GError **error);
 static gboolean test7 (GError **error);
 static gboolean test8 (GError **error);
 static gboolean test9 (GError **error);
+static gboolean test10 (GError **error);
 
 TestFunc tests[] = {
 	test1,
@@ -40,7 +41,8 @@ TestFunc tests[] = {
 	test6,
 	test7,
 	test8,
-	test9
+	test9,
+	test10
 };
 
 int 
@@ -92,18 +94,15 @@ test1 (GError **error)
 	/***/
 	value = gda_value_new_from_string ("my string", G_TYPE_STRING);
 	emitted_signals_reset ();
-	if (!gda_holder_set_value (h, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set GdaHolder's value");
+	if (!gda_holder_set_value (h, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h, "changed", error))
 		return FALSE;
 	if (!emitted_signals_chech_empty (NULL, "changed", error))
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is incorrect");
 		return FALSE;
@@ -114,18 +113,15 @@ test1 (GError **error)
 	value = gda_value_new_from_string ("my other string", G_TYPE_STRING);
 	copy = gda_value_copy (value);
 	emitted_signals_reset ();
-	if (!gda_holder_take_value (h, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set GdaHolder's value");
+	if (!gda_holder_take_value (h, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h, "changed", error))
 		return FALSE;
 	if (!emitted_signals_chech_empty (NULL, "changed", error))
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (copy, cvalue)) {
+	if (!cvalue || gda_value_compare (copy, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is incorrect");
 		return FALSE;
@@ -149,7 +145,7 @@ test2 (GError **error)
 	h = gda_holder_new_boolean ("ABOOL", TRUE);
 	value = gda_value_new_from_string ("TRUE", G_TYPE_BOOLEAN);
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is incorrect");
 		return FALSE;
@@ -166,7 +162,7 @@ test2 (GError **error)
 	h = gda_holder_new_string ("Astring", "A string value");
 	value = gda_value_new_from_string ("A string value", G_TYPE_STRING);
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is incorrect");
 		return FALSE;
@@ -182,7 +178,7 @@ test2 (GError **error)
 	h = gda_holder_new_int ("AnInt",  15);
 	value = gda_value_new_from_string ("15", G_TYPE_INT);
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is incorrect");
 		return FALSE;
@@ -234,7 +230,8 @@ test3 (GError **error)
 	/***/
 	emitted_signals_reset ();
 	value = gda_value_new_from_string ("FALSE", G_TYPE_BOOLEAN);
-	gda_holder_take_value (h, value);
+	if (! gda_holder_take_value (h, value, error))
+		return FALSE;
 	if (!gda_holder_is_valid (h)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder is invalid");
@@ -287,7 +284,7 @@ test4 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_default_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's default value is invalid");
 		return FALSE;
@@ -313,29 +310,24 @@ test4 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	value = gda_value_new_null ();
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (cvalue) {
 		tests_common_display_value ("cvalue", cvalue);
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is invalid");
 		return FALSE;
 	}
-	gda_value_free (value);
 
 	/** set value to "hey" */
 	value = gda_value_new_from_string ("hey", G_TYPE_STRING);
-	if (!gda_holder_set_value (h, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set GdaHolder's value");
+	if (!gda_holder_set_value (h, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h, "changed", error))
 		return FALSE;
 	if (!emitted_signals_chech_empty (NULL, "changed", error))
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		tests_common_display_value ("cvalue", cvalue);
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is invalid");
@@ -350,7 +342,7 @@ test4 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_default_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's default value is invalid");
 		return FALSE;
@@ -397,7 +389,7 @@ test5 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_default_value (h);
-	if (gda_value_compare_ext (defvalue, cvalue)) {
+	if (!cvalue || gda_value_compare (defvalue, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's default value is invalid");
 		return FALSE;
@@ -422,7 +414,7 @@ test5 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (defvalue, cvalue)) {
+	if (!cvalue || gda_value_compare (defvalue, cvalue)) {
 		tests_common_display_value ("cvalue", cvalue);
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is invalid");
@@ -431,18 +423,15 @@ test5 (GError **error)
 
 	/** set value to "hey" */
 	value = gda_value_new_from_string ("hey", G_TYPE_STRING);
-	if (!gda_holder_set_value (h, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set GdaHolder's value");
+	if (!gda_holder_set_value (h, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h, "changed", error))
 		return FALSE;
 	if (!emitted_signals_chech_empty (NULL, "changed", error))
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		tests_common_display_value ("cvalue", cvalue);
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is invalid");
@@ -459,18 +448,15 @@ test5 (GError **error)
 
 	/** set value to "ABC" */
 	value = gda_value_new_from_string ("ABC", G_TYPE_STRING);
-	if (!gda_holder_set_value (h, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set GdaHolder's value");
+	if (!gda_holder_set_value (h, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h, "changed", error))
 		return FALSE;
 	if (!emitted_signals_chech_empty (NULL, "changed", error))
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		tests_common_display_value ("cvalue", cvalue);
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is invalid");
@@ -518,7 +504,7 @@ test6 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_default_value (h);
-	if (gda_value_compare_ext (defvalue, cvalue)) {
+	if (!cvalue || gda_value_compare (defvalue, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "GdaHolder's default value is invalid");
 		return FALSE;
@@ -552,13 +538,10 @@ test6 (GError **error)
 
 	/** set value to "hey" */
 	value = gda_value_new_from_string ("hey", G_TYPE_STRING);
-	if (!gda_holder_set_value (h, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set GdaHolder's value");
+	if (!gda_holder_set_value (h, value, error))
 		return FALSE;
-	}
 	cvalue = gda_holder_get_value (h);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		tests_common_display_value ("cvalue", cvalue);
 		g_set_error (error, 0, 0,
 			     "GdaHolder's value is invalid");
@@ -595,7 +578,8 @@ test7 (GError **error)
 	emitted_signals_monitor_holder (h2);
 	emitted_signals_reset ();
 
-	gda_holder_set_bind (h1, h2);
+	if (! gda_holder_set_bind (h1, h2, error))
+		return FALSE;
 	if (!emitted_signals_find (h1, "changed", error))
 		return FALSE;
 	if (!emitted_signals_notfind (h2, "changed", error))
@@ -605,7 +589,7 @@ test7 (GError **error)
 
 	value = gda_value_new_from_string ("Master string", G_TYPE_STRING);
 	cvalue = gda_holder_get_value (h1);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		tests_common_display_value ("Slave", cvalue);
 		g_set_error (error, 0, 0,
 			     "Slave GdaHolder's value is wrong");
@@ -615,11 +599,8 @@ test7 (GError **error)
 	
 	/***/
 	value = gda_value_new_from_string ("A string", G_TYPE_STRING);
-	if (!gda_holder_set_value (h1, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set Slave's value");
+	if (!gda_holder_set_value (h1, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h1, "changed", error))
 		return FALSE;
 	if (!emitted_signals_notfind (h2, "changed", error))
@@ -628,7 +609,7 @@ test7 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h1);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		tests_common_display_value ("Slave", cvalue);
 		g_set_error (error, 0, 0,
 			     "Slave GdaHolder's value is wrong");
@@ -638,11 +619,8 @@ test7 (GError **error)
 
 	/***/
 	value = gda_value_new_from_string ("A string", G_TYPE_STRING);
-	if (!gda_holder_set_value (h2, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set Master's value");
+	if (!gda_holder_set_value (h2, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h1, "changed", error))
 		return FALSE;
 	if (!emitted_signals_find (h2, "changed", error))
@@ -651,25 +629,26 @@ test7 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h2);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "Master GdaHolder's value is wrong");
 		return FALSE;
 	}
 	cvalue = gda_holder_get_value (h1);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "Slave GdaHolder's value is wrong");
 		return FALSE;
 	}
 
 	/***/
-	gda_holder_set_bind (h1, NULL);
+	if (! gda_holder_set_bind (h1, NULL, error))
+		return FALSE;
 	if (!emitted_signals_chech_empty (NULL, "changed", error))
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h1);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "Slave GdaHolder's value is wrong");
 		return FALSE;
@@ -677,11 +656,8 @@ test7 (GError **error)
 	gda_value_free (value);
 
 	value = gda_value_new_from_string ("Another string", G_TYPE_STRING);
-	if (!gda_holder_set_value (h1, value)) {
-		g_set_error (error, 0, 0,
-			     "Could not set Master's value");
+	if (!gda_holder_set_value (h1, value, error))
 		return FALSE;
-	}
 	if (!emitted_signals_find (h1, "changed", error))
 		return FALSE;
 	if (!emitted_signals_notfind (h2, "changed", error))
@@ -690,14 +666,14 @@ test7 (GError **error)
 		return FALSE;
 
 	cvalue = gda_holder_get_value (h1);
-	if (gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "Slave GdaHolder's value is wrong");
 		return FALSE;
 	}
 
 	cvalue = gda_holder_get_value (h2);
-	if (!gda_value_compare_ext (value, cvalue)) {
+	if (!cvalue || !gda_value_compare (value, cvalue)) {
 		g_set_error (error, 0, 0,
 			     "Master GdaHolder's value is wrong");
 		return FALSE;
@@ -763,9 +739,9 @@ test9 (GError **error)
 	
 	if (!gda_holder_set_source_model (h, model, 1, error)) 
 		return FALSE;
-	if (!emitted_signals_find (h, "source_changed", error))
+	if (!emitted_signals_find (h, "source-changed", error))
 		return FALSE;
-	if (!emitted_signals_chech_empty (NULL, "source_changed", error))
+	if (!emitted_signals_chech_empty (NULL, "source-changed", error))
 		return FALSE;
 
 	if ((gda_holder_get_source_model (h, &col) != model) || (col != 1)) {
@@ -777,9 +753,9 @@ test9 (GError **error)
 	/***/
 	if (!gda_holder_set_source_model (h, model, 0, error))
 		return FALSE;
-	if (!emitted_signals_find (h, "source_changed", error))
+	if (!emitted_signals_find (h, "source-changed", error))
 		return FALSE;
-	if (!emitted_signals_chech_empty (NULL, "source_changed", error))
+	if (!emitted_signals_chech_empty (NULL, "source-changed", error))
 		return FALSE;
 	if ((gda_holder_get_source_model (h, &col) != model) || (col != 0)) {
 		g_set_error (error, 0, 0,
@@ -790,9 +766,9 @@ test9 (GError **error)
 	/***/
 	if (!gda_holder_set_source_model (h, NULL, 0, error))
 		return FALSE;
-	if (!emitted_signals_find (h, "source_changed", error))
+	if (!emitted_signals_find (h, "source-changed", error))
 		return FALSE;
-	if (!emitted_signals_chech_empty (NULL, "source_changed", error))
+	if (!emitted_signals_chech_empty (NULL, "source-changed", error))
 		return FALSE;
 
 	if (gda_holder_get_source_model (h, NULL)) {
@@ -807,6 +783,101 @@ test9 (GError **error)
 	return TRUE;
 }
 
+/*
+ * "validate-change" signal
+ */
+
+static GError *
+t10_validate_change_cb (GdaHolder *h, const GValue *value, gchar *token)
+{
+	/* only accept GDA_VALUE_NULL or the "hi!" G_TYPE_STRING value */
+	g_assert (!strcmp (token, "AToken"));
+
+	if (gda_value_is_null (value) || !strcmp (g_value_get_string (value), "hi!")) {
+		g_print ("GdaHolder change accepted\n");
+		return NULL;
+	}
+	else {
+		GError *error = NULL;
+		g_print ("GdaHolder change refused\n");
+		g_set_error (&error, 0, 0,
+			     "GdaHolder change refused!");
+		return error;
+	}
+}
+
+static gboolean
+test10 (GError **error)
+{
+	GdaHolder *h;
+	const GValue *cvalue;
+	GValue *value;
+
+	h = gda_holder_new (G_TYPE_STRING);
+	emitted_signals_monitor_holder (h);
+
+	/***/
+	value = gda_value_new_from_string ("my string", G_TYPE_STRING);
+	emitted_signals_reset ();
+	if (!gda_holder_set_value (h, value, error))
+		return FALSE;
+	if (!emitted_signals_find (h, "changed", error))
+		return FALSE;
+	if (!emitted_signals_chech_empty (NULL, "changed", error))
+		return FALSE;
+	gda_value_free (value);
+	
+	/***/
+	g_signal_connect (G_OBJECT (h), "validate-change",
+			  G_CALLBACK (t10_validate_change_cb), "AToken");
+
+	/***/
+	value = gda_value_new_from_string ("my other string", G_TYPE_STRING);
+	emitted_signals_reset ();
+	if (gda_holder_set_value (h, value, error)) {
+		g_set_error (error, 0, 0,
+			     "GdaHolder's change should have failed");
+		return FALSE;
+	}
+	if (!emitted_signals_chech_empty (NULL, "changed", error))
+		return FALSE;
+	gda_value_free (value);
+
+	/***/
+	value = gda_value_new_from_string ("my string", G_TYPE_STRING);
+	cvalue = gda_holder_get_value (h);
+	if (!cvalue || gda_value_differ (value, cvalue)) {
+		g_set_error (error, 0, 0,
+			     "GdaHolder's value is incorrect");
+		return FALSE;
+	}
+	gda_value_free (value);
+
+	
+	/***/
+	value = gda_value_new_from_string ("hi!", G_TYPE_STRING);
+	emitted_signals_reset ();
+	if (!gda_holder_set_value (h, value, error)) 
+		return FALSE;
+	if (!emitted_signals_find (h, "changed", error))
+		return FALSE;
+	if (!emitted_signals_chech_empty (NULL, "changed", error))
+		return FALSE;
+
+	/***/
+	cvalue = gda_holder_get_value (h);
+	if (!cvalue || gda_value_differ (value, cvalue)) {
+		g_set_error (error, 0, 0,
+			     "GdaHolder's value is incorrect");
+		return FALSE;
+	}
+	gda_value_free (value);
+
+	g_object_unref (h);
+
+	return TRUE;
+}
+
 
 /*
  * Signals testing
@@ -893,7 +964,7 @@ source_changed_cb (GObject *obj)
 	EmittedSignal *es;
 	es = g_new0 (EmittedSignal, 1);
 	es->obj = obj;
-	es->signal_name = "source_changed";
+	es->signal_name = "source-changed";
 	emitted_signal_add (es);
 }
 
@@ -902,6 +973,6 @@ emitted_signals_monitor_holder (GdaHolder *h)
 {
 	g_signal_connect (G_OBJECT (h), "changed",
 			  G_CALLBACK (changed_cb), NULL);
-	g_signal_connect (G_OBJECT (h), "source_changed",
+	g_signal_connect (G_OBJECT (h), "source-changed",
 			  G_CALLBACK (source_changed_cb), NULL);
 }
