@@ -782,11 +782,12 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 		GdaMetaTable *mt = GDA_META_TABLE (dbo);
 		GSList *list;
 
-		model = gda_data_model_array_new (4);
+		model = gda_data_model_array_new (5);
 		gda_data_model_set_column_title (model, 0, _("Column"));
 		gda_data_model_set_column_title (model, 1, _("Type"));
 		gda_data_model_set_column_title (model, 2, _("Nullable"));
 		gda_data_model_set_column_title (model, 3, _("Default"));
+		gda_data_model_set_column_title (model, 4, _("Extra"));
 		if (dbo->obj_type == GDA_META_DB_VIEW)
 			g_object_set_data_full (G_OBJECT (model), "name", 
 						g_strdup_printf (_("List of columns for view '%s'"), 
@@ -808,6 +809,35 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			values = g_list_append (values, val);
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->default_value);
 			values = g_list_append (values, val);
+			if (tcol->extra) {
+				GString *string;
+				gint i;
+				gboolean first = TRUE;
+				string = g_string_new ("");
+				for (i = 0; i < tcol->extra->len; i++) {
+					const gchar *tmp = g_array_index (tcol->extra, gchar *, i);
+					const gchar *astmp = NULL;
+					if (!strcmp (tmp, GDA_EXTRA_AUTO_INCREMENT)) 
+						astmp = _("Auto increment");
+					else {
+						g_warning ("Unknown extra keyword '%s'", tmp);
+						TO_IMPLEMENT;
+					}
+					if (astmp) {
+						if (first)
+							first = FALSE;
+						else
+							g_string_append (string, ", ");
+						g_string_append (string, astmp);
+					}
+				}
+				g_value_take_string ((val = gda_value_new (G_TYPE_STRING)), string->str);
+				g_string_free (string, FALSE);
+			}
+			else
+				val = gda_value_new_null ();
+			values = g_list_append (values, val);
+
 			gda_data_model_append_values (model, values, NULL);
 			g_list_foreach (values, (GFunc) gda_value_free, NULL);
 			g_list_free (values);
@@ -846,14 +876,24 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			for (i = 0; i < nrows; i++) {
 				GString *string = NULL;
 				const GValue *cvalue;
-				const gchar *str;
+				const gchar *str = NULL;
 
-				cvalue = gda_data_model_get_value_at (model, 0, i);
+				cvalue = gda_data_model_get_value_at (model, 0, i, error);
+				if (!cvalue) {
+					gda_internal_command_exec_result_free (res);
+					res = NULL;
+					goto out;
+				}
 				str = g_value_get_string (cvalue);
 				if (*str == 'P') {
 					/* primary key */
 					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i);
+					cvalue = gda_data_model_get_value_at (model, 1, i, error);
+					if (!cvalue) {
+						gda_internal_command_exec_result_free (res);
+						res = NULL;
+						goto out;
+					}
 					string = g_string_new (_("Primary key"));
 					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
 					str = "SELECT column_name, ordinal_position "
@@ -873,8 +913,15 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 0, j)));
+								cvalue = gda_data_model_get_value_at (cols, 0, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append_c (string, ')');
 						}
@@ -884,7 +931,12 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 				else if (*str == 'F') {
 					/* foreign key */
 					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i);
+					cvalue = gda_data_model_get_value_at (model, 1, i, error);
+					if (!cvalue) {
+						gda_internal_command_exec_result_free (res);
+						res = NULL;
+						goto out;
+					}
 					string = g_string_new (_("Foreign key"));
 					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
 
@@ -905,23 +957,51 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 2, j)));
+								cvalue = gda_data_model_get_value_at (cols, 2, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append (string, ") references ");
 
 							const GValue *v1, *v2;
 							
-							v1 = gda_data_model_get_value_at (cols, 0, 0);
-							v2 = gda_data_model_get_value_at (cols, 1, 0);
+							v1 = gda_data_model_get_value_at (cols, 0, 0, error);
+							if (!v1) {
+								gda_internal_command_exec_result_free (res);
+								res = NULL;
+								g_object_unref (cols);
+								g_string_free (string, TRUE);
+								goto out;
+							}
+							v2 = gda_data_model_get_value_at (cols, 1, 0, error);
+							if (!v2) {
+								gda_internal_command_exec_result_free (res);
+								res = NULL;
+								g_object_unref (cols);
+								g_string_free (string, TRUE);
+								goto out;
+							}
 							g_string_append_printf (string, " %s.%s (", 
 										g_value_get_string (v1),
 										g_value_get_string (v2));
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 2, j)));
+								cvalue = gda_data_model_get_value_at (cols, 2, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append_c (string, ')');
 						}
@@ -931,7 +1011,12 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 				else if (*str == 'U') {
 					/* Unique constraint */
 					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i);
+					cvalue = gda_data_model_get_value_at (model, 1, i, error);
+					if (!cvalue) {
+						gda_internal_command_exec_result_free (res);
+						res = NULL;
+						goto out;
+					}
 					string = g_string_new (_("Unique"));
 					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
 					str = "SELECT column_name, ordinal_position "
@@ -951,8 +1036,15 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								g_string_append (string, 
-										 g_value_get_string (gda_data_model_get_value_at (cols, 0, j)));
+								cvalue = gda_data_model_get_value_at (cols, 0, j, error);
+								if (!cvalue) {
+									gda_internal_command_exec_result_free (res);
+									res = NULL;
+									g_object_unref (cols);
+									g_string_free (string, TRUE);
+									goto out;
+								}
+								g_string_append (string, g_value_get_string (cvalue));
 							}
 							g_string_append_c (string, ')');
 						}
@@ -979,6 +1071,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 	else 
 		TO_IMPLEMENT;
 
+ out:
 	g_object_unref (mstruct);
 	return res;
 }
